# /tcl/km-00-defs.tcl
#
# $Id: km-00-defs-procs.tcl,v 1.2 2009/02/13 20:40:39 jeffd Exp $

util_report_library_entry
proc km_file_path {} { return [ad_parameter FilePath library "[acs_root_dir]/library-files"] }

ad_proc util_GetUserAgentHeader {} {
 get users browser etc
} {
    set header [ns_conn headers]
    set userag [ns_set iget $header "USER-AGENT"]
    return $userag
}

ad_proc km_get_community_name {} {
 returns the name of the current community
} {
    array set node [site_node_closest_ancestor_site_node "acs-subsite"] 
    return $node(instance_name) 
}

ad_proc msie_p {} {
  check if browser is microsoft or other
} {
    return [regexp -nocase {msie} [util_GetUserAgentHeader]]
}

ad_proc -public km_conn {args} {

    km_conn works much like ad_conn. It caches http-session specific
    information.

    Example:
    set admin_p [km_conn admin_p]

} {
  global km_conn

  set flag [lindex $args 0]
  if {[string index $flag 0] != "-"} {
    set var $flag
    set flag "-get"
  } else {
    set var [lindex $args 1]
  }

  switch -- $flag {
    -set {
      set km_conn($var) [lindex $args 2]
    }

    -unset {
      unset km_conn($var)
    }

    -reset {
      if {[info exists km_conn]} {
	unset km_conn
      }
      array set km_conn {
          admin_p ""
	  publish_p ""
          read_p ""
          write_p ""
          delete_p ""
	  create_p ""
          public_p ""
	  archived_p ""
          deleted_p ""
          expiration_date ""
          object_type_id ""
          object_name ""
          overview ""
          last_modified ""
          creation_date ""
          user_checkoff_date ""
          original_author_id ""
          original_author_name ""
	  access_total ""
	  access_month ""
	  public_until ""
	  start_date ""
	  end_date ""
	  publisher_id ""
          in_review_p ""
	  review_state ""
      }
    }

    -get {
      if { [info exists km_conn($var)] } {
	return $km_conn($var)
      } else {
	return ""
      }
    }

    default {
      error "km_conn: unknown flag $flag"
    }
  }
}



ad_proc -public km_static {args} {

    km_static caches information about objects whose value is more or
    less static throughout the system's life time. The first time a
    particular information is being looked for, this information is being
    loaded from the database into the cache. km_static does _not_ do any
    error checking at the moment. It is also not the smartest caching
    system under the sun (not yet at least). 

    - There should be proc that loads the cache on server
      startup.

    Example:
    set pretty_type_plural [km_static object_type_pretty_plural $object_type_id]

} {
#    global km_static

    set flag [lindex $args 0]
    if {[string index $flag 0] != "-"} {
        set var $flag
        set flag "-get"
        set array_index [lindex $args 1]
        set value [lindex $args 2]
    } else {
        set var [lindex $args 1]
        set array_index [lindex $args 2]
        set value [lindex $args 3]
    }
   
    if {[empty_string_p $array_index]} {
        set var_index "${var}"
    } else {
        set var_index "${var}${array_index}"
    }

    switch -- $flag {
        -set {
            nsv_set km_static ${var_index} $value
        }
        
        -reset {
	    server_cluster_httpget_from_peers "/SYSTEM/flush-km-static-cache.tcl?id=$var"
            km_static_reset $var
        }
        
        -get {
            if { [nsv_exists km_static $var_index] } {
		set retval [nsv_get km_static $var_index]
                return $retval
            } else {
                switch -regexp $var {
		    {package_object_types} {
			set package_object_types_list [list]
			db_foreach km_static_10 {select pretty_name, object_type_id 
			from sn_object_types
			where context_id=:array_index} {
			    lappend package_object_types_list [list $pretty_name $object_type_id]
			}
			km_static -set package_object_types $array_index $package_object_types_list
			return $package_object_types_list
		    }
                    {^object_type} {
                        db_1row object_type_info {
			    select pretty_name, pretty_plural,
				graphic, graphic_p,
				decode(deleted_p,'t',1,0) as deleted_p,
				decode(browse_p,'t',1,0) as browse_p, 
				decode(public_p,'t',1,0) as public_p, 
				decode(create_p,'t',1,0) as create_p, 
				decode(archive_p,'t',1,0) as archive_p, 
				decode(copy_p,'t',1,0) as copy_p, 
				default_age_filter, start_date, end_date,
                                long_description, 
                                public_until, linked_question_id, short_name
			    from sn_object_types
                            where object_type_id=:array_index}
                        set short_description [db_list get_short_description_list {
                            select short_description
                            from sn_types_map_short_name
                            where object_type_id=:array_index
                            order by position}]
                        km_static -set object_type_pretty_name $array_index $pretty_name
                        km_static -set object_type_short_name $array_index $short_name
                        km_static -set object_type_pretty_plural $array_index $pretty_plural
                        km_static -set object_type_graphic $array_index $graphic
                        km_static -set object_type_graphic_p $array_index $graphic_p
                        km_static -set object_type_deleted_p $array_index $deleted_p
                        km_static -set object_type_browse_p $array_index $browse_p
                        km_static -set object_type_public_p $array_index $public_p
                        km_static -set object_type_create_p $array_index $create_p
                        km_static -set object_type_archive_p $array_index $archive_p
                        km_static -set object_type_copy_p $array_index $copy_p
                        km_static -set object_type_default_age_filter $array_index $default_age_filter
                        km_static -set object_type_short_description $array_index $short_description
                        km_static -set object_type_long_description $array_index $long_description
                        km_static -set object_type_public_until $array_index $public_until
                        km_static -set object_type_start_date $array_index $start_date
                        km_static -set object_type_end_date $array_index $end_date
                        km_static -set object_type_linked_question_id $array_index $linked_question_id
			regsub {object_type_} $var "" var
			return [set $var]
                    }
                    {^question_} {
                        db_1row km_static_30 {
                            select pretty_name, abstract_data_type, order_by, target_object_type_id,
			           presentation_type, defaults_question_id, max_categories
                            from sn_questions
                            where question_id=:array_index}
                        km_static -set question_pretty_name $array_index $pretty_name
                        km_static -set question_abstract_data_type $array_index $abstract_data_type
                        km_static -set question_order_by $array_index $order_by
			km_static -set question_target_object_type_id $array_index $target_object_type_id
			km_static -set question_presentation_type $array_index $presentation_type
			km_static -set question_defaults_question_id $array_index $defaults_question_id
			km_static -set question_max_categories $array_index $max_categories
			return [km_static ${var} ${array_index}]
                    }
                    {km_dummy_object_id} {
                        set new_km_dummy_object_id [db_string km_static_40 {
			    select object_id
			    from acs_magic_objects
			    where name='km_dummy_object'}]
                        km_static -set km_dummy_object_id $new_km_dummy_object_id
                        return $new_km_dummy_object_id
                    }
                    {^approval_} {
			set coordinators [db_list km_static_10 {
			    select coordinator_id
			    from approval_coordinators 
			    where package_id=:array_index
			}]
			km_static -set approval_coordinators $array_index $coordinators
                        if {[empty_string_p $coordinators]} {
                            km_static -set approval_p $array_index 0
                        } else {
                            km_static -set approval_p $array_index 1
                        }
			return [km_static ${var} ${array_index}]
                    }
                    default {
                        error "km_static: unknown flag $flag"
                    }
                }
            }
        }
        
    default {
        error "km_static: unknown flag $flag"
    }
}
}

ad_proc -public km_static_reset { id } {
    Actually flush the km_static cache.
} {
    set all_the_cached_crud [nsv_array names km_static]
    foreach cached_crud_item $all_the_cached_crud {
	if {[regexp "$id\$" $cached_crud_item match matched_id]} {
	    nsv_unset km_static $cached_crud_item
	}
    }
}

ad_proc approval_coordinator_p {user_id} {

    checks whether the user with user_id is approval coordinator
} {
    set package_id [ad_conn package_id]
    if {[lsearch [km_static approval_coordinators $package_id] $user_id] > -1} {
	return 1
    } else {
	return 0
    }
}

ad_proc km_question_attributes {question_id_list} {

    Returns the question_id, abstract_data_type, pretty_name as triples 
    for each question_id in the list.

} {
    set integer_list [filter_integers $question_id_list]
    if {![null_p $integer_list]} {


        if {[llength $integer_list] > 1} {
	    set where_clauses "where question_id in ([join $integer_list ","])"
        } else {
            set question_id [lindex $integer_list 0]
	    set where_clauses "where question_id = :question_id"
        }


	set sql "
	    select question_id, pretty_name, abstract_data_type
	    from sn_questions
	    $where_clauses
            "
	set result [db_list_of_lists km_question_attributes $sql]
	return $result
    }
}

# ********** Metadata **********

ad_proc km_get_all_object_types {
    {
	-browsable_only_p 1
	-createable_only_p 0
    }  
} {
    Returns a list of object_type_id, pretty_name, pretty_plural, browse_p, graphic, sort_key
} {
    set user_id [ad_conn user_id]
    set package_id [ad_conn package_id]

    if {$browsable_only_p} {
	set browse_not "and browse_p='t'"
    } else {
	set browse_not ""
    }

    if {$createable_only_p} {
	set privilege "create"
    } else {
	set privilege "read"
    }

    set result [db_list_of_lists object_types "
	select object_type_id, pretty_name, pretty_plural, 
	    decode(browse_p,'t',1,0), graphic, sort_key
	from sn_object_types
	where deleted_p = 'f'
	    and context_id = :package_id
	    $browse_not
	    and acs_permission.permission_p(object_type_id, :user_id, :privilege) = 't'
	order by sort_key
    "]

    return $result
}

ad_proc km_get_questions {
    {
	-all_properties_p 0 
	-form_view ""
	-root_node_p 1
	-branch_children_p 1
	-question_ids {}
	-object_id 0
	-object_type_id 0
	-question_states {active}
        -answers_p 0
        -answered_p 0
	-mandatory_p 0
    } 
} {

    Returns the column_name, pretty_name, abstract_data_type of all
    questions for this object type or for this object as a *keyed* list. 
    root_node_p means it returns only top level questions for
    branches. branch_children_p true means that branch children will ALSO be
    listed individually

} {
    set columns {q.question_id q.pretty_name q.abstract_data_type}
    set order_by ""
    set tables {"sn_questions q" "sn_question_object_type_map m"}

    if ![null_p $question_ids] {
	# Return information about the given question.

	lappend where_clauses "q.question_id=m.question_id" 

        if {[llength $question_ids] > 1} {
	    lappend where_clauses "q.question_id in ([join $question_ids ","])"
        } else {
            set question_id [lindex $question_ids 0]
	    lappend where_clauses "q.question_id = :question_id"
        }
        set order_by "order by m.sort_key"
    } else {
	if { $object_id } { set object_type_id [km_get_object_type $object_id] }

	lappend join_tables "sn_questions q" "sn_question_object_type_map m"
	lappend where_clauses "(m.object_type_id=$object_type_id and q.question_id=m.question_id)"
	set order_by "order by m.sort_key"
    } 

    if { $form_view == 1 } {
        lappend where_clauses "form_number=1"
    } elseif { $form_view == 2 } {
	lappend where_clauses "q.abstract_data_type in ('text', 'option', 'category', 'other_category', 'date', 'file', 'integer', 'object_link', 'content_link', 'user_link', 'child_object', 'nephew_object')"
    } 

    if {$answers_p} {
        lappend columns "content as answer" "decode(nvl(html_p,'f'),'t',1,0) as html_p"
	lappend where_clauses "c.object_id=$object_id and c.question_id=m.question_id"
	lappend tables "sn_content c"
    }

    # we only want to know here, _if_ there is an answer. If html_p is null,
    # then the outer join has retrieved a null row for sn_content, so there is
    # no answer. Otherwise there is an answer.
    if {$answered_p} {
        lappend columns "nvl2(c.object_id,1,0) as answered_p"
	lappend where_clauses "c.object_id(+)=$object_id and c.question_id(+)=m.question_id"
	lappend tables "sn_content c"
    }

    if { $all_properties_p } {
	lappend columns presentation_type tag_height tag_width entry_explanation year_from year_to \
	    m.mandatory_p m.question_state m.form_number m.sort_key q.node_id q.defaults_question_id \
	    q.order_by q.default_value q.target_object_type_id q.references_question_id m.object_type_id \
	    "decode(m.branch_p,'t',1,0) as branch_p" q.max_categories browse_p \
	    "(select decode(count(*),0,0,1) from dual
	      where exists (select 1
	      from sn_question_object_type_map m2
	      where m2.parent_question_id = q.question_id 
	          and m2.branch_p='t'
		  and m2.object_type_id=$object_type_id)) as root_branch_p" \
	    parent_question_id
    } 
    
    # Both branch and composite questions can have have parent and child questions
    if { $root_node_p && !$branch_children_p } {
	
	# We want to include questions at the root of branches in the selection but no children of branches
	lappend where_clauses " m.parent_question_id=[km_static km_dummy_object_id]"
	
    } elseif { $root_node_p } {
	
	# We want branch children, too, but no composite children
	lappend where_clauses "  ( m.parent_question_id=[km_static km_dummy_object_id] or branch_p='t')"
    }

    if ![null_p $question_states] { 
	lappend where_clauses "m.question_state in ('[join $question_states "','"]')" 
    }

    if { $mandatory_p } { lappend where_clauses "m.mandatory_p = 't'" }

    # Get the question attributes out of the table
    set sql_km_get_questions "
	select [join $columns ", "]
	from [join $tables ", "]
        where [join $where_clauses " and "]
	$order_by"
    set result [km_db_to_named_list_list km_get_questions $sql_km_get_questions]

    return $result
}
  
ad_proc km_get_question {
    {
	-previous_p 0
	-next_p 0
	-object_id 0
	-entry_explanation_p 0
	-all_types_p 0
    } 
    question_id object_type_id
} {

    Returns the question_id, pretty_name, abstract_data_type, question_state,
    optionally the entry_explanation as a list. 

    Set previous_p to 1 and it returns the previous question in the series
    that belongs to this object type. Set next_p to 1 and it returns the
    next in the series. This proc does NOT do branch analysis. 
    See km-branch.tcl.

    What it does do is search for the next unanswered question that is not
    in a branch when called with a branch question - not necessarily higher
    on the sort key list.

} {
    if { $entry_explanation_p } {
	set entry_string ", q.entry_explanation"
    } else {
	set entry_string ""
    }

    set sql_km_get_question_1 "select sort_key, decode(branch_p,'t',1,0) 
        as branch_p, sort_key as current_sort_key, question_state
	from sn_question_object_type_map
        where question_id=:question_id and object_type_id=:object_type_id"

    if { ![db_0or1row km_get_question_1 $sql_km_get_question_1]} {
	return ""
    }

    if { $next_p || $previous_p } {
   	# We need some extra limitations here so that we don't pull out composite questions or branches.

	set question_state_sql "and question_state <> 'invisible'"
	if { $all_types_p } {
	    set question_state_sql ""
	}
	if { $next_p } {
	    
	    if { $branch_p } {
		# This is a branch leaf. We determine the next question
		# relative to its root.
		set root_id [km_get_root_branch $question_id $object_type_id]
		set current_sort_key "(select sort_key from sn_question_object_type_map where question_id=:root_id and object_type_id = :object_type_id)"
	    }
	    set limitation " and qm.sort_key = (
		select min(sort_key) 
		from sn_question_object_type_map 
	        where object_type_id=:object_type_id
		    and sort_key > :current_sort_key
		    $question_state_sql
		    and branch_p='f' 
		    and parent_question_id=[km_static km_dummy_object_id])"

	} elseif { $previous_p } {
	     set limitation " and qm.sort_key = (
		select max(sort_key) from sn_question_object_type_map
	        where object_type_id=:object_type_id
		    and sort_key < :current_sort_key
	            $question_state_sql
		    and branch_p='f' 
		    and parent_question_id=[km_static km_dummy_object_id])"
	}
    } else {
	set limitation "and q.question_id=:question_id"
    }

    set question_state_sql "and qm.question_state <> 'invisible'"
    if { $all_types_p } {
	set question_state_sql ""
    }
    set result [km_db_1row_to_list km_get_question_2 "
	select q.question_id, q.pretty_name, q.abstract_data_type, qm.question_state $entry_string 
        from sn_questions q, sn_question_object_type_map qm
        where qm.question_id = q.question_id
	    and qm.object_type_id = :object_type_id
	    $question_state_sql
	    $limitation"]

    if { ($next_p || $previous_p) && ![empty_string_p $result] } {
	# We want to skip deprecated and read-only questions
	# which have not been answered.

	set new_question_id [fst $result]
	set new_abstract_data_type [thd $result]
	set new_question_state [lindex $result 3]

	set answer [fst [km_get_object_data -questions [list {question_id pretty_name abstract_data_type question_state} $result] $object_id]]

	if { $new_question_state != "active" && ![km_answered_p $new_abstract_data_type $answer] } {
	    set result [km_get_question -next_p $next_p -previous_p $previous_p -entry_explanation_p $entry_explanation_p -object_id $object_id $new_question_id $object_type_id]
	}
    }

    return $result
}

ad_proc km_get_child_questions {composite_id} {
    Returns a list of child question_ids for this composite question. 
} {
    set sql "select question_id from sn_question_object_type_map
             where parent_question_id = :composite_id and branch_p='f' and question_state <> 'invisible'"
    set result [db_list km_get_child_questions $sql]
    return $result
}

ad_proc km_get_answer_options {question_id} {

    Returns the list of key-value-sort_key tuples of answer options 
    for this question_id, assuming that this question is actually 
    a question of type option.

} { 
   set result [db_list_of_lists km_get_answer_options "
	select option_id, answer_option, sort_key 
	from sn_answer_options 
	where question_id = :question_id
	order by sort_key"]
   return $result 
}

ad_proc km_answer_option {option_id} {
    Returns an the pretty name of an answer option.
} {
   set sql "select answer_option from sn_answer_options where option_id = :option_id" 
   set result [db_string km_answer_option $sql -default ""] 
   return $result 
}

proc km_pretty_question_state { question_state } {
    switch $question_state {
	"active" { return "Active" }
	"deprecated" { return "To be phased out" }
	"read-only" { return "Read-Only" }
	"invisible" { return "Invisible" }
    }
}

# ******************** Input checking ********************

ad_proc km_sort_form { key_value_tuples } {
    Sorts form vars into a list of tuples of keys and value_lists. Date variables are assembled to make one value. Note that
    even if one key has only one value associated with it that this value will be represented as a single element list.
} {
    set sorted_tuples [list]
    set sister_tag_p 0

    #Put dates together
    set tuples  [km_assemble_dates -filter_p 0 [fst [transpose $key_value_tuples]] [snd [transpose $key_value_tuples]]]
    # Sort the tuple values into new value_lists by key
    foreach tuple $tuples {
	
	set key [fst $tuple]
	set value [snd $tuple]

	# Put textareas and their sister html_p tags together
	if {[regexp {(.*)\.html_p} $key match x] } { 
	    set key $x
	    set sister_tag_p 1
	} 

	# Get the index of the element in sorted_tuples to be manipulated
	set ix [lsearch [fst [transpose $sorted_tuples]] $key]
	
	if  {$ix >= 0} {

	    # This key is already in sorted_tuples list so add this value to the value_list in the tuple
	    set new_value_list [concat [snd [lindex $sorted_tuples $ix]] $value]
	    
	    # Replace the old element of sorted_tuples with the new tuple
	    set sorted_tuples [lreplace $sorted_tuples $ix $ix [list $key $new_value_list]] 
	    set sister_tag_p 0

	} elseif {$sister_tag_p} {

	    # Then this is an html_p tag that has no sister textarea tag: i.e. the textarea tag is an empty string
	    lappend sorted_tuples [list $key [list "" $value]] 
	    set sister_tag_p 0

	} else {
            
     
	    # This key isn't yet in the sorted_tuple list so append a new tuple to it
	    lappend sorted_tuples [list $key [list $value]]

	}
    }
    return $sorted_tuples
}

ad_proc km_filter_valid_keys { key_value_list} {
    Returns the tuple if it is valid- i.e. it has an integer key value.
} {
    set filtered_tuples [list]
    foreach tuple $key_value_list {
	if {[integer_p [fst $tuple]]} {lappend filtered_tuples $tuple}
    }
    return $filtered_tuples
}

ad_proc km_filter_other_category_keys { key_value_list} {
    Returns the tuple if it is originating from extra input field for category 'Other'
} {
    set filtered_tuples [list]
    foreach tuple $key_value_list {
	set key [fst $tuple]
	if { [regexp {([0-9]+)_other$} $key match question_id] && [integer_p $question_id] } {
	    lappend filtered_tuples [list $question_id [snd $tuple]]
	}
    }
    return $filtered_tuples
}

ad_proc km_required_questions {object_id} {
    Returns a list of required (or mandatory) questions for this object.
} {
    set context_id [ad_conn package_id]
    set result [db_list km_required_questions "
	select q.question_id
        from sn_questions q, sn_question_object_type_map qm, sn_objects o
        where q.question_id = qm.question_id
	    and qm.object_type_id = o.object_type_id
            and o.object_id = :object_id
            and qm.question_state = 'active' 
            and o.context_id=:context_id
	    and qm.mandatory_p = 't'"]
    return $result
}

ad_proc km_mandatory_child_p { { -parent_id 0 -child_ids {} } } {

    Returns 1 if this composite parent has a mandatory child 
    or if one of the child_ids passed is mandatory.

} {
    if { $parent_id } { set child_ids [km_get_child_questions $parent_id] }

    if [null_p $child_ids] { return 0 }
    set children [km_get_questions -all_properties_p 1 -branch_children_p 1 -root_node_p 0 -question_ids $child_ids]
    set key [fst $children]
    set children [tail $children]

    foreach child $children {
	set mandatory_p [ad_decode [lindex $child [lsearch $key mandatory_p]] t 1 0]
	if { $mandatory_p } { return 1 }
    } 
    
    return 0
}
    
ad_proc km_is_required {
    {
	-object_id 0
	-object_type_id 0
    }
    question_id
} {
    Checks if a question is mandatory.
} {
    if !$object_type_id { set object_type_id [km_get_object_type $object_id] }
    set result [db_list km_is_required "
	select decode(mandatory_p,'t',1,0)
	from sn_question_object_type_map
	where question_id=:question_id and object_type_id=:object_type_id"]
    return $result
}

ad_proc km_answered_p { abstract_data_type answer } {
    Determines on the basis of the abstract_data_type if this question has been answered or is empty.
} {
    switch $abstract_data_type {
	"object_link" { return [expr ![null_p [tail $answer]]] }
	"composite" {
	    # This is a bit tricky. Composite questions come in as little
	    # packets of other questions from km_object_data.
	    # A composite question is considered answered if at least one
	    # child has been answered.

	    foreach item $answer {
		if [km_answered_p [lindex $item 3] [fst $item]] { return 1 }
	    }
	    return 0
	}
	"option" - "other_category" -
	"category" { return [expr ![null_p $answer]] }
	"integer" -
	"text" { return [expr ![empty_string_p [fst $answer]]] }
	default { return [expr ![empty_string_p $answer]]}
    }
}

ad_proc km_check_input {
    {
	-required_p 0
	-complete_p 0
    }
    object_id
    key_value_list
} {

    This proc determines if the list of question_ids and their value
    are valid entries according to their abstract data type.
    It will also check if the set of keys and values submitted are
    enough to make this object complete if complete_p is true. Or it will
    check if a each key is a required question and if so, it will
    check if that question is answered when required_p is true.

    This proc returns a tuple for ad_return_complaint if it has reason
    to complain.

} {
    set invalid_questions {}
    set missing_ids {}
    set required_ids {}

    #Remove any key value tuples that do not have an integer key
    set key_value_list [km_filter_valid_keys $key_value_list]

    # The keys in key value tuples are the question_ids
    set submitted_ids [fst [transpose $key_value_list]]
    set values [snd [transpose $key_value_list]]
 
    #Get a list of required questions to check for completeness 
    if {$complete_p || $required_p} { set required_ids [km_required_questions $object_id] }

    # Get the abstract data types and pretty_names of the questions
    set questions [km_question_attributes [concat $submitted_ids $required_ids]]
    set question_ids [fst [transpose $questions]]
    set abstract_data_types [thd [transpose $questions]]
    set pretty_names [snd [transpose $questions]]

    foreach submitted_id $submitted_ids {
	
	# check the input
	set abstract_data_type [lindex $abstract_data_types [lsearch $question_ids $submitted_id]]
	set value_list [lindex $values [lsearch $submitted_ids $submitted_id]]

	switch $abstract_data_type {
	    "text" {
		if {($complete_p || $required_p) && ([elem_p $submitted_id $required_ids] && ([null_p $value_list] \
                     || [empty_string_p [fst $value_list]])) } {
		    lappend missing_ids $submitted_id
		}
	    }
	    "category" - "other_category" - "option" {
		set max_categories [km_static question_max_categories $submitted_id]
		if {($complete_p || $required_p) && [elem_p $submitted_id $required_ids] && [null_p $value_list]} {
		    lappend missing_ids $submitted_id
		} elseif {![empty_string_p $max_categories] && ([llength $value_list] > $max_categories)} {
		    lappend invalid_questions [list $submitted_id "more than the allowed $max_categories categories chosen"]
		}
	    }
	    "date" {
		if {($complete_p || $required_p) && [elem_p $submitted_id $required_ids] && [empty_date_p $value_list]} {
		    lappend missing_ids $submitted_id
		} elseif {![empty_date_p $value_list] && ![date_p $value_list]} {
		    lappend invalid_questions [list $submitted_id "no valid date"]
		}
	    }
	    "integer" {
		if {($complete_p || $required_p) && ([elem_p $submitted_id $required_ids] && ([null_p $value_list] \
		    || [empty_string_p [fst $value_list]])) } {
		    lappend missing_ids $submitted_id
		} elseif {![integer_p [fst $value_list]]} {
		     lappend invalid_questions [list $submitted_id "no valid number"]
		}
	    }
	    "file" {
		set client_filename [fst $value_list]

		if {($complete_p || $required_p) && [elem_p $submitted_id $required_ids] } {
		    if [empty_string_p $client_filename] {
			# Go to the db and see if a file already exists for this variable.
			if ![km_upload_exists_p $object_id $submitted_id] {
			    lappend missing_ids $submitted_id
			}
		    }
		}

		if { ![empty_string_p $client_filename] } {
		    # If the temporary file is empty, there has been an error.
		    set tmpfilename [ns_queryget $submitted_id.tmpfile]
		    if ![file size $tmpfilename] {
			lappend invalid_questions [list $submitted_id "invalid file"]
		    }
		}
	    }
	    default { continue }
	}
    }

    foreach question_id $required_ids {
	set abstract_data_type [lindex $abstract_data_types [lsearch [fst [transpose $questions]] $question_id]]
	switch $abstract_data_type {
	    "user_link" {
		if ![null_p [km8_get_to_links $object_id $question_id]] {
		    # If there already are links for this question, we pretend
		    # it has been submitted.
		    lappend submitted_ids $question_id
		}
	    }
	     "file" {
		 if ![km_upload_exists_p $submitted_id $object_id] {
		     lappend submitted_ids $question_id
		 }
	     }
	    "child_object" {
		 if {![null_p [km_get_child_objects $object_id $question_id]] } {
		     lappend submitted_ids $question_id
		 }
	     }
	    "nephew_object" {
		 if {![null_p [km_get_nephew_objects $object_id $question_id]] } {
		     lappend submitted_ids $question_id
		 }
	     }
	 }
    }

    # Check if there are some other keys that also should be submitted to make this complete
    if { $complete_p } { set missing_ids [concat $missing_ids [minus $required_ids $submitted_ids]] }

    #Prepare the complaint messages
    foreach missing_id $missing_ids {
	set pretty_name [lindex $pretty_names [lsearch $question_ids $missing_id]]  
	append message "You must supply a value for $pretty_name<br>"
    }
    foreach invalid_question $invalid_questions {
	set invalid_id [fst $invalid_question]
	set invalid_msg [snd $invalid_question]
	set pretty_name [lindex $pretty_names [lsearch $question_ids $invalid_id]]  
	append message "You have entered an invalid entry for $pretty_name ($invalid_msg)<br>"
    }
    set i [expr [llength $invalid_questions] + [llength $missing_ids]]
    if { $i } { 
	return [list $i $message]
    } else { 
	return 
    }
}

ad_proc km_upload_exists_p { object_id question_id } {
    Looks if a file has been uploaded for this object_id and question_id
} {
    set client_filename [db_string km_upload_exists_p "
	select content from sn_content
        where object_id=:object_id
	    and question_id=:question_id" -default ""]

    if ![empty_string_p $client_filename] { 
	return 1 
    } else {
	return 0
    }
}

ad_proc km_original_author_p { user_id object_id } {
    Test if the user is the original author of this object.
} {
    set context_id [ad_conn package_id]
    set result [db_string km_original_author_p "
	select count(*) from sn_objects where object_id=:object_id and original_author_id=:user_id and context_id=:context_id"]

    return $result
}

ad_proc km_type_has_data_type_p { object_type_id data_type } {
    Determines if an object type has a column of the given abstract datatype.
} {
    return [db_string km_type_has_data_type_p "
	select decode(count(*),0,0,1) from dual where exists (select 1
	from sn_questions q, sn_question_object_type_map qm
        where qm.question_id = q.question_id
	    and qm.object_type_id = :object_type_id 
	    and qm.question_state <> 'invisible'
	    and q.abstract_data_type = :data_type)"]
}

# ******************** Date Procs ********************

ad_proc month_list  {} {
    Returns a list of  months, the second row holds the corresponding 2 digit values.
} {
    set months [list Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec]
    set values [list 01 02 03 04 05 06 07 08 09 10 11 12]
    return [list $months $values]
}

ad_proc km_default_date {question_id} {
    Calculates a default date based on the question_id
} {
    set default_value [db_string km_default_date_1 "
	select default_value from sn_questions where question_id=:question_id" -default ""]
    if [empty_string_p $default_value] {
	set result {}
    } else {
	if { ![catch {set default_date [db_string km_default_date_2 "select $default_value from dual"]}] } {
	    set result [km_break_date $default_date]
	} else {
	    set result {}
	}
    }
    return $result
}

ad_proc km_get_year_range {question_id} {
    Calculates the year range for this date question
} {
    db_1row get_year_range {
	select (to_char(add_months(sysdate,12*year_from),'YYYY')) as year_from,
               (to_char(add_months(sysdate,12*year_to),'YYYY')) as year_to
	from sn_questions
	where question_id = :question_id
    }
    return [list $year_from $year_to]
}

ad_proc km_assemble_dates { {-filter_p 1} keys values } {
    
    Given a list of keys and values, this proc searches for all keys with the form
    key_name:month, key_name:year, key_name:day, which is how km-date-tags
    returns them in this module. It returns a list of key value tuples
    of the form:  key_name YYYY-MM-DD. It does not check if these are valid dates, that's
    what date_p does. If filter_p is set to 1, it returns only the date tuples back.
    Otherwise it returns all other keys and values in the same order. 
  
} {
    # we need a list of key_name tags and their values, sort them so that
    # all key_names are grouped together and that :day, :month, :year
    # will be evaluated in that order

    set key_name ""
    set tuples [zip $keys $values]
    set tuples [qsort $tuples]
    set last_key_name ""
    set result [list]

    set counter 0
    foreach tuple $tuples {

	if { [regexp {(.*)\.(month|year|day)} [fst $tuple] match key_name] } {
	    # Looks like a date value.

	    if { [string compare $key_name $last_key_name] == 0 } {   
		lappend ${key_name}_value [snd $tuple]
	    } else {
		
		# It's a different key_name so append the last_key_name and 
		# its date to the list (if there is one).

		if ![empty_string_p $last_key_name] {
		    lappend dates [list $last_key_name [set ${last_key_name}_value]]
		}
		
		# Start assembling the next date
		lappend ${key_name}_value [snd $tuple]
	    }

	    set last_key_name $key_name
	    incr counter

	} elseif !$filter_p {
	    # No date value, but we are not allowed to filter other values out.
	    # So just copy it.

	    lappend result $tuple
	}
    }

    if { $counter } {

	# Append the last last_key_name and its date to the list.
	lappend dates [list $last_key_name [set ${last_key_name}_value]]

	# Oracle will want the date YYY-MM-DD instead of a list DD MM YYYY, 
	# so split the tuples up again and format the date part.

	set formatted_dates [map {flip join "-"} [map reverse [snd [transpose $dates]]]]

	# Pack them up again to be returned as a list of tuples.
	set result "$result [zip [fst [transpose $dates]] $formatted_dates]"
    }

    return $result
}


# ******************** Navigation ********************

ad_proc generate_path_id {} {

    Generates a random number to uniquely define this excursion state in
    the knowledge management library.

    This number will be passed from one form to the next throughout the excursion.

} {
    while 1 {
	# Note that ns_rand only gives us six digits - so we call it twice.
	set path_id "[ns_rand 1000000][ns_rand 1000000]"

	if ![db_string generate_path_id "select count(*) from km_path where path_id=:path_id"] {
	    return $path_id
	}
    }
}

ad_proc set_path_values {{-question_id ""} object_id category_id } {

    Returns a simple list of name and value tuples that should be stored
    for library excursions in the km_excursion_path table

} {
    
    if ![empty_string_p $question_id] {
	set values [list object_id $object_id question_id $question_id category_id $category_id]
    } else {
	set values [list object_id $object_id category_id $category_id]
    }
    return $values
}

ad_proc init_path {
    {
	-last_path_id ""
    } 
    values_list return_to action
} {

    Initializes the path variable for an excursion through the library.

    Begins an excursion by saving all the information that will be
    necessary at the end of this excursion to return to the same place it
    started. It is assumed that an excursion is only occuring while
    editing, creating or linking an object. In all other cases there will
    be no excursion state and also no object_id. Important! The
    last_path_id that is being passed is the id of the excursion state out
    of which this excursion is being launched!

} {
    set path_id [generate_path_id]
    if [empty_string_p $last_path_id] { set last_path_id [db_null] }

    # Insert the info about this particular excursion into the database

    db_dml init_path "insert into km_path
	(path_id, last_path_id, action, values_list, return_to, start_time) 
        values (:path_id, :last_path_id, :action,
	    :values_list, :return_to, sysdate)"

    return $path_id
}

ad_proc get_path_return_url {path_id} {

    Returns the destination of a path if it's a valid path, otherwise
    it returns an empty string.

} {
    if [empty_string_p $path_id] { return "" }

    set calc_excursion_timeout [expr [excursion_timeout]/24]
    set sql "select return_to from km_path 
	where path_id=:path_id and sysdate - start_time < :calc_excursion_timeout"

    if {[catch {set destination [db_string get_path_return_url $sql]}]} {
	set result ""
    } else {
	set result $destination
    }

    return $result
}

ad_proc get_path_values { {-action_only_p 1} path_id } {

    Given an excursion's path_id, this proc returns the action of this
    excursion as a two-part list: {action action_value}. This is often
    necessary for a page to know what it is supposed to do. If the option
    action_only_p is set to 0 then a simple list is returned with the name
    and value of the variables in the same order that they were given to
    km_set_path_values

} {
	
    set timeout [excursion_timeout]

    if !$action_only_p {
	# Return a list with all excursion details 
	set values ", values_list "
    } else {
	set values ""
    }
	
    # Get out the action and the list of values from oracle and
    # turn it into a list of action and the other values
	
    set values_list {}
    if { ![db_0or1row get_path_values "
	select action $values from km_path 
      where path_id=:path_id and sysdate - start_time < :timeout/24"]} {
	set result {}
    } else {

	split $values_list

	set result {}
	foreach value $values_list { lappend result $value }
	lappend result action $action
    }

    return $result
}

ad_proc get_path_action {path_id} {
    Returns the path's action as a plain value.
} {
    return [snd [get_path_values $path_id]]
}

ad_proc get_last_path {path_id} {

    If this was an excursion launched within another excursion then we
    want to return the path_id of that excursion.

} {
    set last_path_id [db_string get_last_path "
	select last_path_id from km_path where path_id=:path_id" -default ""]

    return $last_path_id
}

ad_proc path_valid_p {path_id} {
    Checks if the given path (still) exists.
} {
    if [empty_string_p $path_id] { return 0 }

    set calc_excursion_timeout [expr [excursion_timeout]/24]
    set result [db_string path_valid_p "
	select count(*) from km_path 
	where path_id=:path_id and sysdate - start_time <= :calc_excursion_timeout"]
    return $result
}

ad_proc value_from_tuples {key_values key} {

    Given a list of key/value tuples, returns the value associated with
    the given key.

} {
    if { [lsearch $key_values $key] == -1 } { 
	return ""
    } else {
	return [lindex $key_values [expr [lsearch $key_values $key] + 1]]
    }
}

ad_proc sn_success_story_id {} {
    Returns the object_type_id for the Success Stories.
} {
    set context_id [ad_conn package_id]
    set result [db_string sn_success_story_id "
	select object_type_id from sn_object_types 
	where upper(pretty_name) = upper('success story') and context_id=:context_id" -default "0"]

    return $result
}


ad_proc km_workflow_task_per_object {object_id} {
    Return information about the current workflow task for this object_id
} {
    # dirk: remove the rownum - it's only in for debugging purposes.
    if {[db_0or1row km_workflow_task_per_object {
        select t.task_id, 
            t.case_id, 
            t.transition_key, 
            t.state, 
            t.transition_name as task_name
        from   wf_cases c,
               wf_user_tasks t
        where  c.case_id = t.case_id
           and c.object_id=:object_id
           and rownum=1}]} {
        return [list \
                    workflow_task_p t \
                    task_id $task_id \
                    case_id $case_id \
                    transition_key $transition_key \
                    state $state \
                    task_name $task_name]
    } else {
        return [list \
                    workflow_task_p f \
                    task_id "" \
                    case_id "" \
                    transition_key "" \
                    state "" \
                    task_name ""]
    }
    return ""
}


ad_proc library_approval_process_notifications {} {

    Processes the notifications queued in table wf_library_notifications.

} {
    # Make sure that only one thread is processing the queue at a time.
    if {[nsv_incr library library_notifications_p] > 1} {
	nsv_incr library library_notifications_p -1
	return
    }

    with_finally {

	db_foreach get_notifications {
	    select notification_id, task_id, template_key, recipient_id, sender_id, body
	    from wf_library_notifications
	} {

	    db_1row ticket_info {
		select o.object_id,
		       o.one_line_description as shortname, sn.node_id,
	               u1.email as sender_email, u2.email as recipient_email,
		       u1.first_names || ' ' || u1.last_name as sender_name,
		       u2.first_names || ' ' || u2.last_name as recipient_name,
		       apm_package.name(o.context_id) as package_name
		from wf_tasks wt, wf_cases wc, sn_objects o, users u2,
		     site_nodes sn, users u1
		where u1.user_id = :sender_id
	        and u2.user_id = :recipient_id
		and wc.case_id = wt.case_id
		and wt.task_id = :task_id
	        and o.object_id = wc.object_id
                and sn.object_id = o.context_id
	    }

	    set community_id [sn_community::leaf_community $node_id]
	    
	    set replacement_list \
		    [list \
		    object_id		$object_id \
		    object_link		"[ad_url]/o/$object_id" \
		    package_name		$package_name \
		    shortname		$shortname \
		    submitter_name		$sender_name \
		    submitter_email	$sender_email \
		    recipient_email	$recipient_email \
		    notification_sender	$sender_email \
		    body                   $body]
	    
	    db_dml delete_notification {
		delete from wf_library_notifications
		where notification_id = :notification_id
	    }

	    et_send_email [et_process -community_id $community_id $template_key $replacement_list]
	}

    } finally {
	nsv_incr library library_notifications_p -1
    }
}


util_report_successful_library_load
