# /tcl/km-browsing.tcl
#
# Browsing of knowledge objects.
# Created by carsten@arsdigita.com in June 2000
#
# $Id: km-browse-procs.tcl,v 1.2 2009/02/13 20:40:39 jeffd Exp $

ad_proc km_ancestors_context_list {ancestors {category_id ""} {pre_url ""}} {

    Get a list of links to the parent objects, plus a link 
    to the root parent's object type.

} {
    set root_object_id [fst [fst $ancestors]]

    db_1row get_root_info {
	select o.one_line_description as ancestor_name, 
	    o.object_type_id as ancestor_object_type_id, 
	    ot.pretty_plural as ancestor_object_type,
	    o.archived_p as ancestor_archived_p
	from sn_objects o, sn_object_types ot
	where o.object_id = :root_object_id
	    and ot.object_type_id = o.object_type_id
    }

    if {$ancestor_archived_p == "t"} {
	set ancestor_object_type "$ancestor_object_type (Archived)"
	set archive t
    }
    lappend result [list "${pre_url}browse-one-type?[export_vars [list [list object_type_id $ancestor_object_type_id] archive]]" $ancestor_object_type]

    if {![empty_string_p $category_id] && $category_id != "none"} {
	if {[db_string is_object_of_category {select count(*) from sw_object_category_map where category_id = :category_id and object_id = :root_object_id}] > 0} {
	    lappend result [list "${pre_url}browse-one-category?[export_vars [list [list object_type_id $ancestor_object_type_id] category_id archive]]" [km_category_name $category_id]]
	}
    }

    foreach ancestor $ancestors {
	set name [sn_striphtml [snd $ancestor]]
	if {[empty_string_p $name]} {
	    set name "Unnamed [km_static object_type_pretty_name [thd $ancestor]]"
	}
	lappend result [list "${pre_url}object-view?object_id=[fst $ancestor]&category_id=$category_id" $name]
    }

    return $result
}

ad_proc km_sort_by_bar { sort_by child_type_p } {
    Sort: Object name | Modification date
} {
    set sort_by_list {{name "name"} {last_modified "modification date"}}
    lappend sort_by_list {average_rating "user rating"}
    if { $child_type_p } { lappend sort_by_list {parent "parent object"} }

    foreach sort_item $sort_by_list {
	set sort_column [lindex $sort_item 0]
	set sort_description [lindex $sort_item 1]

	if {$sort_column == $sort_by} {
	    lappend sort_text "<b>$sort_description</b>"
	} else {
	    lappend sort_text "<a href=\"[ns_conn url]?[export_ns_set_vars url {sort_by}]&sort_by=$sort_column\">$sort_description</a>"
	}
    }
    
    return "\[ [join [concat $sort_text] " | "] \]"
}

ad_proc km_category_display_limit {} {

    If the object list contains more than this many objects,
    no categorization will be shown.

} {
    return [ad_parameter CategoryDisplayLimit library 20]
}

ad_proc km8_output_object_list {
    {
	-object_type_id 0
	-header ""
	-current_action ""
	-path_id ""
	-pool all
	-age all
	-sort_by ""
	-show short
	-category_id ""
	-question_id 0
	-where_clauses {}
	-minus_join_tables {}
	-minus_where_clauses {}
	-first_row 0
	-last_row 0
	-begin_letter ""
	-archived "f"
	-start_date ""
	-end_date ""
        -more_join_tables {}
    }
} {

    Output a list of objects.

} {
    ! Entered km8_output_object_list +more_join_tables
    set user_id [ad_conn user_id]
    set package_id [ad_conn package_id]
    set instance_read_p [ad_permission_p $package_id "read"]

    template::multirow create object_list public_p archived_p review_p object_view_link object_name user_link last_modified number_ratings rating_avg rating_avg_img rating_max rating_max_img access_total access_month overview categories_html

    set child_type_p [db_string km_output_object_list_1 "
	select decode(count(*),0,0,1) from sn_questions
	where abstract_data_type in ('child_object','nephew_object')
	    and target_object_type_id = :object_type_id"]

    # If the user is linking, or adding a SN reference,
    # we add a button to each object.

    # Make sure the user cannot see expired or private objects.
    # Exceptions: Administrators, Sharenet Managers, and users who have been granted 
    # view permission for the object.

    set join_tables [list "sn_objects obj"]

    set package_id [ad_conn package_id]
    lappend where_clauses \
	"obj.expiration_date > sysdate" \
	"obj.object_type_id = :object_type_id" \
	"obj.context_id = :package_id" \
	"obj.archived_p = :archived"

    if { !$instance_read_p && $pool == "all" } {
	# Only count published objects, or objects for which the user
	# has view permission.

	# Note that we only check direct read permission on flat
	# groups and users. Since the context_id of private objects is
	# NULL, and we only grant permissions to private groups, this
	# is sufficient.

	lappend where_clauses \
	    "(obj.public_p = 't' or obj.object_id in (
	      select object_id from acs_permissions 
	      where (grantee_id = :user_id
		     or grantee_id in (select group_id from membership_rels
				       where person_id = :user_id))
		  and privilege = 'read'))"
    }

    if { [string compare $age all] != 0 } {
	# Filter by time of last modification.
	lappend where_clauses "sysdate - obj.last_modified < :age"
    }

    # Filter by author and co-owners.
    switch $pool {
	own { lappend where_clauses "obj.original_author_id = :user_id" }
	coown {
	    lappend join_tables "acs_permissions p"

	    lappend where_clauses \
		"obj.object_id = p.object_id" \
		"p.privilege = 'write'" \
		"(p.grantee_id = :user_id
		  or p.grantee_id in (select group_id from membership_rels where person_id = :user_id))"
	}
    }

    # Sort by time of last modification or by name.
    set columns {1}
    switch $sort_by {
	average_rating {
	    set sort_by_clause "nvl(cr.rating_sum,0) / nvl(cr.rating_count,1) desc, obj.last_modified desc"
	    set sort_by_clause2 "rating_avg desc, last_modified desc"
	}
	last_modified { 
	    set sort_by_clause "obj.last_modified desc, lower(obj.one_line_description)" 
	    set sort_by_clause2 "last_modified desc, lower(name)" 
	}
	name { 
	    set sort_by_clause "lower(obj.one_line_description), obj.last_modified desc" 
	    set sort_by_clause2 "lower(name), last_modified desc" 
	}
	parent {
	    set sort_by_clause "lower(obj2.one_line_description), lower(obj.one_line_description), obj.last_modified desc"
	    set sort_by_clause2 "lower(parent_name), lower(name), last_modified desc"

	    lappend columns "obj2.one_line_description as parent_name"
	    lappend join_tables "km_flat_object_hierarchy fo" "sn_objects obj2"
	    lappend where_clauses \
		"fo.child(+) = obj.object_id" \
		"(fo.distance = 1 or fo.distance is null)" \
		"fo.parent = obj2.object_id(+)"
	}
	default { 
	    set sort_by_clause "$sort_by, obj.last_modified desc"
	    set sort_by_clause2 "$sort_by, last_modified desc"
	}
    }

    if { ![string equal $begin_letter all] } {
	if { ![string equal $begin_letter other] } {
	    lappend where_clauses "substr(upper(obj.one_line_description),1,1) = '[string toupper $begin_letter]'"
	} else {
	    lappend where_clauses "(substr(upper(obj.one_line_description),1,1) < 'A' or substr(upper(obj.one_line_description),1,1) > 'Z')"
	}
    }

    if { ![empty_string_p $start_date] && ![empty_string_p $end_date]} {
	lappend where_clauses "obj.start_date <= :end_date" "obj.end_date >= :start_date"
    }

    set join_tables [concat $join_tables $more_join_tables]

    # We first need to get the number of listed objects, because
    # depending on this number, we will or will not display object
    # categorization.

    ! Get the number of listed objects
    set num_found_objects [db_string count_all_objects "
	select count(unique obj.object_id)
	from [join $join_tables ", "]
	where [join $where_clauses "\nand "]"]
    ! Got the number of listed objects +num_found_objects

    if { ![null_p $minus_join_tables] } {
        ! Get the number of excluded objects
	set num_excluded_objects [db_string count_excluded_objects "
	    select count(unique obj.object_id)
	    from [join [concat $join_tables $minus_join_tables] ", "]
	    where [join [concat $where_clauses $minus_where_clauses] "\nand "]"]
        ! Got the number of excluded objects +num_excluded_objects

	set num_found_objects [expr $num_found_objects - $num_excluded_objects]
    }

    if { $child_type_p } {
	lappend columns "km_utilities.km_ancestor_list(obj.object_id) as ancestors"
    }

    set display_limit [km_category_display_limit]
    set category_questions {}
    if { $num_found_objects < $display_limit } {
	# Get node_id, category, question_id, question_pretty_name.
	set category_questions [km_node_question_list $object_type_id]

	foreach category_tuple $category_questions {
	    set root_node_id [fst $category_tuple]
	    lappend columns "km_utilities.km_category_list(obj.object_id, $root_node_id) as cat$root_node_id"
	}
    }

    if { [null_p $minus_join_tables] } {
	set sql "
    select * from (select obj.object_id,
        decode(obj.public_p,'t',1,0) as public_p,
        decode(obj.archived_p,'t',1,0) as archived_p,
        decode(obj.in_review_p,'t',1,0) as review_p,
        u.first_names || ' ' || u.last_name as owner, u.user_id as owner_id,
        obj.one_line_description as name, obj.overview, 
        decode(obj.overview_html_p,'t',1,0) as overview_html_p,
        obj.last_modified, nvl(obj.access_total,0) as access_total,
	nvl(obj.access_month,0) as access_month,
        round(nvl(cr.rating_sum,0) / nvl(cr.rating_count,1),1) as rating_avg,
	nvl(cr.rating_max,0) as rating_max,
        nvl(cr.rating_count,0) as number_ratings,
	dense_rank() over (order by $sort_by_clause) dr,
        [join $columns ", "]
    from [join $join_tables ", "], sn_comments_ratings cr, users u
    where u.user_id = obj.original_author_id
	and cr.object_id(+) = obj.object_id
        and [join $where_clauses "\nand "])
    where dr >= :first_row and (:last_row = 0 or dr <= :last_row)
    order by $sort_by_clause2"
    } else {
	# This will be a bit complicated. We were called from browse-one-category
	# to display a list of uncategorized objects. The fastest way to do this
	# is with a MINUS, using only the object_id column. We then join the result
	# with the query we would normally use to get the data we want.

	set sql "
select * from (select obj.object_id,
    decode(obj.public_p,'t',1,0) as public_p,
    decode(obj.archived_p,'t',1,0) as archived_p,
    decode(obj.in_review_p,'t',1,0) as review_p,
    u.first_names || ' ' || u.last_name as owner, u.user_id as owner_id,
    obj.one_line_description as name, obj.overview, 
    decode(obj.overview_html_p,'t',1,0) as overview_html_p,
    obj.last_modified, nvl(obj.access_total,0) as access_total,
    nvl(obj.access_month,0) as access_month,
    round(nvl(cr.rating_sum,0) / nvl(cr.rating_count,1),1) as rating_avg,
    nvl(cr.rating_max,0) as rating_max,
    nvl(cr.rating_count,0) as number_ratings,
    dense_rank() over (order by $sort_by_clause) dr,
    [join $columns ", "]
from [join $join_tables ", "], sn_comments_ratings cr, users u,
    (select obj.object_id
     from [join $join_tables ", "]
     where [join $where_clauses "\nand "]
     minus
     select /*+ORDERED*/ obj.object_id
     from [join [concat $join_tables $minus_join_tables] ", "]
     where [join [concat $where_clauses $minus_where_clauses] "\nand "]) filtered
where u.user_id = obj.original_author_id
    and cr.object_id(+) = obj.object_id
    and obj.object_id = filtered.object_id)
where dr >= :first_row and (:last_row = 0 or dr <= :last_row)
order by $sort_by_clause2"
    }

    # We pass use a path to pass a couple of values to object-view,
    # so they can be used as return URLs. (This is important because
    # we want to preserve the user's filter and sort settings.)

    set value_list [list browse_type [export_url_vars pool age sort_by show begin_letter]]
    set parameters ""
    if { ![empty_string_p $category_id] } {
	set value_list [concat $value_list [list browse_category [export_url_vars pool age sort_by show begin_letter question_id column_name]]]
	set parameters "&category_id=$category_id"
    }
    set new_path_id [init_path -last_path_id $path_id $value_list "" "browse"]

    ! Getting the object list
    set old_rank "0"
    db_foreach get_object_list $sql {
	## filter out duplicate rows with the same dense_rank than the row
	## before
	if {$dr == $old_rank} {
	    continue
	}
	set old_rank $dr
  	set categories_html ""

	# Display the object's parent.
	if { $child_type_p && ![empty_string_p $ancestors] } {
	    append categories_html "Related Objects: "
	    foreach ancestor $ancestors {
		append categories_html "<a href=\"object-view?object_id=[fst $ancestor]\">[snd $ancestor]</a>; "
	    }
            set categories_html [string range $categories_html 0 end-2]
	}

  	if { $num_found_objects <= $display_limit && ![null_p $category_questions] } {
	    # List the object's categories.
	    foreach category_pair $category_questions {
		set root_node_id [fst $category_pair]
		set question_name [lindex $category_pair 3]
		set category_value [set cat$root_node_id]

		if ![empty_string_p $category_value] {
		    if ![empty_string_p $categories_html] { append categories_html "; " }
		    append categories_html "$question_name: $category_value"
		}
  	    }
  	}

	if ![empty_string_p $overview] {
	    set overview [util_trim_string_with_hrefs [sn_striphtml $overview]]
	}

	if [empty_string_p $name] { 
	    set name "Unnamed [km_static object_type_pretty_name $object_type_id]"
	}

	set object_view_link "object-view?object_id=$object_id$parameters&path_id=$new_path_id"
        template::multirow append object_list $public_p $archived_p $review_p $object_view_link [sn_striphtml $name] [ad_present_user $owner_id $owner] [util_AnsiDatetoPrettyDate $last_modified] $number_ratings $rating_avg [gf_feedback_points_img $rating_avg 5] $rating_max [gf_feedback_points_img $rating_max 5] $access_total $access_month $overview $categories_html
    }
    ! Got the object list

    ! Exiting km8_output_object_list
    return $num_found_objects
}
