# /tcl/km-feedback.tcl
#
# General comment system adapted to KM.
#
# Created by carsten@arsdigita.com in June 2000
# modified by bernd@arsdigita.com in Jan. 2001
#
# ported by dirk@arsdigita.com in July 2001

ad_proc km_check_comment_id { user_id comment_id } {

    Checks if the user may edit the comment. If not, 0 is returned.
    If he has edit permission, the associated object_id is returned.
    
} {
    if {![db_0or1row km_check_comment_id_10 {
	select object_id, user_id as comment_user_id
	from sn_comments where comment_id = :comment_id
    }] } {
	ad_return_exception_page 404 "" \
	    "The entry $comment_id does not exist in table general_comments."
	return 0
    }

    if ![km_check_object_id $object_id] { return 0 }

    if { $user_id != $comment_user_id && ![km_conn admin_p] } {
	ad_return_complaint 1 "<li>You cannot edit this entry because you did not post it."
	return 0
    }

    return $object_id
}

ad_proc km_general_comment_add {
    object_id user_id html_p reuse_points scale feedback_category_id public_p comment
} {
    
    Inserts a comment into the general comment system with additional
    possibility of giving feedback.

} {
    set ip_address [ns_conn peeraddr]
    set package_id [ad_conn package_id]
    set short_comment [string range $comment 0 3999]
    
    if {$reuse_points > $scale} {
	return 0
    }

    db_transaction {
	set new_comment_id [db_exec_plsql km_general_comment_add_10 {
	    begin
                :1 := sn_comment.new_comment (
                object_id            => :object_id,
                html_p               => :html_p,
		display_p            => :public_p,
                rating               => :reuse_points,
		scale                => :scale,
                user_id              => :user_id,
                ip_address           => :ip_address,
	        package_id           => :package_id,
                category_id          => :feedback_category_id,
	        short_content        => :short_comment
                );
	    end;
	}]

	db_dml km_general_comment_add_20 {
            update sn_comments
            set content = empty_clob()
            where comment_id = :new_comment_id
            returning content into :1
	} -clobs [list $comment]
    }

    return $new_comment_id
}

ad_proc km_general_comment_update {
    {
	-html_p "f"
	-public_p "t"
	-feedback_category_id ""
    }
    user_id object_id comment_id comment
} {
    Updates a comment in the general comment system.
} {
    set ip_address [ns_conn peeraddr]
    set short_comment [string range $comment 0 3999]

    db_exec_plsql km_general_comments_update_10 {
	begin
	sn_comment.update_comment (
        comment_id           => :comment_id,
	html_p               => :html_p,
	display_p            => :public_p,
	user_id              => :user_id,
	ip_address           => :ip_address,
	category_id          => :feedback_category_id,
	short_content        => :short_comment
	);
	end;
    }

    db_transaction {
        db_dml km_general_comments_update_20 "
        update sn_comments
        set content = empty_clob()
        where comment_id = :comment_id
        returning content into :1"  -clobs [list $comment]

    }
}

ad_proc feedback_points_img { feedback_points scale } {
    Returns an IMG tag that shows via stars how valuable a posting was.
    We have up to 5 stars, so we might need to scale.
} {
    if { $scale == 5 } {
	set scaled_feedback_points $feedback_points
    } else {
	set scaled_feedback_points [expr [expr [expr $feedback_points * 1.0] / $scale] * 5]
    }

    if {$scaled_feedback_points <= 0} {
	set gif_number "00"
    } elseif {$scaled_feedback_points > 0 && $scaled_feedback_points < 1} {
	set gif_number "05"
    } elseif {$scaled_feedback_points == 1} {
	set gif_number "10"
    } elseif {$scaled_feedback_points > 1 && $scaled_feedback_points < 2} {
	set gif_number "15"
    } elseif {$scaled_feedback_points == 2} {
	set gif_number "20"
    } elseif {$scaled_feedback_points > 2 && $scaled_feedback_points < 3} {
	set gif_number "25"
    } elseif {$scaled_feedback_points == 3} {
	set gif_number "30"
    } elseif {$scaled_feedback_points > 3 && $scaled_feedback_points < 4} {
	set gif_number "35"
    } elseif {$scaled_feedback_points == 4} {
	set gif_number "40"
    } elseif {$scaled_feedback_points > 4 && $scaled_feedback_points < 5} {
	set gif_number "45"
    } else {
	set gif_number "50"
    }

    set points [format "%.1f" $feedback_points]
    regsub {\.0} $points "" points
    return "stars[set gif_number].gif"

#    return "<nobr>$points&nbsp;<img src=\"graphics/stars[set gif_number].gif\" alt=\"$feedback_points feedback points\" width=\"50\" height=\"9\"></nobr>"
}









