# /tcl/km-users.tcl
# User centric knowledge management.
# (My Objects, User Contributions, etc.)
#
# Created by carsten@arsdigita.com in June 2000
#
# $Id: km-users-procs.tcl,v 1.2 2009/02/13 20:40:39 jeffd Exp $

ad_proc km_workspace_toolbar {object_id {category_id ""} {child_p 0}} {

    Show a toolbar to allow the user to add an object to his workspace
    or list of alerts, or send it to a colleague.

} {
    set user_id [ad_conn user_id]
    set original_author_id [km_conn original_author_id]

    if { $user_id != $original_author_id } {
	set parameters "[export_url_vars object_id category_id]&return_url=[ns_urlencode "object-view?[export_url_vars object_id category_id]"]"

    }

    # Always allow the user to send the page to a colleague.

    set page_url "[ns_conn url]?[export_ns_set_vars url {path_id}]"
    set return_title [km_conn object_name]

    lappend result "<a href=\"send-page-ref?[export_url_vars object_id category_id page_url return_title]\">Send this page to a colleague</a>"

    if { $user_id != $original_author_id && [km_conn public_p] && ![km_conn archived_p] } {
	lappend result "<a href=\"comment-add?[export_url_vars object_id category_id]\">Give Feedback</a>"
    }

    set object_type_id [km_conn object_type_id]
    set object_type [util_quotehtml [km_static object_type_pretty_name $object_type_id]]
    set copy_p [km_static object_type_copy_p $object_type_id]
    set create_p [km_conn create_p]
    set write_p [km_conn write_p]
    if {![km_static object_type_create_p $object_type_id]} {
	set create_p 0
    }

    if {$create_p && !$child_p} {
	lappend result "<a href=\"object-edit?object_type_id=$object_type_id\">Create new $object_type</a>"
    }

    if {$write_p && $copy_p && $create_p} {
	lappend result "<a href=\"object-copy-1?[export_url_vars object_id category_id]\">Copy this $object_type</a>"
    }

    return [join $result " | "]
}

ad_proc km_list_user_searches {user_id} {
   
    Returns a list of the searches the user has done in the past.
    Each row looks like this: name url_params

} {
    set result [db_list_of_lists km_list_user_searches "
	select name, url_params from km_library_searches 
	where user_id=:user_id
        and name is not null"]

    return $result
}

ad_proc km_user_info {user_id} {

    Given an user_id, returns a list with
    full name, email, affiliation, work phone, country

} {
    set result [km_db_1row_to_list km_user_info "
	select first_names || ' ' || last_name, email
		from users u
	where u.user_id=$user_id"]
    return $result
}



