# packages/library/tcl/library-init.tcl
#
# $Id: library-init.tcl,v 1.2 2009/02/13 20:40:39 jeffd Exp $

ad_proc excursion_timeout {} {
    
    Returns the amount of time in hours that an excursion exists
    before being timed out.

} {
    return 120
}

proc purge_old_paths {} {
    set timeout [expr [excursion_timeout]/24]
    db_dml purge_old_paths {
	delete from km_path 
	where sysdate - start_time > :timeout
    }
}

proc reset_access_counts_monthly {} {
    if {[ns_fmttime [ns_time] "%e"] == 1} {
        ns_log Notice "resetting access_month counts to 0"
        db_dml reset_access_counts_monthly {
            update sn_objects
            set access_month = 0
	    where access_month <> 0
        }
    }
}

if { ([server_cluster_enabled_p] && [ad_canonical_server_p]) || ![server_cluster_enabled_p]} {
    # Register sn_object with the bookmarks module, if it's present.
    ad_call_proc_if_exists bookmarks::register_type "sn_object"
    ad_call_proc_if_exists yp::register_type "sn_object"
    
    # Do the same for incentives.
    ad_call_proc_if_exists incentives::register_charge_category \
        -key_usage key_with_default \
        kl_rating_give library "Object feedback given"
    ad_call_proc_if_exists incentives::register_charge_category \
        -key_usage key_with_default \
        -value_meaning factor \
        kl_rating_receive library "Object feedback received"
    ad_call_proc_if_exists incentives::register_charge_category \
        -key_usage key_with_default \
        kl_object_publish library "Objects published"
    
    # Register library as a module which can be used in communities.
    ad_call_proc_if_exists sn_community::register_package library library f f

    # Register library as a module using category trees
    ad_call_proc_if_exists categories::register_package library
}

# Register sweeper proc for outdated knowledge objects
ad_schedule_daily -thread t 3 17 km_sweeper_daily

ad_schedule_proc -once t 10 reset_access_counts_monthly

# clear sweeper tmp table if sweeper didn't finish
ad_schedule_daily 21 0 km_clean_sweeper_tmp_table

#schedule emails for workflow tasks
nsv_set library library_notifications_p 0
ad_schedule_proc -thread t 300 library_approval_process_notifications

ad_schedule_daily -thread t 3 15 purge_old_paths
