ad_page_contract {
    /packages/library/www/admin/create-object-type-2.tcl

    Create a new KM library object type.

    @author Carsten Clasohm (carsten@arsdigita.com)
    @author Dirk Gomez (dirk@arsdigita.com)

    @cvs-id $Id: create-object-type-2.tcl,v 1.2 2005/03/01 00:01:15 jeffd Exp $
} {
    dc_signature
    pretty_name:trim,notnull
    pretty_plural:trim,notnull
    browse_p:oneof(0|1)
    public_p:oneof(0|1)
    create_p:oneof(0|1)
    archive_p:oneof(0|1)
    copy_p:oneof(0|1)
    sweeper:oneof(none|outdated|expired)
    sweeper_action:oneof(private|archive)
    sweeper_warning_time:integer
    sweeper_outdated_time:naturalnum
    {default_age_filter:integer 365}
    {short_name:trim ""}
    graphic:optional
    graphic.tmpfile:optional
}
csrf::authenticate

if ![empty_string_p $short_name] {
    set context_id [ad_conn package_id]
    if [db_0or1row get_short_name {
            select 1 from sn_object_types
            where context_id=:context_id
	      and short_name=:short_name
	      and rownum=1 }] {
        ad_return_complaint 1 "<li>Short name: $short_name is already used in this library instance."
	return
    }
}

if [empty_string_p $short_name] {
    set short_name [db_null]
}

if {[empty_string_p $default_age_filter]} {
    set default_age_filter 365
}

if {[doubleclick::check_all]} {
    ad_returnredirect index
    return
}

set browse_p [ad_decode $browse_p 1 t f]
set public_p [ad_decode $public_p 1 t f]
set create_p [ad_decode $create_p 1 t f]
set archive_p [ad_decode $archive_p 1 t f]
set copy_p [ad_decode $copy_p 1 t f]

if {$copy_p == "f" && [string equal $sweeper "archive"]} {
    set sweeper "none"
}

if { [empty_string_p $default_age_filter] || ![integer_p $default_age_filter] } {
    set default_age_filter 365
}

set context_id [ad_conn package_id]
set user_id [ad_conn user_id]
set creation_ip [ad_conn peeraddr]

db_transaction {
    set object_type_id [db_exec_plsql create_object_type {
	begin
	:1 := object_type.insert_object_type (
	    v_pretty_name => :pretty_name,
	    v_pretty_plural => :pretty_plural,
	    v_browse_p => :browse_p,
	    v_default_age_filter => :default_age_filter,
	    v_creation_user => :user_id,
	    v_creation_ip => :creation_ip,
	    v_short_name => :short_name,
	    v_context_id => :context_id);
	end;
    }]

    db_dml set_create_p_flag {
	update sn_object_types
	set create_p = :create_p,
           public_p = :public_p,
            archive_p = :archive_p,
            copy_p = :copy_p,
            sweeper = :sweeper,
            sweeper_action = :sweeper_action,
            sweeper_warning_time = :sweeper_warning_time,
            sweeper_outdated_time = :sweeper_outdated_time
	where object_type_id = :object_type_id
    }

    if [info exists graphic] {
	# Get the uploaded graphic, if there was one.
	if { [regexp {\.([a-zA-Z]+)$} $graphic match graphic_ext] } {
	    set filename "$object_type_id.$graphic_ext"

	    ns_cp [ns_queryget graphic.tmpfile] \
		"[ns_info pageroot]/library-files/$filename"

	    db_dml update_graphic {
		update sn_object_types
		set graphic = :filename, graphic_p = 't'
		where object_type_id = :object_type_id
	    }
	}
    }
}

##km_static -reset

ad_returnredirect "."
