ad_page_contract {
    /packages/library/www/admin/edit-branch.tcl

    @author Carsten Clasohm (carsten@arsdigita.com)
    @author Dirk Gomez (dirk@arsdigita.com)
    
    @cvs-id $Id: edit-branch.tcl,v 1.1 2003/07/02 12:19:44 peterm Exp $
} {
    object_type_id 
    question_id
    {operator ""}
    {branch_id 0}
} -properties {
    message:onevalue
    link:onevalue
    form_vars:onevalue
    abstract_data_type:onevalue
}

set values {}
set items {}
set link ""
set question [km_get_question $question_id $object_type_id]
set pretty_name [snd $question]
set abstract_data_type [thd $question]
set default_branch [km_default_branch $question_id]

set orig_question_id $question_id
set orig_pretty_name $pretty_name

if { $branch_id } {
    # Then we need to get the data that applies to this branch so we can show it on the form.

    set this_branch [km_get_branches -branch_id $branch_id -question_attributes_p 1 $question_id $object_type_id]
    set key [fst $this_branch]
    set this_branch [snd $this_branch]
    set this_branch_answer [snd $this_branch]
    set branch_operator [lindex $this_branch [lsearch $key branch_operator]]
    if { [lsearch {"==" "<" ">" } $branch_operator] < 0 } { set branch_operator "==" }

    set message "Edit this branch by changing the answer or the question that it leads to."
    set title "Edit Branch"

    if { $branch_id == $default_branch } {
	set link "
<p><a href=\"unset-default-branch?[export_url_vars question_id branch_id object_type_id]\">Stop
using this as the default branch.</a>"
    } else {
	set link "
<p><a href=\"set-default-branch?[export_url_vars question_id branch_id object_type_id]\">Make
this the default branch.</a>"
    }
} else {

    set attributes ""
    set this_branch_answer ""
    set branch_operator "="
    set title "New Branch"
    set message "To create a branch, define an answer to $pretty_name that will
             lead to the following question."
    set link ""
}


# Get the branch answers for category and option types so that we
# only offer the user a choice of branch answers that have not yet
# already been mapped

set branch_answers [snd [transpose [tail [km_get_branches -question_attributes_p 1 -branch_id $branch_id $question_id $object_type_id]]]]


set form_vars [export_form_vars question_id object_type_id]

switch $abstract_data_type {
    "category" - "other_category" {
	# Get the categories for this question
	set categories [km_get_child_categories $question_id]
	
	set values {}
	set items {}
	foreach item $categories {
	    set value [fst $item]
	    set item  [snd $item]

	    if { [lsearch $branch_answers $value] == -1 || $this_branch_answer == $value } {
		lappend values $value
		lappend items $item
	    }
	}
	if [null_p $values] {
	    ad_return_complaint 1  "There are no answers available for a new branch."
	    return
	}
    }
    "option" {
	# Get the answer options for this question
	set options [km_get_answer_options $question_id]
	
	foreach item $options {
	    set value [fst $item]
	    set item  [snd $item]

	    if { [lsearch $branch_answers $value] < 0 || $this_branch_answer == $value } {
		lappend values $value
		lappend items $item
	    }
	}
	if [null_p $values] {
	    ad_return_complaint 1  "There are no answers available for a new branch."
	    return
	}
    }
    "integer" {
        set values_list [list == < >]
        set items_list [list Equals {Less Than} {Greater Than}]
    }
    "default" {
        #Can't deal with any other abstract_data_types here
        ad_returnredirect "view-questions?[export_url_vars object_type_id]"
    }
}
  
# Get a list of questions that exist for this object_type 
# that are not already in a branch of this object_type.

set root_question_id [km_get_root_branch $question_id $object_type_id]
if ![empty_string_p $root_question_id] {
    set question_list [km_branch_questions -branch_id $branch_id -root_question_id $root_question_id -composite_children_p 0 $object_type_id]
} else {
    set question_list [km_branch_questions -branch_id $branch_id -root_question_id $question_id -composite_children_p 0 $object_type_id]
}

set branch_ids [list $branch_id]
set key [fst $question_list]
set question_list [tail $question_list]

template::multirow create questions pretty_name short_explanation question_state abstract_data_type presentation_type checkmark checked_string question_id url_vars 

foreach question $question_list {
    set pretty_name [lindex $question [lsearch $key pretty_name]]
    
    # Cut the entry explanation a bit smaller.
    set short_explanation [km_shorten_question [lindex $question [lsearch $key entry_explanation]]]
    
    set question_id [lindex $question [lsearch $key "question_id"]]
    set mandatory_p [ad_decode [lindex $question [lsearch $key mandatory_p]] t 1 0]
    set this_abstract_data_type [km_pretty_adt [lindex $question [lsearch $key "abstract_data_type"]]]
    set presentation_type [km_pretty_tag [lindex $question [lsearch $key "presentation_type"]]]
    set question_state [lindex $question [lsearch $key "question_state"]]

    if { $mandatory_p } {
        set checkmark "<img src=\"../graphics/check.gif\" width=16 height=16 border=0>"
    } else {
        set checkmark "&nbsp;"
    }
    if { [lsearch $branch_ids $question_id] >= 0 } { 
        set checked_string "checked"
    } else {
        set checked_string ""
    }
    
    set url_vars [export_url_vars object_type_id question_id]
    template::multirow append questions $pretty_name $short_explanation $question_state \
      $this_abstract_data_type $presentation_type $checkmark $checked_string $question_id $url_vars 
}
    
set_context_bar_data \
    [list "." "Knowledge Library"] \
    [list "view-questions?object_type_id=$object_type_id" "All Questions"]  \
    [list "view-branches?[export_url_vars question_id object_type_id]" "All Branches"] "$title for $pretty_name"

set object_type_name [db_string view_branches_20 "select pretty_name
  from sn_object_types 
  where object_type_id=:object_type_id"]
set_context_bar_data [list "view-questions?object_type_id=$object_type_id" "$object_type_name Questions"] [list "view-branches?question_id=$orig_question_id&object_type_id=$object_type_id" "Navigational Branches for $orig_pretty_name"] $title
ad_return_template
