ad_page_contract {
    /packages/library/www/admin/edit-object-type-2.tcl

    @author Carsten Clasohm (carsten@arsdigita.com)
    @author Dirk Gomez (dirk@arsdigita.com)
    
    @cvs-id $Id: edit-object-type-2.tcl,v 1.1 2003/07/02 12:19:44 peterm Exp $
} {
    pretty_name
    pretty_plural
    browse_p:oneof(0|1)
    public_p:oneof(0|1)
    create_p:oneof(0|1)
    archive_p:oneof(0|1)
    copy_p:oneof(0|1)
    sweeper:oneof(none|outdated|expired)
    sweeper_action:oneof(private|archive)
    sweeper_warning_time:integer
    sweeper_outdated_time:naturalnum
    object_type_id:integer
    default_age_filter:integer
    {short_name:trim ""}
    {graphic ""}
    {warned_p 0}
}
csrf::authenticate

set i 0
if {[empty_string_p $pretty_name]} {
    append complaint "<li>Please enter a presentation name."
    incr i
}

if {[empty_string_p $pretty_plural]} {
    append complaint "<li>Please enter a plural form of the presentation name."
    incr i
}

if {[empty_string_p $sweeper_warning_time]} {
    set sweeper_warning_time 0
}

if {![empty_string_p $short_name]} {
    set context_id [ad_conn package_id]
    if [db_0or1row get_short_name {
            select 1 from sn_object_types
            where context_id=:context_id
	      and short_name=:short_name
	      and rownum=1
	      and not object_type_id=:object_type_id }] {
        append complaint "<li>Short name: $short_name is already used in this library instance."
        incr i
    }
}

if {[empty_string_p $short_name]} {
    set short_name [db_null]
}

if { $i } {
    ad_return_complaint $i $complaint
    return
}

if { $public_p && $browse_p && !$warned_p } {
    # Check if the object type has a short_description.
    set short_description [db_list get_short_description_list {
        select short_description
        from sn_types_map_short_name
        where object_type_id=:object_type_id
        order by position}]
        
    if {[empty_string_p $short_description]} {
	ad_return_error "Missing short description" "
The object type is going to be public and browsable, 
but has no assigned short description.<br>
Please <a href=\"view-questions?[export_url_vars object_type_id]\">select
a question</a> and click on the link <cite>Make
this question the short description</cite> on the question's properties page."
	return
    }
}

set browse_p [ad_decode $browse_p 1 t f]
set public_p [ad_decode $public_p 1 t f]
set create_p [ad_decode $create_p 1 t f]
set archive_p [ad_decode $archive_p 1 t f]
set copy_p [ad_decode $copy_p 1 t f]

if {$copy_p == "f" && [string equal $sweeper "archive"]} {
    set sweeper "none"
}

# Get the uploaded graphic, if there was one
if { [regexp {\.([a-zA-Z]+)$} $graphic match graphic_ext] } {
    set filename "$object_type_id.$graphic_ext"

    ns_cp [ns_queryget graphic.tmpfile] \
	"[ns_info pageroot]/library-files/$filename"

    set graphic_update ", graphic = :filename, graphic_p = 't'"
} else {
    set graphic_update ""
}
 

if { [empty_string_p $default_age_filter] } { set default_age_filter 365 }
 
db_dml update_object_type "
    update sn_object_types 
    set pretty_name = :pretty_name, pretty_plural = :pretty_plural, short_name = :short_name,
        browse_p = :browse_p, public_p=:public_p, create_p = :create_p,
        archive_p = :archive_p, copy_p = :copy_p, sweeper = :sweeper,
        sweeper_action = :sweeper_action,
        sweeper_warning_time = :sweeper_warning_time,
        sweeper_outdated_time = :sweeper_outdated_time,
        default_age_filter=:default_age_filter 
	$graphic_update
    where object_type_id = :object_type_id"

# Invalidate global cache.
km_static -reset $object_type_id

ad_returnredirect "."
