ad_page_contract {
    /packages/library/www/admin/edit-question.tcl

    Edit data associated with one object type's question.

    @author Carsten Clasohm (carsten@arsdigita.com)
    @author Dirk Gomez (dirk@arsdigita.com)
    
    $Id: edit-question.tcl,v 1.1 2003/07/02 12:19:44 peterm Exp $
} {
    object_type_id
    {question_id 0}
} -properties {
}

set package_id [ad_conn package_id]
set pretty_type [km_static object_type_pretty_name $object_type_id]
set pretty_plural [km_static object_type_pretty_plural $object_type_id]
set graphic [km_static object_type_graphic $object_type_id]

set adts [km_adt_list -pretty_names_p 1]
set abstract_data_types [fst [transpose $adts]]
set pretty_data_types [snd [transpose $adts]]

set special_type_1_output_p f
set special_type_2_output_p f
set special_type_3_output_p f
set default_date_output_p f
set refers_to_output_p f
set date_correct_p f
set warning_output_p f
set state_output_p f
set order_output_p f
set appear_first_form_output_p f
set short_id ""
set long_id ""
set linked_question_id ""
set default_date_output ""
set default_date ""
set date_correct_output ""
set max_categories_p f
set browseable_p f
set is_public_until_p f
set is_start_date_p f
set is_end_date_p f

if {$question_id} {
    set edit_p 1

    # Get all existing data for this question
    set attributes [km_get_questions -question_states {} -all_properties_p 1 -root_node_p 0 -question_ids [list $question_id]]
    set key [fst $attributes]
    set question [snd $attributes]
    
    set pretty_name [lindex $question [lsearch $key "pretty_name"]]
    set entry_explanation [lindex $question [lsearch $key "entry_explanation"]]
    set mandatory_p [ad_decode [lindex $question [lsearch $key "mandatory_p"]] t 1 0]
    set form_number [ad_decode [lindex $question [lsearch $key "form_number"]] 1 1 0]
    set abstract_data_type [lindex $question [lsearch $key "abstract_data_type"]]
    set presentation_type [lindex $question [lsearch $key "presentation_type"]]
    set question_state [lindex $question [lsearch $key "question_state"]]
    set order_by [lindex $question [lsearch $key "order_by"]]
    set default_value [lindex $question [lsearch $key "default_value"]]
    set branch_p [ad_decode [lindex $question [lsearch $key "branch_p"]] "t" 1 0]
    set target_object_type_id [lindex $question [lsearch $key "target_object_type_id"]]
    set defaults_question_id [lindex $question [lsearch $key "defaults_question_id"]]
    set max_categories [lindex $question [lsearch $key "max_categories"]]
    set browse_p [lindex $question [lsearch $key "browse_p"]]
    set references_question_id [lindex $question [lsearch $key "references_question_id"]]
    set root_branch_p [km_root_branch_p $question_id]
    set parent_question_id [lindex $question [lsearch $key "parent_question_id"]]

    if { $parent_question_id == [km_static km_dummy_object_id] || $branch_p } {
	set composite_p 0
    } else {
	set composite_p 1
    }

    if { ([lsearch {object_link child_object nephew_object} $abstract_data_type] != -1)
       && [empty_string_p $target_object_type_id] } {
	ad_returnredirect "choose-object-type?[export_url_vars object_type_id question_id abstract_data_type]"
	return
    }

    if {[lsearch [list "category" "other_category" "option"] $abstract_data_type] > -1} {
	## psn: maximum number of categories selectable
	set max_categories_p t
    }

    if {[lsearch [list "category" "other_category"] $abstract_data_type] > -1} {
	## category browseable?
	set browseable_p t
    }

    if {[lsearch [list "text" "date" "integer" "category" "other_category"] $abstract_data_type] > -1} {
	## psn: child and nephew questions can automatically get the defaults
	## from linked parent/uncle questions
	db_multirow defaults_questions get_possible_defaults_questions {
	    select ot.pretty_name as object_type, ot.object_type_id,
	    q2.pretty_name as question_name, q2.question_id
	    from sn_questions q, sn_question_object_type_map qotm, sn_object_types ot,
	    sn_questions q2, sn_question_object_type_map qotm2
	    where qotm.question_id = q.question_id
	    and qotm.question_state = 'active'
	    and q.abstract_data_type in ('child_object','nephew_object')
	    and q.target_object_type_id = :object_type_id
	    and ot.object_type_id = qotm.object_type_id
	    and ot.context_id = :package_id
	    and qotm2.object_type_id = ot.object_type_id
	    and qotm2.question_state = 'active'
	    and qotm2.question_id = q2.question_id
	    and q2.abstract_data_type = :abstract_data_type
	}
    } else {
	template::multirow create defaults_questions object_type object_type_id question_name question_id
    }

} else {

    set edit_p 0
    set question_id [km_new_acs_object_id]
    set pretty_name ""
    set entry_explanation ""
    set mandatory_p 0
    set form_number 0
    set presentation_type ""
    set question_state "active"
    set order_by ""
    set default_value ""
    set branch_p 0
    set root_branch_p 0
    set composite_p 0
    set references_question_id 0
    set target_object_type_id 0
}

# Do some display things differently in the case of an edit
if {$edit_p} {

    set title "Edit Question: $pretty_name"
    set context_title $pretty_name
    set short_id [km_static object_type_short_description $object_type_id]
    set long_id [km_static object_type_long_description $object_type_id]
    set public_until_id [km_static object_type_public_until $object_type_id]
    set start_date_id [km_static object_type_start_date $object_type_id]
    set end_date_id [km_static object_type_end_date $object_type_id]
    set linked_question_id [km_static object_type_linked_question_id $object_type_id]

    if { $question_state == "active" && !$branch_p } {
	if { [lsearch {text file integer date user_link object_link content_link} $abstract_data_type] != -1 && 
	     [lsearch $short_id $question_id]==-1 && $question_id != $long_id &&
	     $question_id != $linked_question_id } {

	    # Show a link to set this question as the short/long description or link list.

	    if { $abstract_data_type == "object_link" } {
		set special_type_1_output_p t
	    } else {
		set special_type_2_output_p t
	    }
	}
	if { $abstract_data_type == "date" } {
	    set special_type_3_output_p t
	}
    }

    if {$root_branch_p} {
	set warning_output_p t
    }
} else {
    set title "New Question"
    set context_title $title
}

if { $edit_p && [lsearch $short_id $question_id]==-1 && $question_id != $long_id && !$branch_p } {
    if { $abstract_data_type == "composite" } {
	# For composite questions, we don't support the read-only state.
	set state_values {active deprecated invisible}
	set state_items {{Active} {To be phased out} {Invisible}}
    } else {
	set state_values {active deprecated read-only invisible}
	set state_items {{Active} {To be phased out} {Read-Only} {Invisible}}
    }
    set state_output_p t
}

if {$edit_p} {
    if { $abstract_data_type == "object_link" || $branch_p || $composite_p } {
	if !$form_number { set form_number 1 }
	append footnote [export_form_vars form_number]
    } else {
	set appear_first_form_output_p t
    }

    if { [lsearch {object_link child_object content_link nephew_object} $abstract_data_type] != -1 } {
	set order_output_p t
        set order_output_values [list "name" "name desc" "last_modified" "last_modified desc" "link_id" "link_id desc"]
        set order_output_items [list "Name" "Name desc." "Last modified" "Last modified desc." "Link date" "Link date desc."]
    }

    if { $abstract_data_type == "object_link" } {
	set refers_to_list [db_list_of_lists edit_question_10 "
	select q.question_id, ot.pretty_name || ' - ' || q.pretty_name as references_object_question
	from sn_questions q, sn_question_object_type_map qm, sn_object_types ot
            where q.target_object_type_id = :object_type_id
		and q.abstract_data_type = 'object_link'
                and q.question_id = qm.question_id
		and qm.question_state <> 'invisible'
		and qm.object_type_id = :target_object_type_id
                and ot.object_type_id = qm.object_type_id"]
    
        set refers_to_values "null"
        set refers_to_strings "None"
        foreach list_element $refers_to_list {
            lappend refers_to_values [lindex $list_element 0]
            lappend refers_to_strings [lindex $list_element 1]
        }

	set refers_to_output_p t
    }

    if { $abstract_data_type == "date" } {
	set default_date_output_p t
	if {![empty_string_p $default_value]} {
	    set db [ns_db gethandle]
	    if { ![catch {set default_date [db_string edit_question_19 "select $default_value from dual"]}] } {
		set date_correct_p t
		set date_correct_output "[util_AnsiDatetoPrettyDate $default_date]"
	    } else {
		set date_correct_p f
	    }
	}
    }
} else {
    set appear_first_form_output_p t
    set form_number 1
}

if {$branch_p} {
    set parent_id [km_branch_parent $question_id $object_type_id]

    append footnote "
<p>Note: This question is part of a branch. Therefore, you cannot change its state.
If you want to do so, you first have to remove it from the <a href=\"view-branches?question_id=$parent_id&object_type_id=$object_type_id\">branch tree</a>."
}


set branch_p [ad_decode $branch_p 1 t f]
set edit_p [ad_decode $edit_p 1 t f]
set form_vars [export_form_vars question_id object_type_id abstract_data_type]
set url_vars [export_url_vars question_id object_type_id abstract_data_type]
set_context_bar_data [list "view-questions?object_type_id=$object_type_id" "$pretty_type Questions"] $title

if {$short_id == $question_id} {
    set is_short_p t
    set is_part_of_short_p f
} else {
    set is_short_p f
    if {[lsearch $short_id $question_id]==-1} {
        set is_part_of_short_p f
    } else {
        set is_part_of_short_p t
    }   
}

if {$edit_p} {
    if {$public_until_id == $question_id} {
	set is_public_until_p t
    }
    if {$start_date_id == $question_id} {
	set is_start_date_p t
    }
    if {$end_date_id == $question_id} {
	set is_end_date_p t
    }
}

set dc_export [doubleclick::signature_html]
set csrf_link [csrf::link_token]

ad_return_template
