ad_page_contract {
    /packages/library/www/admin/index.tcl

    @author Carsten Clasohm (carsten@arsdigita.com)
    @author Dirk Gomez (dirk@arsdigita.com)
    
    $Id: view-questions.tcl,v 1.1 2003/07/02 12:19:45 peterm Exp $
} {    
    object_type_id
    {invisible_p 0}
} -properties {
    object_type_id:onevalue
    context_bar:onevalue
    branch_p:onevalue
    invisible_p:onevalue
    questions:multirow
    exported_url_vars:onevalue
    exported_form_vars:onevalue
    choice_bar:onevalue
    form_vars:onevalue
    url_vars:onevalue
    input_count:onevalue
    graphic:onevalue
}

# Object type info for page display
set pretty_type [km_static object_type_pretty_name $object_type_id]
set graphic [km_static object_type_graphic $object_type_id]

if { $invisible_p } {
    # Allow all question states.
    set question_states {}
} else {
    set question_states {active deprecated read-only}
}

set question_items [km_get_questions -question_states $question_states -root_node_p 0 -all_properties_p 1 -object_type_id $object_type_id]
set key [fst $question_items]
set question_items [tail $question_items]

set short_description [km_static object_type_short_description $object_type_id]
set long_description [km_static object_type_long_description $object_type_id]
set public_until [km_static object_type_public_until $object_type_id]
set start_date [km_static object_type_start_date $object_type_id]
set end_date [km_static object_type_end_date $object_type_id]
set linked_question_id [km_static object_type_linked_question_id $object_type_id]

template::multirow create questions question_id pretty_name abstract_data_type presentation_type question_state mandatory_check pretty_question_state action sort_key edit_question_link

set input_count 0

foreach question $question_items {
    set question_id [lindex $question [lsearch $key "question_id"]]
    set pretty_name [lindex $question [lsearch $key "pretty_name"]]
    set abstract_data_type [lindex $question [lsearch $key "abstract_data_type"]]
    set presentation_type [lindex $question [lsearch $key "presentation_type"]]
    set sort_key [lindex $question [lsearch $key "sort_key"]]
    set mandatory_check [ad_decode [lindex $question [lsearch $key "mandatory_p"]] "t" "checked" ""]
    set question_state [lindex $question [lsearch $key "question_state"]]
    set pretty_question_state [km_pretty_question_state $question_state]
    set branch_p [ad_decode [lindex $question [lsearch $key "branch_p"]] "t" 1 0]
 
    if { $question_id == $short_description } { 
        append pretty_name " <sup><font size=-2>1</font></sup>" 
    } else {
        if { [lsearch $short_description $question_id]>=0} { append pretty_name " <sup><font size=-2>1</font></sup>" }
    }
    if { $question_id == $long_description } { append pretty_name " <sup><font size=-2>2</font></sup>" }
    if { $question_id ==  $public_until } { append pretty_name " <sup><font size=-2>3</font></sup>" }
    if { $question_id == $linked_question_id } { append pretty_name " <sup><font size=-2>4</font></sup>" }
    if { $question_id ==  $start_date } { append pretty_name " <sup><font size=-2>5</font></sup>" }
    if { $question_id ==  $end_date } { append pretty_name " <sup><font size=-2>6</font></sup>" }
	
    # Find out if there already is a branch leading from this question
    if { [km_composite_child_p $question_id $object_type_id] } {
	set branch_link ""
    } elseif { [km_root_branch_p $question_id] } {
	set branch_link "&nbsp;| <a href=\"view-branches?[export_url_vars question_id object_type_id]\"><font size=-1>Branches</font></a>"
    } elseif { $question_state == "active" } {
	set branch_link "&nbsp;| <a href=\"edit-branch?[export_url_vars question_id object_type_id]\"><font size=-1>Branches</font></a>"
    } else {
	set branch_link ""
    }

    # Certain abstract data types lead to certain actions
    switch $abstract_data_type {
	"integer" {
	    set action "&nbsp;|
<a href=\"choose-presentation-type?[export_url_vars question_id object_type_id]\"><font 
size=-1>Presentation Type</font></a>$branch_link"
	}
	"text" {
	    set action "&nbsp;|
<a href=\"choose-presentation-type?[export_url_vars question_id object_type_id]\"><font 
size=-1>Presentation Type</font></a>"
	}
	"category" - "other_category" {
	    set cat_target "href=\"choose-category?[export_url_vars question_id object_type_id]\""
	    set action "&nbsp;|
<a href=\"choose-presentation-type?[export_url_vars question_id object_type_id]\"><font 
size=-1>Presentation Type</font></a>$branch_link&nbsp;|
<a $cat_target><font size=-1>Categories</font></a>"
	}
	"option" { 
	    set action "&nbsp;|
<a href=\"choose-presentation-type?[export_url_vars question_id object_type_id]\"><font
size=-1>Presentation Type</font></a>$branch_link &nbsp;|
<a href=\"view-options?[export_url_vars question_id object_type_id]\"><font 
size=-1>Options</font></a>"
	}
	"composite" { 
	    set action "&nbsp;|
<a href=\"view-composite?[export_url_vars question_id object_type_id]\"><font 
size=-1>Questions</font></a>"
	}
	"date" {
	    set action "&nbsp;|
<a href=\"choose-presentation-type?[export_url_vars question_id object_type_id]\"><font 
size=-1>Presentation Type</font></a>"
	}
	"nephew_object" {
	    set action "&nbsp;|
<a href=\"choose-presentation-type?[export_url_vars question_id object_type_id]\"><font 
size=-1>Presentation Type</font></a>"
	}
	"default" {
	    set action ""
	}
    }

    if [empty_string_p $presentation_type] { set presentation_type "Default" }
    
    if { $question_state != "invisible" } {
	set question_state_tag "<input type=text size=4 name=$question_id value=\"$sort_key\">"
	incr input_count
    } else {
	set question_state_tag "&nbsp;"
    }

    if { $branch_p } { set pretty_question_state "Branch" }

    set mandatory_check_tag "<input type=checkbox name=mandatory_p value=$question_id $mandatory_check>"
    set edit_question_link "<a href=\"edit-question?[export_url_vars object_type_id question_id]\"><font size=-1>Properties</font></a>"
    template::multirow append questions $question_id $pretty_name [km_pretty_adt $abstract_data_type] [km_pretty_tag $presentation_type] $question_state_tag $mandatory_check_tag $pretty_question_state $action $sort_key $edit_question_link
}

set branch_p 0

set title "$pretty_type Questions"

set choice_bar [ad_choice_bar {"Active, To be phased out, Read-Only" "All"} [list "view-questions?[export_url_vars object_type_id question_id]&invisible_p=0" "view-questions?[export_url_vars object_type_id question_id]&invisible_p=1"] {0 1} $invisible_p]

set form_vars [export_form_vars object_type_id]
set url_vars [export_url_vars object_type_id]

set_context_bar_data $title

ad_return_template
