! Entering browse-one-category
ad_page_contract {
    /packages/library/www/browse-one-category.tcl

    Show the subcategories and objects associated with a category.

    @cvs-id $Id: browse-one-category.tcl,v 1.1 2003/07/02 12:19:42 peterm Exp $
} {
    {object_type_id:integer 0}
    {object_type ""}
    {category_id 0}
    {category ""}
    {question_id:integer 0}
    {path_id:integer ""}
    {show short}
    {pool all}
    {age ""}
    {sort_by last_modified}
    {first_row:integer 1}
    {last_row:integer 0}
    {begin_letter all}
    {archive "f"}
    start:array,optional
    end:array,optional
} -properties {
    object_list:multirow
    title:onevalue
    graphic_width:onevalue
    category_action:onevalue
    category_id:onevalue
    total_num_objects:onevalue
    num_displayed_objects:onevalue
    category_display_limit:onevalue
    pretty_question:onevalue
    object_type_id:onevalue
    category_id:onevalue
    path_id:onevalue
    system_name:onevalue
    age_bar:onevalue
    ownership_bar:onevalue
    archive_bar:onevalue
    object_view_bar_top_level:onevalue
    object_sort_bar:onevalue
    objects:onevalue
    first_row:onevalue
    last_row:onevalue
    alphabet:multirow
    begin_letter:onevalue
    subsite_url:onevalue
    subsite_name:onevalue
    default_age_filter:onevalue
    archive:onevalue
    archive_p:onevalue
    start_date:onevalue
    end_date:onevalue
    date_filter_vars:onevalue
    date_filter_p:onevalue
}

set package_id [ad_conn package_id]
set form [ad_conn form]
# For backward compatibility, we still support the object_type parameter.

if { [empty_string_p $object_type] && !$object_type_id } {
    ad_return_warning "Missing object type" \
	"Either object_type or object_type_id have to be given."
    return
}

if [empty_string_p $show] {
    set show short
}

if !$object_type_id {
    set object_type_id [db_string object_type_id {
	select object_type_id from sn_object_types
	where short_name = :object_type
	and context_id = :package_id
	and rownum = 1
    } -default ""]
    if {[empty_string_p $object_type_id]} {
	ad_return_complaint 1 "No such object type found."
	return
    }
    # manipulate the form data so to export object_type_id
    # via export_ns_set_vars
    ns_set delkey $form object_type
    ns_set put $form object_type_id $object_type_id
}

if { $category_id != "none" && !$category_id } {
    set category_id [db_string category_id {
	select c.category_id
	from categories c, sn_questions q, sn_question_object_type_map qm,
             sw_category_dim d
	where c.short_name = :category
	and qm.object_type_id = :object_type_id
	and qm.question_id = q.question_id
	and qm.question_state <> 'invisible'
	and d.parent_node_id = q.node_id
	and c.category_id = d.object_id
	and rownum = 1
    } -default ""]
    if {[empty_string_p $category_id]} {
	ad_return_complaint 1 "No such category found."
	return
    }
    # manipulate the form data so to export category_id
    # via export_ns_set_vars
    ns_set delkey $form category
    ns_set put $form category_id $category_id
}

set child_type_p [db_string child_object_type {
    select decode(count(*),0,0,1) from dual where exists
    (select 1
     from sn_questions
     where abstract_data_type = 'child_object' 
	and target_object_type_id = :object_type_id)
}]

set user_id [ad_conn user_id] 
if ![km_check_object_type_id $object_type_id] { return }

if ![info exists start] {
    set start_date ""
} else {
    set start_date "$start(year)-$start(month)-$start(day)"
    if ![date_p $start_date] {
	ad_return_complaint 1 "Invalid start date"
	return
    }
}
if ![info exists end] {
    set end_date ""
} else {
    set end_date "$end(year)-$end(month)-$end(day)"
    if ![date_p $end_date] {
	ad_return_complaint 1 "Invalid end date"
	return
    }
}

set archive_p [km_static object_type_archive_p $object_type_id]
if {!$archive_p} {
    set archive f
}

if [empty_string_p $path_id] {
    set current_action browse
} else {
    set current_action [get_path_action $path_id]
}

set default_age_filter [km_static object_type_default_age_filter $object_type_id]

if [empty_string_p $age] {
    # First, we check if there is a default age filter for this object
    # type. If not, we use "all" as our default.

    set age $default_age_filter
    if [empty_string_p $age] { set age all }
}

set return_uri [ad_urlencode "[ad_conn url]?[ad_conn query]"]
set date_filter_vars [export_ns_set_vars form {first_row last_row start.day start.month start.year end.day end.month end.year} $form]
if {![empty_string_p [km_static object_type_start_date $object_type_id]] && ![empty_string_p [km_static object_type_end_date $object_type_id]]} {
    set date_filter_p t
} else {
    set date_filter_p f
}

# Get the name of the object type.
set pretty_type [km_static object_type_pretty_name $object_type_id]
set pretty_type_plural [km_static object_type_pretty_plural $object_type_id]
if {$archive=="t"} {
    set pretty_type_plural "$pretty_type_plural (archived)"
}
set graphic [km_static object_type_graphic $object_type_id]
# Get the name of the category

if { $category_id == "none" } {

    set have_alert 0
    set category "Uncategorized"

    if !$question_id {
	ad_return_error "Deep trouble" "
	We're looking for uncategorized things, but we weren't
	told which question should be uncategorized. Probably our programming error."
	return
    }

    set node_id 0
    set root_node [km_root_node -question_id $question_id $object_type_id]
    set pretty_question [snd [km_get_question $question_id $object_type_id]]

} else {

    set have_alert [alerts::exists -category_id $category_id -content_type_id $object_type_id]
    set category [km_category_name $category_id]

    if { [empty_string_p $category] } {
	ad_return_error "Deep trouble" \
	    "We're in deep trouble, because we couldn't find category_id $category_id"
	return
    }

    set node_id [db_string node_id "
	select node_id from sw_category_dim where object_id=:category_id" -default ""] 
    set parent_info [km_node_question_parent $object_type_id $node_id]
    set question_id [fst $parent_info]
    set root_node [snd $parent_info]
    set pretty_question [snd [km_get_question $question_id $object_type_id]]
}
# Check if the corresponding question has been made invisible.

set question_info [km_get_question $question_id $object_type_id]
if [empty_string_p $question_info] {
    ad_return_warning "Invalid Category" "
	The given categorization does not exist, probably because it has been removed
	from this object type. Please go back to the 
        <a href=\"browse-one-type?[export_ns_set_vars url {category_id first_row last_row show} $form]\">browsing page</a>
	for $pretty_type_plural."
    return
}

set parameters [export_ns_set_vars url {first_row last_row category_id question_id show} $form]

if { $category_id == "none" } {
    set_context_bar_data [list "browse-one-type?$parameters" $pretty_type_plural] "Uncategorized by $pretty_question"
} else {

    set context_list [list]

    lappend context_list [list "browse-one-type?$parameters" $pretty_type_plural]

    if {!$root_node} {
	set parent_info [km_category_question_parent $object_type_id $category_id]
	set root_category [snd $parent_info]
    } else {
        set root_category $root_node
    }

    set parentage_list [ad_category_parentage_list $category_id $root_category]

    set ancestor_count [llength $parentage_list]

    # We don't want to show a link for the current category.
    incr ancestor_count -1 
    for { set i 1 } { $i < $ancestor_count } { incr i } {
	set parent [lindex $parentage_list $i]
	set parent_id [fst $parent]
	set parent_name [snd $parent]
	lappend context_list ["[ad_conn url]?$parameters&category_id=$parent_id" $parent_name]
    }

    lappend context_list [km_category_name $category_id]

    eval set_context_bar_data $context_list


}

array set node [site_node_closest_ancestor_site_node "acs-subsite"]
set subsite_url $node(url)
set subsite_name $node(instance_name)

set create_p [km_conn create_p]
set copy_p [km_static object_type_copy_p $object_type_id]
set object_type_pretty_name [km_static object_type_pretty_name $object_type_id]
set graphic_width [library_icon_width]

if {![km_static object_type_create_p $object_type_id]} {
    set create_p 0
}

if { [empty_string_p $category] } {
    set maybe_category ""
} else {
    set maybe_category "in $category"
}

set title "$pretty_type_plural $maybe_category"

! Getting count of objects in this category or subcategories

set category_display_limit [km_category_display_limit]
set category_counts [ns_set create]

if { $category_id == "none" } {

    km_get_category_counts -age $age -pool $pool -begin_letter $begin_letter -archived $archive -start_date $start_date -end_date $end_date $object_type_id $category_counts
    set total_num_objects [ns_set get $category_counts "uncat $question_id"]

} else {

    km_get_category_counts -age $age -pool $pool -begin_letter $begin_letter -archived $archive -start_category_id $category_id -start_date $start_date -end_date $end_date $object_type_id $category_counts
    set total_num_objects [ns_set get $category_counts $category_id]

}

! Got the counts +total_num_objects
    
set system_name [km_get_community_name]
set age_bar [km_age_bar $age]
set ownership_bar [km_ownership_bar $pool]

set target_url "[ns_conn url]?[export_ns_set_vars url {category_id} $form]&category_id="

km_category_table -object_type_id $object_type_id \
	-question_id $question_id -parent_node_id $node_id -target $target_url \
	-category_counts $category_counts -maximum_sub_columns 3 -uncategorized_p 0

set object_view_bar_top_level [km_object_view_bar_top_level $show "just this category" "this category and its subcategories"]
set object_sort_bar [km_sort_by_bar $sort_by $child_type_p]

set minus_join_tables {}
set where_clauses {}
set minus_where_clauses {}
set more_join_tables {}

if { $category_id == "none" } {

    lappend minus_join_tables \
	"sw_object_category_map cm" \
	"sw_flat_cat fc"

    lappend minus_where_clauses \
	"cm.object_id = obj.object_id" \
	"cm.category_id = fc.child_category_id" \
	"fc.parent = $root_node"

} else {

    switch $show {

	all {
            ! We are showing objects on +category_id level AND all sublevels
	    lappend where_clauses "obj.object_id = swcm.object_id"
	    lappend where_clauses "fc.parent_category_id = :category_id"
            lappend where_clauses "swcm.category_id = fc.child_category_id"

            lappend more_join_tables "sw_object_category_map swcm"
            lappend more_join_tables "sw_flat_cat fc"
	}

	default {
            ! We are showing just objects on +category_id level
	    set show "short"
	    lappend where_clauses "obj.object_id = swcm.object_id"
	    lappend where_clauses "swcm.category_id = :category_id"

            lappend more_join_tables "sw_object_category_map swcm"
	}
    }
}

if [empty_string_p $total_num_objects] { set total_num_objects 0 }

if !$last_row {
    if { $total_num_objects > [expr $first_row + 99] } {
	set last_row [expr $first_row + 99]
    }
} elseif { $last_row < $total_num_objects && $total_num_objects <= [expr $first_row + 99] } {
    set last_row $total_num_objects
}

set url_stub "browse-one-category?[export_ns_set_vars url {first_row last_row} $form]"

set num_displayed_objects 0
if { $total_num_objects > 0 } {
    set header $pretty_type_plural
    ! Calling km8_output_object_list +where_clauses +minus_join_tables +minus_where_clauses +more_join_tables
    set num_displayed_objects [km8_output_object_list -object_type_id $object_type_id \
	-header $header -pool $pool -age $age -sort_by $sort_by \
        -show $show -current_action $current_action -path_id $path_id \
        -category_id $category_id -question_id $question_id \
	-first_row $first_row -last_row $last_row \
        -where_clauses $where_clauses -begin_letter $begin_letter \
        -minus_join_tables $minus_join_tables \
	-minus_where_clauses $minus_where_clauses \
	-archived $archive -start_date $start_date -end_date $end_date \
        -more_join_tables $more_join_tables]
    ! Done with km8_output_object_list

    if { $last_row > $num_displayed_objects } { 
	set last_row $num_displayed_objects 
    } elseif !$last_row { 
	set last_row $num_displayed_objects 
    } 
}

set url_vars [export_ns_set_vars url {begin_letter first_row last_row} $form]
template::multirow create alphabet letter url
foreach letter {all A B C D E F G H I J K L M N O P Q R S T U V W X Y Z other} {
    if [string equal $letter $begin_letter] {
	template::multirow append alphabet $letter ""
    } else {
	template::multirow append alphabet $letter "browse-one-category?$url_vars&begin_letter=$letter"
    }
}

set archive_bar [km_archive_bar $archive]

query get_package_name package_name onevalue {
    select instance_name from apm_packages where package_id = :package_id
}
array set node [site_node_closest_ancestor_site_node "acs-subsite"] 
set community_id $node(package_id) 

if [integer_p $category_id] {
    query get_category_tree_info tree_info onerow {
       select gt.tree_name, gt.tree_id, c.short_name
         from sw_category_dim scd, generic_trees gt, categories c
        where scd.object_id = :category_id
          and scd.tree_id = gt.tree_id
          and c.category_id = :category_id
    }
    set search_contexts [list \
        [list "lcat:$object_type_id,$tree_info(tree_id),$category_id" "$tree_info(short_name)"] \
        [list "ltype:$object_type_id" "$pretty_type_plural"] \
        [list "lib:$package_id" "$package_name"] \
        [list "sc:$community_id" "$subsite_name"] \
                             [sws_km_site_search_context]]
} else {
    set search_contexts [list \
        [list "ltype:$object_type_id" "$pretty_type_plural"] \
        [list "lib:$package_id" "$package_name"] \
        [list "sc:$community_id" "$subsite_name"] \
                             [sws_km_site_search_context]]
}

ad_return_template
! Exiting browse-one-category
