! Entering browse-one-type
ad_page_contract {
    /packages/library/www/browse-one-type.tcl

    Show all categories associated with the given object type.

    @author Carsten Clasohm (carsten@arsdigita.com)
    @author Dirk Gomez (dirk@arsdigita.com)

    @cvs-id $Id: browse-one-type.tcl,v 1.1 2003/07/02 12:19:42 peterm Exp $
} {
    {object_type_id:integer 0}
    {short_name ""}
    {path_id:integer ""}
    {show ""}
    {pool all}
    {age ""}
    {sort_by last_modified}
    {first_row:integer 1}
    {last_row:integer 0}
    {begin_letter all}
    {archive "f"}
    start:array,optional
    end:array,optional
} -properties {
    title:onevalue
    graphic_width:onevalue
    total_num_objects:onevalue
    category_display_limit:onevalue
    pretty_type_plural:onevalue
    system_name:onevalue
    object_type_id:onevalue
    age_bar:onevalue
    path_id:onevalue
    ownership_bar:onevalue
    archive_bar:onevalue
    object_view_bar_top_level:onevalue
    show:onevalue
    object_sort_bar:onevalue
    objects:onevalue
    first_row:onevalue
    last_row:onevalue
    categorization_widget:multirow
    alphabet:multirow
    begin_letter:onevalue
    subsite_url:onevalue
    subsite_name:onevalue
    default_age_filter:onevalue
    search_contexts:onevalue
    archive:onevalue
    archive_p:onevalue
    start_date:onevalue
    end_date:onevalue
    date_filter_vars:onevalue
    date_filter_p:onevalue
}

set user_id [ad_conn user_id]
set package_id [ad_conn package_id]
set instance_name [ad_conn instance_name]
set form [ad_conn form]

if { [empty_string_p $short_name] && !$object_type_id } {
    ad_return_warning "Invalid object type" \
	"short_name or object_type_id are either missing or invalid."
    return
}

if !$object_type_id {
    set object_type_id [db_string object_type_id {
	select object_type_id
	from sn_object_types
	where short_name = :short_name
	and context_id = :package_id
	and rownum = 1
    } -default ""]
    # manipulate the form data so to export object_type_id
    # via export_ns_set_vars
    if {[empty_string_p $object_type_id]} {
	ad_return_complaint 1 "No such object type found."
	return
    }
    ns_set delkey $form short_name
    ns_set put $form object_type_id $object_type_id
}

if ![km_check_object_type_id $object_type_id] { return }

if ![info exists start] {
    set start_date ""
} else {
    set start_date "$start(year)-$start(month)-$start(day)"
    if ![date_p $start_date] {
	ad_return_complaint 1 "Invalid start date"
	return
    }
}
if ![info exists end] {
    set end_date ""
} else {
    set end_date "$end(year)-$end(month)-$end(day)"
    if ![date_p $end_date] {
	ad_return_complaint 1 "Invalid end date"
	return
    }
}

set archive_p [km_static object_type_archive_p $object_type_id]
if {!$archive_p} {
    set archive f
}

if ![path_valid_p $path_id] { 
    set current_action "browse"
} else {
    set current_action [get_path_action $path_id]
}

if [empty_string_p $pool] { set pool all }
if [empty_string_p $sort_by] { set sort_by last_modified }

set default_age_filter [km_static object_type_default_age_filter $object_type_id]

if [empty_string_p $age] {
    # First, we check if there is a default age filter for this object
    # type. If not, we use "all" as our default.

    set age $default_age_filter
    if [empty_string_p $age] { set age all }
}


set return_uri [ad_urlencode "[ad_conn url]?[ad_conn query]"]
set have_alert [alerts::exists -content_type_id $object_type_id]
set date_filter_vars [export_ns_set_vars form {first_row last_row start.day start.month start.year end.day end.month end.year} $form]
if {![empty_string_p [km_static object_type_start_date $object_type_id]] && ![empty_string_p [km_static object_type_end_date $object_type_id]]} {
    set date_filter_p t
} else {
    set date_filter_p f
}

# Get the pretty name of the object type

set pretty_type [km_static object_type_pretty_name $object_type_id]
set pretty_type_plural [km_static object_type_pretty_plural $object_type_id]
if {$archive=="t"} {
    set pretty_type_plural "$pretty_type_plural (archived)"
    set have_alert 1
}
set graphic [km_static object_type_graphic $object_type_id]
set age_bar [km_age_bar $age]

set child_type_p [db_string child_object_type {
    select decode(count(*),0,0,1) from dual where exists
    (select 1
     from sn_questions
     where abstract_data_type = 'child_object' and target_object_type_id=:object_type_id)
}]

# Get the category counts.

set category_counts [ns_set create]
km_get_category_counts -age $age -pool $pool -begin_letter $begin_letter -archived $archive -start_date $start_date -end_date $end_date $object_type_id $category_counts

set total_num_objects [ns_set get $category_counts total]
set category_display_limit [km_category_display_limit]

array set node [site_node_closest_ancestor_site_node "acs-subsite"]
set subsite_url $node(url)
set subsite_name $node(instance_name)
set subcommunity_id $node(package_id)

set graphic_width [library_icon_width]
set_context_bar_data -- $pretty_type_plural

set title $pretty_type_plural
set object_type_pretty_name [km_static object_type_pretty_name $object_type_id]
set system_name [km_get_community_name]
set ownership_bar [km_ownership_bar $pool]

if ![km_static object_type_create_p $object_type_id] {
    set create_p 0
} else {
    set create_p [km_conn create_p]
}
set copy_p [km_static object_type_copy_p $object_type_id]

set target_url "browse-one-category?[export_ns_set_vars url {category_id show first_row last_row} $form]&category_id="

set categories_p [km_category_table -object_type_id $object_type_id \
    -target $target_url -category_counts $category_counts]

if [empty_string_p $show] {
    if { !$categories_p || $total_num_objects < $category_display_limit } {
	set show all
    } else {
	set show short
    }
}

if { $show != "short" } {
    set object_sort_bar [km_sort_by_bar $sort_by $child_type_p]
} else {
    set object_sort_bar ""
}

if !$last_row {
    if { $total_num_objects > [expr $first_row + 99] } {
	set last_row [expr $first_row + 99]
    }
} elseif { $last_row < $total_num_objects && $total_num_objects <= [expr $first_row + 99] } {
    set last_row $total_num_objects
}

set url_stub "browse-one-type?[export_ns_set_vars url {first_row last_row} $form]"

set objects ""
set total_num_objects2 [km8_output_object_list \
    -object_type_id $object_type_id \
    -pool $pool -age $age -sort_by $sort_by \
    -show $show -path_id $path_id -current_action $current_action \
    -first_row $first_row -last_row $last_row -begin_letter $begin_letter \
    -archived $archive -start_date $start_date -end_date $end_date]

if !$last_row { set last_row $total_num_objects }

set url_vars [export_ns_set_vars url {begin_letter first_row last_row} $form]
template::multirow create alphabet letter url
foreach letter {all A B C D E F G H I J K L M N O P Q R S T U V W X Y Z other} {
    if [string equal $letter $begin_letter] {
	template::multirow append alphabet $letter ""
    } else {
	template::multirow append alphabet $letter "browse-one-type?$url_vars&begin_letter=$letter"
    }
}

set object_view_bar_top_level [km_object_view_bar_top_level $show \
    "just category descriptions" \
    "all $pretty_type objects ($total_num_objects)"]

set archive_bar [km_archive_bar $archive]

set search_contexts [list \
        [list "ltype:$object_type_id" "$pretty_type_plural"] \
        [list "lib:$package_id" "$instance_name"] \
        [list "sc:$subcommunity_id" "$subsite_name"] \
                         [sws_km_site_search_context]]

ad_return_template
! Exiting browse-one-type
