ad_page_contract {
    /packages/library/www/comment-add-2.tcl

    Let the user enter a comment and the number of reuse points.

    @author Carsten Clasohm (carsten@arsdigita.com)
    @author Bernd Schmeil (bernd@arsdigita.com) - did modifications
    @author Timo Hentschel (timo@arsdigita.com)

    @cvs-id $Id: comment-add-2.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    object_id:notnull,integer
    {feedback_category_id:integer ""}
    {category_id ""}
    {path_id:integer ""}
    {comment ""}
    {html_p:oneof(t|f) "f"}
    {public_p:optional "t"}
    {reuse_points:integer "-1"}
} -properties {
    title:onevalue
    headline:onevalue
    object_name:onevalue
    object_type:onevalue
    form_vars:onevalue
    feedback_category:onevalue
    feedback_category_id:onevalue
    ratings:multirow
    overview_with_href_and_mailto:onevalue
    author_id:onevalue
    author_name:onevalue
    subsite_url:onevalue
    subsite_name:onevalue
    header_stuff:onevalue
    comment:onevalue
    html_p:onevalue
    public_p:onevalue
    reuse_points:onevalue
}
	
set user_id [auth::require_login]
set package_id [ad_conn package_id]

array set node [site_node_closest_ancestor_site_node "acs-subsite"] 
set community_id $node(package_id) 

if ![km_check_object_id $object_id] { return }
set author_id [km_conn original_author_id]
set author_name [km_conn original_author_name]

if {$feedback_category_id == 0} {
    ad_return_complaint 1 "Please select a feedback category!"
    return
}

if {[km_conn archived_p]} {
    ad_return_complaint 1 "Only feedback for not archived knowledge objects is allowed!"
    return
}

set object_name [km_conn object_name]
set object_type [km_static object_type_pretty_name [km_conn object_type_id]]

array set node [site_node_closest_ancestor_site_node "acs-subsite"]
set subsite_url $node(url)
set subsite_name $node(instance_name)
set header_stuff "<SCRIPT LANGUAGE=\"JavaScript\" SRC=\"$subsite_url\guidelines/jscript/functions.js\" TYPE=\"text/javascript\"> </SCRIPT>"
 
if {![empty_string_p $feedback_category_id]} {
    set feedback_category [db_string get_category_name {
	select long_name from categories
	where category_id = :feedback_category_id} -default ""]

    # don't let user give feedback points if user is author
    # or object is private
    if {($user_id == $author_id) || ![km_conn public_p] || ![db_0or1row get_rating_scale {
        select rating_p as reuse_p, scale
        from sn_comments_category_settings
        where object_id = :package_id
        and category_id = :feedback_category_id }]} {
            set reuse_p "f"
            set scale 0
     }
} else {
    set reuse_p "f"
    set scale 0
    set feedback_category ""
}

if {$reuse_p=="t"} {
    set what "Give feedback"
    set headline "Give Feedback"
    set title "Give feedback for \"$object_name\""
} else {
    set what "Add comment"
    set headline "Your Comment"
    set title "Add a comment to \"$object_name\""
}

set_the_usual_klib_context_bar $object_id $category_id
if {[empty_string_p $feedback_category_id]} {
    append_context_bar_data $what
} else {
    append_context_bar_data [list "comment-add?[export_url_vars object_id category_id path_id]" "Feedback category"] $what
}

set overview_with_href_and_mailto [util_make_href_and_mailto_links [km_conn overview]]
set form_vars [export_form_vars object_id feedback_category_id category_id path_id]
set original_author_p [km_original_author_p $user_id $object_id]

template::multirow create ratings number
 
if { $reuse_p=="t" } {
    # Generate the list of valid reuse points.
    if { $scale > 0} {
        for { set counter 0 } { $counter <= $scale } { incr counter } {
            template::multirow append ratings $counter
        }
    }
}

ad_return_template

