ad_page_contract {
    /packages/library/www/comment-add-3.tcl

    Ask the user for confirmation before adding comment and reuse points.

    @author Carsten Clasohm (carsten@arsdigita.com)
    @author Bernd Schmeil (bernd@arsdigita.com) - did modifications
    @author Timo Hentschel (timo@arsdigita.com)

    @cvs-id $Id: comment-add-3.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    object_id:notnull,integer
    comment:notnull
    html_p:notnull,oneof(t|f)
    {reuse_points:integer "-1"}
    {feedback_category_id:integer ""}
    {public_p:optional "f"}
    {category_id ""}
    {path_id:integer ""}
} -properties {
    title:onevalue
    object_name:onevalue
    object_type:onevalue
    form_vars:onevalue
    feedback_category:onevalue
    comment:onevalue
    html_p:onevalue
    reuse_points:onevalue
    original_author_p:onevalue
    author_id:onevalue
    author_name:onevalue
    dc_export:onevalue
    subsite_url:onevalue
    subsite_name:onevalue
    own_shares:multirow
    author_shares:multirow
    admin_p:onevalue
}
	
set user_id [auth::require_login]
set package_id [ad_conn package_id]

if ![km_check_object_id $object_id] { return }
set author_id [km_conn original_author_id]
set author_name [km_conn original_author_name]
set admin_p [km_conn admin_p]

if {$user_id == $author_id} {
    set reuse_points -1
}
 
if {[km_conn archived_p]} {
    ad_return_complaint 1 "Only feedback for not archived knowledge objects is allowed!"
    return
}

set object_name [km_conn object_name]
set object_type [km_static object_type_pretty_name [km_conn object_type_id]]
set title "Confirm comment on \"$object_name\""

set form_vars [export_form_vars object_id comment html_p reuse_points feedback_category_id public_p category_id path_id]
set original_author_p [km_original_author_p $user_id $object_id]

set reuse_p "f"
if {![empty_string_p $feedback_category_id]} {
    set feedback_category [db_string get_category_name "
        select long_name from categories 
        where category_id = :feedback_category_id" -default ""]
    if {[db_0or1row get_rating_scale {
        select rating_p as reuse_p, scale
        from sn_comments_category_settings
        where object_id = :package_id
        and category_id = :feedback_category_id }] && $reuse_p == "t" } {
	    if {[empty_string_p $reuse_points] || $reuse_points < 0 || $reuse_points > $scale} {
		ad_return_complaint 1 "Please provide a rating!"
		return
	    }
    }
} else {
    set feedback_category ""
}

if {$reuse_p=="t"} {
    set what "Give feedback"
    set confirm "Confirm feedback"
} else {
    set what "Add comment"
    set confirm "Confirm comment"
}

set_the_usual_klib_context_bar $object_id $category_id
if [empty_string_p $feedback_category_id] {
    append_context_bar_data [list "comment-add-2?[export_url_vars object_id category_id path_id]" $what] $confirm
} else {
    append_context_bar_data [list "comment-add?[export_url_vars object_id category_id path_id]" "Feedback category"] [list "comment-add-2?[export_url_vars object_id category_id path_id feedback_category_id]" $what] $confirm
}

if { $html_p == "f" } {
    set comment [util_make_href_and_mailto_links_and_convert_to_html $comment]
}

template::multirow create own_shares bank_id bank_name currency amount balance
if {$reuse_points > -1 && ($user_id != $author_id) && ![empty_string_p $feedback_category_id] && [info exists incentives::initialized]} {
    foreach bank_id [incentives::bank_list $package_id] {
	set amount [incentives::charge_value -key $feedback_category_id kl_rating_give $bank_id]
	if {$amount > 0} {
	    db_1row get_currency_name {
		select pretty_name as bank_name, currency_name
		from ir_banks
		where bank_id = :bank_id
	    }
	    template::multirow append own_shares $bank_id $bank_name $currency_name $amount [expr $amount+[thd [incentives::account_balance $bank_id]]]
	}
    }
}

template::multirow create author_shares bank_id bank_name currency amount balance
if {$reuse_points > 0 && ($user_id != $author_id) && ![empty_string_p $feedback_category_id] && [info exists incentives::initialized]} {
    foreach bank_id [incentives::bank_list -user_id $author_id $package_id] {
	set factor [incentives::charge_value -key $feedback_category_id kl_rating_receive $bank_id]
	set amount [expr {$factor * $reuse_points}]
	if {$amount > 0} {
	    db_1row get_currency_name {
		select pretty_name as bank_name, currency_name
		from ir_banks
		where bank_id = :bank_id
	    }
	    template::multirow append author_shares $bank_id $bank_name $currency_name $amount [expr $amount+[thd [incentives::account_balance -user_id $author_id $bank_id]]]
	}
    }
}

array set node [site_node_closest_ancestor_site_node "acs-subsite"]
set subsite_url $node(url)
set subsite_name $node(instance_name)

set dc_export [doubleclick::signature_html]

ad_return_template
