ad_page_contract {
    /packages/library/www/comment-add-4.tcl

    Store comment and reuse points.

    @author Carsten Clasohm (carsten@arsdigita.com)
    @author Bernd Schmeil (bernd@arsdigita.com) - modifications done in Jan 2001

    @cvs-id $Id: comment-add-4.tcl,v 1.3 2005/03/01 00:01:14 jeffd Exp $
} {
    object_id:notnull,integer
    comment:notnull
    {html_p:notnull,oneof(t|f) "f"}
    {public_p:optional "f"}
    {reuse_points:integer "-1"}
    {feedback_category_id:integer ""}
    {category_id ""}
    {path_id:integer ""}
}
csrf::authenticate

set user_id [auth::require_login]
set package_id [ad_conn package_id]
if ![km_check_object_id $object_id] { return }
set author_id [km_conn original_author_id]

if {$user_id == $author_id} {
    set reuse_points -1
}
 
if {[km_conn archived_p]} {
    ad_return_complaint 1 "Only feedback for not archived knowledge objects is allowed!"
    return
}

set object_type [km_static object_type_pretty_name [km_conn object_type_id]]

set return_url "object-view?[export_url_vars object_id category_id path_id]"

if {[doubleclick::check_all]} {
    ns_sleep 2
    ad_returnredirect $return_url
    return
}

set category_forward_email ""
set scale 0

if {[db_0or1row get_rating_scale {
    select rating_p as reuse_p, scale, email as category_forward_email
    from sn_comments_category_settings
    where object_id = :package_id
    and category_id = :feedback_category_id
}] && $reuse_p == "t" } {
    if {[empty_string_p $reuse_points] || $reuse_points < 0 || $reuse_points > $scale} {
	ad_return_complaint 1 "Please provide a rating!"
	return
    }
}

db_transaction {
    # Save the comment.
    set comment_id [km_general_comment_add $object_id $user_id $html_p \
		$reuse_points $scale $feedback_category_id $public_p $comment]

    # add comment to yellow page
    db_dml insert_content_area {
	insert into acs_object_areas (object_id, area_id, package_id)  
                              values (:comment_id, null, :package_id)
    }

    set author_id [km_conn original_author_id]
    db_1row get_original_author_email {
	select email as author_email from users where user_id = :author_id
    }

    if {$comment_id && ($user_id != $author_id)} {
	if {$reuse_points > -1
	    && ![empty_string_p $feedback_category_id]
	    && [info exists incentives::initialized]
	} {
	    # NOTE: this code almost exactly the same as the code in
	    # bboard/www/comment-add-3.tcl.

	    # We have to award shares twice: to the user who received
	    # the rating, and to the user who gave the rating.

	    # Note that we do need to loop over the banks twice,
	    # because the two users may not be in the same groups.

	    # First handle the user who gives the rating.
	    foreach bank_id [incentives::bank_list [ad_conn package_id]] {
		set amount [incentives::charge_value -key $feedback_category_id kl_rating_give $bank_id]
		incentives::award_shares kl_rating_give $amount $bank_id $comment_id
	    }

	    # Then the user who gets the rating.  Check for 0 just so
	    # we don't bother with listing the banks and all if we're
	    # not going to award any shares in the end.
	    set shares_list [list]
	    set shares_awarded_p 0
	    if {$reuse_points > 0} {
		foreach bank_id [incentives::bank_list -user_id $author_id [ad_conn package_id]] {
		    set factor [incentives::charge_value -key $feedback_category_id kl_rating_receive $bank_id]
		    if {$factor > 0} {
			set amount [expr {$factor * $reuse_points}]
			incentives::award_shares -user_id $author_id \
				kl_rating_receive $amount $bank_id $comment_id
			db_1row get_currency_name {
			    select pretty_name as bank_name, currency_name
			    from ir_banks
			    where bank_id = :bank_id
			}
			set balance [thd [incentives::account_balance -user_id $author_id $bank_id]]
			lappend shares_list "$amount $currency_name (new total: $balance) for $bank_name ([ad_url]/incentives/user/shares?bank_id=$bank_id)"
			set shares_awarded_p 1
		    }
		}
		set shares_list [join $shares_list "\n"]
	    }
	}

	# Send email to object owner.

	set object_name [sn_striphtml [km_conn object_name]]

	db_1row object_info {
	    select u.email as sender_email, 
		u.first_names || ' ' || u.last_name as sender_name,
		site_node.url(sn.node_id) as package_url
	    from users u, site_nodes sn
	    where u.user_id = :user_id
		and sn.object_id = :package_id
	        and rownum = 1
	}

	if {![empty_string_p $feedback_category_id]} {
	    set feedback_category [db_string comment_add_4_20 {
		select long_name from categories
		where category_id = :feedback_category_id
	    } -default ""]
	} else {
	    set feedback_category "Comment"
	}

	if { $html_p == "t" } { set comment [ad_html_to_text -- $comment] }

	set replacement_list \
	    [list \
		 knowledge_object	    $object_name \
		 knowledge_object_link      [ad_url]${package_url}object-view?object_id=$object_id \
		 knowledge_object_type      $object_type \
		 feedback_type		    $feedback_category \
		 recipient_email	    $author_email \
		 sender_email	            $sender_email \
		 sender_name		    $sender_name \
		 comment		    $comment]

	if {$reuse_points == -1} {
	    set template_name "km_object_comment"
	} else {
	    set template_name "km_object_feedback"
	    lappend replacement_list points $reuse_points
	    if {$shares_awarded_p} {
		set template_name "km_object_feedback_shares"
		lappend replacement_list shares_list $shares_list
	    }
	}

	# This uses sn_community_instances_all.  Should it be using
	# sn_community_mounts_all instead?
	set community_id [db_string get_community_id {
	    select community_id
	    from sn_community_instances_all
	    where package_id = :package_id
	}]

	et_queue_email [et_process -community_id $community_id $template_name $replacement_list]

	if { ![empty_string_p $category_forward_email] } {
            set replacement_list \
		[list \
		     knowledge_object	    $object_name \
		     knowledge_object_link  [ad_url]${package_url}object-view?object_id=$object_id \
		     knowledge_object_type  $object_type \
		     feedback_type	    $feedback_category \
		     recipient_email	    $category_forward_email \
		     sender_email	    $sender_email \
		     sender_name	    $sender_name \
		     comment		    $comment]
 
            et_queue_email [et_process -community_id $community_id "km_object_feedback_notify" $replacement_list]
        }
    }
}

ad_returnredirect $return_url
