ad_page_contract {
    /packages/library/www/comment-add.tcl

    Let the user choose a category for his feedback.

    @author Carsten Clasohm (carsten@arsdigita.com)
    @author Timo Hentschel (timo@arsdigita.com)

    @cvs-id $Id: comment-add.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    object_id:notnull,integer
    {feedback_category_id:integer ""}
    {category_id ""}
    {path_id:integer ""}
} -properties {
    title:onevalue
    object_name:onevalue
    object_type:onevalue
    headline:onevalue
    overview_with_href_and_mailto:onevalue
    form_vars:onevalue
    categories:multirow
    author_id:onevalue
    author_name:onevalue
    feedback_category_id:onevalue
    public_p:onevalue
}

set user_id [auth::require_login]
set package_id [ad_conn package_id]

array set node [site_node_closest_ancestor_site_node "acs-subsite"] 
set community_id $node(package_id) 

if ![km_check_object_id $object_id] { return }
set author_id [km_conn original_author_id]
set author_name [km_conn original_author_name]
set public_p [km_conn public_p]

if {[km_conn archived_p]} {
    ad_return_complaint 1 "Only feedback for not archived knowledge objects is allowed!"
    return
}

set object_name [km_conn object_name]
set object_type [km_static object_type_pretty_name [km_conn object_type_id]]

set title "Give feedback for \"$object_name\""
set headline "Give feedback"
set_the_usual_klib_context_bar $object_id $category_id
append_context_bar_data "Feedback category"
set overview_with_href_and_mailto [util_make_href_and_mailto_links [km_conn overview]]

set form_vars [export_form_vars object_id path_id category_id]

set feedback_category_tree_assigned_p [db_0or1row feedback_category_tree {
    select gt.tree_name, octr.subtree_root_node_id, gt.root_node_id as original_root_node_id
    from generic_trees gt,
    acs_rels ar,
    object_category_tree_rels octr
    where ar.object_id_one = :package_id
    and   ar.object_id_two = gt.tree_id
    and   ar.rel_id = octr.rel_id
    and rel_type = 'object_category_tree_rel'}] 

if { !$feedback_category_tree_assigned_p } {
    ad_returnredirect "comment-add-2?[export_url_vars object_id path_id category_id]"
    return
} else {
    # get feedback categories
    if { $subtree_root_node_id == $original_root_node_id } {
        set level_correction 1
    } else {
        set level_correction 0
    }
 
    template::multirow create categories name category_id level

    set sql_query "
        select /*+INDEX(sw_category_dim sw_category_dim_long_name) */ category.long_name(object_id) as name,
               object_id as category_id, deprecated_p, node_id,
               level as sortlevel
        from  sw_category_dim
        start with node_id = :subtree_root_node_id
        connect by prior node_id = parent_node_id"

    # don't allow feedback categories with feedback points
    # if user is author of the object or the object is private
    if {($user_id == $author_id) || !$public_p} {
	set sql_query "
	    select cat.name, cat.category_id, cat.deprecated_p,
	           cat.node_id, cat.sortlevel
	    from sn_comments_category_settings ccs,
	         ($sql_query) cat
	    where ccs.object_id = :package_id
	    and ccs.category_id (+) = cat.category_id
	    and nvl(ccs.rating_p,'f') = 'f'"
    }

    db_foreach  get_one_tree $sql_query {
        if { [string equal $deprecated_p "f"]} {
            if { $node_id!=$subtree_root_node_id || $subtree_root_node_id != $original_root_node_id } {
                template::multirow append categories $name $category_id [gt_repeat_string "&nbsp;" [expr ($sortlevel - 1 -$level_correction) * 5]]
            }
        }
    }
}

ad_return_template
