ad_page_contract {
    /packages/library/www/comment-edit-2.tcl

    Commits the feedback changes to the db.

    @author Timo Hentschel (timo@arsdigita.com)

    @cvs-id $Id: comment-edit-2.tcl,v 1.3 2005/03/01 00:01:14 jeffd Exp $
} {
    comment_id:notnull,integer
    comment:notnull
    {html_p:oneof(t|f) "f"}
    {feedback_category_id:integer ""}
    {public_p:optional "f"}
    {category_id ""}
    {path_id:integer ""}
}
csrf::authenticate

set user_id [auth::require_login]
set package_id [ad_conn package_id]
set object_id [km_check_comment_id $user_id $comment_id]
if !$object_id { return }

set admin_p [km_conn admin_p]
set object_name [sn_striphtml [km_conn object_name]]
set object_type [km_static object_type_pretty_name [km_conn object_type_id]]

set category_forward_email ""
set scale 0

db_0or1row get_rating_scale {
    select scale, email as category_forward_email
    from sn_comments_category_settings
    where object_id = :package_id
    and category_id = :feedback_category_id
}
 
db_1row get_message_data {
    select u.email as feedback_author_email, u.user_id as feedback_author_id
    from sn_comments sc, users u
    where sc.comment_id = :comment_id
    and u.user_id = sc.user_id }

if {!$admin_p && (($user_id != $feedback_author_id) || [km_conn archived_p])} {
    ad_return_forbidden "Only the feedback author or admins can edit feedback!"
    return
}
 
set return_url "object-view?[export_url_vars object_id category_id path_id]"

db_transaction {

    km_general_comment_update -html_p $html_p -public_p $public_p -feedback_category_id $feedback_category_id $feedback_author_id $object_id $comment_id $comment
 
    if { $html_p == "t" } { set comment [ad_html_to_text -- $comment] }
 
    set author_id [km_conn original_author_id]
    set author_email [km_conn original_author_email]

    db_1row object_info {
	select u.email as sender_email, 
	u.first_names || ' ' || u.last_name as sender_name,
	site_node.url(sn.node_id) as package_url
	from users u, site_nodes sn
	where u.user_id = :user_id
	and sn.object_id = :package_id
	and rownum = 1
    }

    if {![empty_string_p $feedback_category_id]} {
	set feedback_category [db_string comment_edit_2_20 {
	    select long_name from categories
	    where category_id = :feedback_category_id
	} -default ""]
    } else {
	set feedback_category ""
    }

    # This uses sn_community_instances_all.  Should it be using
    # sn_community_mounts_all instead?
    set community_id [db_string get_community_id {
        select community_id
        from sn_community_instances_all
        where package_id = :package_id
    }]
 
    if { $user_id != $author_id } {
	set replacement_list \
	    [list \
		 knowledge_object	$object_name \
		 knowledge_object_link  [ad_url]${package_url}object-view?object_id=$object_id \
		 knowledge_object_type  $object_type \
		 feedback_type		$feedback_category \
		 recipient_email	$author_email \
		 sender_email		$sender_email \
		 sender_name		$sender_name \
		 comment		$comment]
 
	et_queue_email [et_process -community_id $community_id "km_object_feedback_edit" $replacement_list]
    }

    if { $admin_p && $feedback_author_id != $user_id } {
        # we have to sent the feedback author an email that an admin changed
        # his feedback

        set replacement_list \
	    [list \
		 knowledge_object	    $object_name \
		 knowledge_object_link      [ad_url]${package_url}object-view?object_id=$object_id \
		 knowledge_object_type      $object_type \
		 feedback_type		    $feedback_category \
		 recipient_email	    $feedback_author_email \
		 sender_email	            $sender_email \
		 sender_name		    $sender_name \
		 comment		    $comment]
 
        et_queue_email [et_process -community_id $community_id "km_object_feedback_edit_admin" $replacement_list]
    }
}
 
ad_returnredirect $return_url
