ad_page_contract {
    /packages/library/www/comment-edit.tcl

    Allow the user to edit one of his general comments.

    @author Carsten Clasohm (carsten@arsdigita.com)
    @author Timo Hentschel (timo@arsdigita.com)

    @cvs-id $Id: comment-edit.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    comment_id:notnull,integer
    {category_id ""}
    {path_id:integer ""}
} -properties {
    title:onevalue
    object_name:onevalue
    object_id:onevalue
    objec_type:onevalue
    overview_with_href_and_mailto:onevalue
    content:onevalue
    rating:onevalue
    scale:onevalue
    admin_p:onevalue
    feedback_category_id:onevalue
    feedback_category:onevalue
    public_p:onevalue
    categories:multirow
    form_vars:onevalue
    user_id:onevalue
    author_id:onevalue
    subsite_url:onevalue
    subsite_name:onevalue
    header_stuff:onevalue
}

set user_id [auth::require_login]
set package_id [ad_conn package_id]

array set node [site_node_closest_ancestor_site_node "acs-subsite"] 
set community_id $node(package_id) 

set object_id [km_check_comment_id $user_id $comment_id]
if { !$object_id } { return }

set object_name [km_conn object_name]
set object_type [km_static object_type_pretty_name [km_conn object_type_id]]

set author_id [km_conn original_author_id]
set author_name [km_conn original_author_name]

set admin_p [km_conn admin_p]

array set node [site_node_closest_ancestor_site_node "acs-subsite"]
set subsite_url $node(url)
set subsite_name $node(instance_name)
set header_stuff "<SCRIPT LANGUAGE=\"JavaScript\" SRC=\"$subsite_url\guidelines/jscript/functions.js\" TYPE=\"text/javascript\"> </SCRIPT>"

if {$admin_p} {
    set title "Edit feedback for \"$object_name\""
    set header "Edit feedback"
} else {
    set title "Edit comment on \"$object_name\""
    set header "Edit comment"
}

set_the_usual_klib_context_bar $object_id $category_id
append_context_bar_data $header
set overview_with_href_and_mailto [util_make_href_and_mailto_links [km_conn overview]]

template::multirow create categories category_id name deprecated_p level

db_1row get_comment_data {
    select sc.content, sc.html_p, rating, scale, c.category_id as feedback_category_id,
           c.long_name as feedback_category, sc.display_p as public_p, sc.user_id as feedback_author_id
    from sn_comments sc, categories c
    where comment_id=:comment_id
    and sc.category_id = c.category_id(+) }

if {!$admin_p && (($user_id != $feedback_author_id) || [km_conn archived_p])} {
    ad_return_forbidden "Only the feedback author or admins can edit feedback!"
    return
}
 
if { [string eq $html_p "f"] } {
    set content [ad_quotehtml $content]
}

set feedback_object_id 0
set form_vars [export_form_vars comment_id category_id path_id feedback_category_id]
 
if { $admin_p } {
    #assuming that only one tree(subtree) is assigned to this object
    
    set feedback_category_tree_assigned_p [db_0or1row feedback_category_tree {
	select gt.tree_name, octr.subtree_root_node_id, gt.root_node_id as original_root_node_id
	from generic_trees gt,
	acs_rels ar,
	object_category_tree_rels octr
	where ar.object_id_one = :package_id
	and   ar.object_id_two = gt.tree_id
	and   ar.rel_id = octr.rel_id
	and rel_type = 'object_category_tree_rel'}]

    if {$feedback_category_tree_assigned_p} {
        db_foreach get_one_tree_edit {
            select  /*+INDEX(sw_category_dim sw_category_dim_long_name) */ category.long_name(object_id) as name,
            category.sort_key(object_id) as sort_key,
            object_id as fb_category_id,  deprecated_p, level
            from  sw_category_dim
            start with node_id = :subtree_root_node_id
            connect by prior node_id = parent_node_id
        } {
            template::multirow append categories $fb_category_id $name $deprecated_p $level
        }

	set form_vars [export_form_vars comment_id category_id path_id]
    }
}

ad_return_template
