# Given an object_type_id or a question_id, this proc returns a
# create/edit form.  You will only see values for a given object if you
# pass its object_id and set show_values_p 1. You set question_id if you
# just want to see one question.  Important!!! YOU have to put in the
# form tags around this html.

if {![info exists object_id]} {
    set object_id 0
}

if {![info exists category_id]} {
    set category_id ""
}

if {![info exists show_values_p]} {
    set show_values_p 0
}

if {![info exists all_types_p]} {
    set all_types_p 0
}

if {![info exists form_view]} {
    set form_view 1
}

if {![info exists question_ids]} {
    set question_ids {}
}

if {![info exists object_type_id]} {
    set object_type_id 0
}

if {![info exists user_id]} {
    set user_id 0
}

if {![info exists edit_p]} {
    set edit_p 0
}

if {![info exists path_id]} {
    set path_id ""
}

if {![info exists fields_only_p]} {
    set fields_only_p 0
}

if {![info exists mandatory_p]} {
    set mandatory_p 0
}

if {![info exists ancestor_id]} {
    set ancestor_id 0
}

if { $object_id && $show_values_p } {
    set question_states {active deprecated read-only}
    if {$all_types_p} {
	lappend question_states invisible
    }
} else {
    set question_states {active}
}

if ![null_p $question_ids] {
    set questions [km_get_questions -question_states $question_states -question_ids $question_ids -root_node_p 0 -all_properties_p 1]
} elseif { $object_type_id } {
    if { $mandatory_p } { set form_view "" }
    set questions [km_get_questions -question_states $question_states -form_view $form_view -branch_children_p 0 -mandatory_p $mandatory_p -all_properties_p 1 -object_type_id $object_type_id]
} else { 
    return
}

if { $object_id && $show_values_p } {
    # try to display some values - the proc get_object_data expects a *keyed* list
    set values [km_get_object_data -answers_only_p 1 -questions $questions $object_id]
} else {
    set values {}
}

# Separate the key list and the questions list.
set key [head $questions]
set questions [tail $questions]

set counter 0
set html ""
set question_id_ix [lsearch $key "question_id"]
set abstract_data_type_ix [lsearch $key "abstract_data_type"]
set pretty_name_ix [lsearch $key "pretty_name"]
set presentation_type_ix [lsearch $key "presentation_type"]
set entry_explanation_ix [lsearch $key "entry_explanation"]
set height_ix [lsearch $key "tag_height"]
set width_ix [lsearch $key "tag_width"]
set mandatory_p_ix [lsearch $key "mandatory_p"]
set question_state_ix [lsearch $key "question_state"]
set defaults_question_id_ix [lsearch $key "defaults_question_id"]

if !$user_id { set user_id [ad_get_user_id] }
set return_url "[ns_conn url]?[ns_conn query]"

template::multirow create fields object_id pretty_name abstract_data_type question_state entry_explanation height width value html_p presentation_type question_id question_state items item_values option_ids new_path_id descriptions option_ids option_answers other_value mandatory_p

foreach question $questions {

    set value [lindex $values $counter]
    set question_id [lindex $question $question_id_ix]
    set abstract_data_type [lindex $question $abstract_data_type_ix]
    set pretty_name [lindex $question $pretty_name_ix]
    set presentation_type [lindex $question $presentation_type_ix]
    set entry_explanation [lindex $question $entry_explanation_ix]
    set height [lindex $question $height_ix]
    set width [lindex $question $width_ix]
    set mandatory_p [lindex $question $mandatory_p_ix]
    set question_state [lindex $question $question_state_ix]
    set defaults_question_id [lindex $question $defaults_question_id_ix]
    set items ""
    set item_values ""
    set option_ids ""
    set new_path_id ""
    set descriptions ""
    set html_p 0
    set option_ids ""
    set option_answers ""
    set other_value ""
    set other_value_def ""
    incr counter

    # if a default for this question should be taken from ancestor object,
    # get the default if no value has been provided yet
    if {([empty_string_p $value]) && ($ancestor_id  > 0) && ($defaults_question_id > 0)} {
	switch $abstract_data_type {
	    "text" - "integer" - "date" {
		db_0or1row get_default_content {
		    select content as value, html_p
		    from sn_content
		    where question_id = :defaults_question_id
		    and object_id = :ancestor_id
		}
		if {[string equal $abstract_data_type "text"]} {
		    set value [list $value $html_p]
		}
	    }
	    "category" {
		set value [km_get_categories $ancestor_id $defaults_question_id]
	    }
	    "other_category" {
		set value [km_get_categories $ancestor_id $defaults_question_id]
		db_0or1row get_default_other_value_field {
		    select content as other_value_def
		    from sn_content
		    where question_id = :defaults_question_id
		    and object_id = :ancestor_id
		}
	    }
	}
    }

    if { $question_state != "active" && ![km_answered_p $abstract_data_type $value] } {
        # Don't show deprecated or read-only questions which haven't been answered yet.
        continue 
    }

    if { $mandatory_p == "t" } { 
        set pretty_name "$pretty_name <span id=\"snattention\">*</span>" 
    }

    # Call the proc that know how to display this abstract_data_type.
    switch $abstract_data_type {
        "date" {
            set value [fst $value] 
        }
        "object_link" {
            set path_vars [set_path_values -question_id $question_id $object_id $category_id]
            set new_path_id [init_path -last_path_id $path_id $path_vars $return_url link]
        }
        "option" {
            # Get the answer options for this question
            util_unlist [transpose [km_get_answer_options $question_id]] \
		    option_ids option_answers
            
            set value [fst [transpose $value]]
            if {![empty_string_p $value]} {
                #pass the list of checked items 
                set item_values [fst [transpose $value]]
            }
        }
        "category" {
            # Get the categories for this question
	    set item_values {}
	    set items {}
	    set descriptions {}
	    set show_descriptions_p 0
            foreach category [km_get_child_categories $question_id] {
		util_unlist $category category_id category_name description node_id level

		lappend item_values $category_id
		lappend items "[gt_repeat_string "&nbsp;" [expr $level*4]]$category_name"
		lappend descriptions $description

		if { ![empty_string_p $description] } { set show_descriptions_p 1 }
	    }
	    if { !$show_descriptions_p } { set descriptions "" }

            if { ![null_p $value] } { set value [fst [transpose $value]] }
        } 
        "other_category" {
            # Get the categories for this question
	    set item_values {}
	    set items {}
	    set descriptions {}
	    set show_descriptions_p 0
            foreach category [km_get_child_categories $question_id] {
		util_unlist $category category_id category_name description node_id level

		lappend item_values $category_id
		lappend items "[gt_repeat_string "&nbsp;" [expr $level*4]]$category_name"
		lappend descriptions $description

		if { ![empty_string_p $description] } { set show_descriptions_p 1 }
	    }
	    if { !$show_descriptions_p } { set descriptions "" }


	    set other_value [snd [fst [km_get_object_content -question_ids [list $question_id] $object_id]]]
	    if [empty_string_p $other_value] { set other_value $other_value_def }

            if { ![null_p $value] } { set value [fst [transpose $value]] }
        } 
        "text" {
            set html_p [snd $value]
            set value [fst $value] 
        }
    }

    template::multirow append fields $object_id $pretty_name $abstract_data_type $question_state $entry_explanation $height $width $value $html_p $presentation_type $question_id $question_state $items $item_values $option_ids $new_path_id $descriptions $option_ids $option_answers $other_value $mandatory_p
}
