# Takes the keyed lists from km_get_linked_objects and
# organizes them into HTML lists sorted by object_type.
  
# Note that edit_links_p and delete_links_p display edit/delete
# hyperlinks for *all* links. We do not check permissions.

if {![info exists edit_links_p]} {
    set edit_links_p 0
}

if {![info exists delete_links_p]} {
    set delete_links_p 0
}

if {![info exists display_object_types_p]} {
    set display_object_types_p 0
}

if {![info exists display_inline_object_types_p]} {
    set display_inline_object_types_p 0
}

if {![info exists display_questions_p]} {
    set display_questions_p 0
}

if {![info exists question_id]} {
    set question_id 0
}

if {![info exists category_id]} {
    set category_id ""
}

if {![info exists return_url]} {
    set return_url ""
}

# This page is used by admin/view-sample-form,
# so km_conn may return empty strings.
set admin_p [km_conn admin_p]
set archived_p [km_conn archived_p]
set write_p [km_conn write_p]

if [empty_string_p $admin_p] { 
    set admin_p [ad_permission_p [ad_conn package_id] "admin"] 
}

if [empty_string_p $archived_p] { set archived_p 0 }
if [empty_string_p $write_p] { set admin_p 0 }

if { $archived_p && !$admin_p } {
    set edit_links_p 0
    set delete_links_p 0
}

set source_object_type_id [km_get_object_type $object_id]
set key [head $linked_object_list]
set linked_objects [tail $linked_object_list]
if [null_p $linked_objects] return

if { $question_id && [llength $linked_objects] > 1 } {
    
    set order_by [km_static question_order_by $question_id]

    proc lindex2 {idx xs} { return [lindex $xs $idx] }
    switch $order_by {
        "name desc" { set linked_objects [reverse $linked_objects] }
        "last_modified" -
        "last_modified desc" {
            set last_modified_ix [lsearch $key last_modified]
            set linked_objects [qsort $linked_objects "lindex2 $last_modified_ix"]
            if { $order_by == "last_modified desc" } {set linked_objects [reverse $linked_objects] }
        }
        "link_id" -
        "link_id desc" {
            set link_id_ix [lsearch $key link_id]
            set linked_objects [qsort $linked_objects "lindex2 $link_id_ix"]
            if { $order_by == "link_id desc" } {set linked_objects [reverse $linked_objects] }
        }
        default {}
    }
}

set linked_object_id_ix [lsearch $key linked_object_id]
set name_ix [lsearch $key name]
set overview_ix [lsearch $key overview]
set overview_html_p_ix [lsearch $key overview_html_p]
set link_comment_ix [lsearch $key link_comment]
set comment_html_p_ix [lsearch $key comment_html_p]
set linked_object_type_ix [lsearch $key linked_object_type]
set question_id_ix [lsearch $key question_id]
set public_p_ix [lsearch $key public_p]
set archived_p_ix [lsearch $key archived_p]
set review_p_ix [lsearch $key review_p]
set from_p_ix [lsearch $key from_p]
set owner_id_ix [lsearch $key owner_id]
set owner_name_ix [lsearch $key owner_name]

set result ""
set last_question_id 0

template::multirow create linked_objects_list object_type_id pretty_type_plural graphic overview link_comment public_p archived_p review_p linked_object_name may_view_p owner_html question from_p object_type_pretty_name present_owner_html edit_links_p delete_links_p link_id linked_object_id url_vars

foreach item $linked_objects {
    set linked_object_id [lindex $item $linked_object_id_ix]
    set linked_object_name [lindex $item $name_ix]
    set overview [lindex $item $overview_ix]
    set overview_html_p [lindex $item $overview_html_p_ix]
    set link_comment [lindex $item $link_comment_ix]
    set comment_html_p [lindex $item $comment_html_p_ix]
    set object_type_id [lindex $item $linked_object_type_ix]
    set question_id [lindex $item $question_id_ix]
    set public_p [lindex $item $public_p_ix]
    set archived_p [lindex $item $archived_p_ix]
    set review_p [lindex $item $review_p_ix]
    set from_p [lindex $item $from_p_ix]
    set owner_id [lindex $item $owner_id_ix]
    set owner_name [lindex $item $owner_name_ix]

    set pretty_type_plural [km_static object_type_pretty_plural $object_type_id]
    set graphic [km_static object_type_graphic $object_type_id]
    
    # Establish the permissions + publishing facts
    set edit_p $write_p
    set view_p [lindex $item [lsearch $key view_p]]
    if [empty_string_p $view_p] { set view_p 0 }

    if {![empty_string_p $link_comment]} {
        if {$comment_html_p} {
            set link_comment "([util_make_href_and_mailto_links $link_comment])"
        } else {
            set link_comment "([util_make_href_and_mailto_links [ad_quotehtml $link_comment]])"
        }
    }

    if {[empty_string_p $linked_object_name]} {
        # Use the default "Unnamed <pretty_object_type>" label.
        set linked_object_name [km_get_object_name $linked_object_id]
    }
    
    if { $public_p || $view_p || $edit_p } {
        set may_view_p t
        set owner_html ""
    } else {
        set may_view_p f
        set owner_html [ad_present_user $owner_id "the owner"]
    }
    set linked_object_name [sn_striphtml $linked_object_name]

    set question ""
    if { $display_questions_p && ![empty_string_p $question_id] && $last_question_id != $question_id } {
        if { $from_p } {
            set question_ot_id $source_object_type_id
        } else {
            set question_ot_id $object_type_id
        }

        set question_info [km_get_question -entry_explanation_p 1 $question_id $question_ot_id]
        set explanation [lindex $question_info 4]
        set short_explanation [km_shorten_question $explanation]
        if ![empty_string_p $short_explanation] {
            set question "$short_explanation"
        }
        set last_question_id $question_id
    }

    set present_owner_html [ad_present_user $owner_id $owner_name]
    if { $public_p || $view_p || $edit_p } {
	if { [string length $overview] > 200 } {
            set overview "[string range $overview 0 200]..."
        }
    } else {
        set overview ""
    }

    set link_id [lindex $item [lsearch $key link_id]]
    set url_vars [export_url_vars object_id category_id link_id return_url]

    template::multirow  append linked_objects_list $object_type_id $pretty_type_plural $graphic $overview $link_comment $public_p $archived_p $review_p $linked_object_name $may_view_p $owner_html $question $from_p [km_static object_type_pretty_name $object_type_id] $present_owner_html $edit_links_p $delete_links_p $link_id $linked_object_id $url_vars

}
