ad_page_contract {
    /www/library/object-access-add.tcl

    Add a user or group to the object's access control list.
    List all linked objects and let the user select the ones that should
    be assigned the same permissions.

    @cvs-id $Id: object-access-add.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    object_id:notnull,integer
    {category_id ""}
    {view_p:boolean 1}
    {edit_p:boolean 1}
    {user_id_from_search:integer,optional}
    {group_id:integer,optional}
} -properties {
    title:onevalue
    form_vars:onevalue
    object_id:onevalue
    links:multirow
}
csrf::authenticate

set user_id [auth::require_login]
if ![km_check_object_id -check_edit_p 1 $object_id] { return }

set object_type_id [km_conn object_type_id]
set object_name [km_conn object_name]
set graphic [km_static object_type_graphic $object_type_id]

set title "Access control for \"$object_name\""
set_the_usual_klib_context_bar $object_id $category_id
append_context_bar_data [list "object-access?[export_url_vars object_id category_id]" "Access control"] "Linked Objects"

if { [info exists user_id_from_search] } {
    set grantee_name [db_string user_name {
	select first_names || ' ' || last_name 
	from users
	where user_id = :user_id_from_search
    }]
    set grantee_id $user_id_from_search
} else {
    set grantee_name [db_string group_name {
	select group_name
	from groups
	where group_id = :group_id
    }]
    set grantee_id $group_id
}

if { $view_p } {
    db_exec_plsql grant_read {
	begin
	acs_permission.grant_permission(:object_id, :grantee_id, 'read');
	end;
    }
}

if { $edit_p } {
    db_exec_plsql grant_write {
	begin
	acs_permission.grant_permission(:object_id, :grantee_id, 'write');
	end;
    }
}

# List the linked objects for which the current user may set
# permissions.

set linked_objects [km_get_linked_objects -from_p 1 -to_p 1 $user_id $object_id]

set linked_object_ix [lsearch [fst $linked_objects] linked_object_id]
set linked_object_type_id_ix [lsearch [fst $linked_objects] linked_object_type]
set name_ix [lsearch [fst $linked_objects] name]
set overview_ix [lsearch [fst $linked_objects] overview]
set form_vars [export_form_vars object_id category_id view_p edit_p grantee_id]

set counter 0
template::multirow create links linked_object_type_id graphic pretty_type_plural object_id object_name short_description overview

foreach link [tail $linked_objects] {
    set linked_object_id [lindex $link $linked_object_ix]
    set linked_object_type_id [lindex $link $linked_object_type_id_ix]
    set linked_object_name [lindex $link $name_ix]

    if { $overview_ix != -1 } {
	set overview [lindex $link $overview_ix]
    } else {
	set overview ""
    }

    if [km_check_object_id -check_edit_p 1 -print_errors_p 0 $linked_object_id] {
	# Only show the object if the user may change the
	# permissions for it.

        set pretty_type_plural [km_static object_type_pretty_plural $linked_object_type_id]
        set graphic [km_static object_type_graphic $linked_object_type_id]
	if ![empty_string_p $overview] {
	    set overview [util_trim_string_with_hrefs $overview]
	    set snd_column "-"
	} else {
	    set snd_column ""
	}

	incr counter

	template::multirow append links $linked_object_type_id $graphic $pretty_type_plural $linked_object_id $linked_object_name $snd_column $overview
    }
}

if !$counter {
    ad_returnredirect "object-access?[export_url_vars object_id category_id]"
    return
}

ad_return_template
