ad_page_contract {
    /packages/library/www/object-access.tcl

    @cvs-id $Id: object-access.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    object_id:notnull,integer
    {category_id ""}
} -properties {
    title:onevalue
    object_id:onevalue
    url_vars:onevalue
    return_url:onevalue
    private_p:onevalue
    form_vars_search:onevalue
    form_vars_access_add:onevalue
    users:multirow
    unpublish_p:onevalue
    review_p:onevalue
    ot_archive_p:onevalue
    task_id:onevalue
    archived_p:onevalue
    approval_p:onevalue
    pretty_type:onevalue
}

set user_id [auth::require_login]
if { ![km_check_object_id -check_edit_p 1 $object_id] } { return }
set package_id [ad_conn package_id]

set admin_p [km_conn admin_p]
set object_name [km_conn object_name]
set stripped_name [sn_striphtml $object_name]
set public_p [km_conn public_p]
set archived_p [km_conn archived_p]
set publish_p [km_conn publish_p]
set review_p [km_conn in_review_p]
set approval_p [km_static approval_p $package_id]
set object_type_id [km_conn object_type_id]
set pretty_type_plural [km_static object_type_pretty_plural $object_type_id]
set pretty_type [km_static object_type_pretty_name $object_type_id]
set ot_archive_p [km_static object_type_archive_p $object_type_id]
set object_url "object-access"
set title "Access control for \"$stripped_name\""
set return_url [ad_urlencode "object-access?[export_url_vars object_id category_id]"]

set unpublish_p 1
if {$approval_p && !$admin_p && !$publish_p} {
    set unpublish_p 0
}

set_the_usual_klib_context_bar $object_id $category_id
append_context_bar_data "Access control"

# Set up variables for user-search.tcl.
set passthrough {object_id category_id}
set target "object-access-add"
set show_myself_p f
set context_bar [list "object-access?[export_url_vars object_id category_id]" "Access control"]
set form_vars_search [export_form_vars target passthrough object_id category_id show_myself_p context_bar]
set form_vars_group [export_form_vars object_id category_id]
set url_vars [export_url_vars object_id category_id object_name object_url]

## get the task_id of the workflow task if object is in review
if { $review_p } {
    set task_id [db_string get_workflow_task {
	select distinct(ut.task_id)
	from wf_cases c, wf_user_tasks ut
	where c.object_id = :object_id
	and c.case_id = ut.case_id
	and c.state = 'active'
	and rownum = 1
	order by ut.task_id desc
    } -default ""]
} else {
    set task_id ""
}

# Get users which have direct permissions on this object.

template::multirow create users_write user_id user_name
db_foreach user_list {
    select u.user_id as access_user_id, 
	u.first_names || ' ' || u.last_name as access_user_name
    from acs_permissions p1, acs_permissions p2,
	users u, sn_objects o
    where o.object_id = :object_id
	and p1.object_id = :object_id
	and p1.grantee_id = u.user_id
	and (:admin_p = 1 or p1.grantee_id <> o.original_author_id)
	and p1.privilege = 'read'
	and p2.object_id = :object_id
	and p2.grantee_id = u.user_id
	and p2.privilege = 'write'
    order by access_user_name
} {
    template::multirow append users_write $access_user_id $access_user_name
}

template::multirow create users_read user_id user_name
db_foreach user_list {
    select u.user_id as access_user_id, 
	u.first_names || ' ' || u.last_name as access_user_name
    from acs_permissions p1, users u, sn_objects o
    where o.object_id = :object_id
	and p1.object_id = :object_id
	and p1.grantee_id = u.user_id
	and (:admin_p = 1 or p1.grantee_id <> o.original_author_id)
	and p1.privilege = 'read'
	and not exists (select 1 from acs_permissions p2
			where p2.object_id = :object_id
			    and p2.grantee_id = u.user_id
			    and p2.privilege = 'write')
    order by access_user_name
} {
    template::multirow append users_read $access_user_id $access_user_name
}


# Get list of private groups this user owns.

db_multirow all_groups all_groups {
    select g.group_id, g.group_name
    from groups g, acs_objects o
    where o.creation_user = :user_id
	and o.object_type = 'private_group'
	and g.group_id = o.object_id
	and g.group_id not in (select grantee_id from acs_permissions
			       where object_id = :object_id and privilege = 'read')
}

# Get private groups which have direct permissions on this object.
# If the user is an admin, show all groups, else only the user's
# private groups.

template::multirow create groups_write group_id group_name
db_foreach group_list {
    select g.group_id as access_group_id, 
	g.group_name as access_group_name
    from acs_permissions p1, acs_permissions p2,
	groups g, acs_objects o
    where o.object_type = 'private_group'
	and (:admin_p = 1 or o.creation_user = :user_id)
	and g.group_id = o.object_id
	and p1.object_id = :object_id
	and p1.grantee_id = g.group_id
	and p1.privilege = 'read'
	and p2.object_id = :object_id
	and p2.grantee_id = g.group_id
	and p2.privilege = 'write'
    order by access_group_name
} {
    template::multirow append groups_write $access_group_id $access_group_name
}

template::multirow create groups_read group_id group_name
db_foreach group_list {
    select g.group_id as access_group_id, 
	g.group_name as access_group_name
    from acs_permissions p1, groups g, acs_objects o
    where o.object_type = 'private_group'
	and (:admin_p = 1 or o.creation_user = :user_id)
	and g.group_id = o.object_id
	and p1.object_id = :object_id
	and p1.grantee_id = g.group_id
	and p1.privilege = 'read'
	and not exists (select 1 from acs_permissions p2
			where p2.object_id = :object_id
			    and p2.grantee_id = g.group_id
			    and p2.privilege = 'write')
    order by access_group_name
} {
    template::multirow append groups_read $access_group_id $access_group_name
}
