ad_page_contract {
    object-copy-2.tcl

    Asks user for confirmation before copying selected objects.
    (a) if parent_id is set, the user can select several objects to be
        copied to parent_id under question_id (-> object_id_list)
    (b) if object_id is set, create a standalone object (check if possible).
        if object_id_list is also there, we came from object-unarchive
        or index/browse-one-*/object-view
        (c) copy all objects in object_id_list to object copied from object_id
            under the respective question_ids

    @author Timo Hentschel (timo@arsdigita.com)
    $Id: object-copy-2.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    {object_type_id:integer ""}
    {parent_id:integer ""}
    {question_id:integer ""}
    {category_id ""}
    {path_id:integer ""}
    {copy_id:integer ""}
    {object_id:integer ""}
    {object_id_list:integer,multiple ""}
    {return_url ""}
    cancel:optional
    copy:optional
} -properties {
    title:onevalue
    form_vars:onevalue
    url_vars:onevalue
    dc_export:onevalue
    objects:multirow
    object_type:onevalue
    object_type_plural:onevalue
}

set user_id [auth::require_login]
set package_id [ad_conn package_id]

if ![empty_string_p $copy_id] {
    set object_id $copy_id
}

if {[info exists cancel]} {
    if ![empty_string_p $question_id] {
	set return_url [get_path_return_url $path_id]
    } else {
	set return_url "browse-one-type?[export_url_vars object_type_id]"
    }
    ad_returnredirect $return_url
    return
}

if {[empty_string_p $object_type_id] && [empty_string_p $question_id] && [empty_string_p $object_id]} {
    ad_return_complaint 1 "Please select an object that should be copied!"
    return
}

if ![empty_string_p $question_id] {
    set object_type_id [km_static question_target_object_type_id $question_id]
} elseif ![empty_string_p $object_id] {
    ## check if object can be copied - correct permissions
    if ![km_check_object_id $object_id] { return }
    if {![km_conn admin_p] && ![km_conn write_p]} { return }
    set object_type_id [km_conn object_type_id]
}

set object_type [km_static object_type_pretty_name $object_type_id]
set object_type_plural [km_static object_type_pretty_plural $object_type_id]
set title "Confirm copy"
set new_object_id [km_new_acs_object_id]
set form_vars [export_vars -form {object_type_id parent_id question_id category_id path_id copy_id object_id object_id_list:multiple new_object_id return_url}]
set url_vars [export_url_vars object_type_id parent_id question_id category_id path_id]

if {![km_static object_type_copy_p $object_type_id]} {
    ad_return_complaint 1 "Not allowed to copy $object_type_plural!"
    return
}

if {[empty_string_p $question_id] && ![km_static object_type_create_p $object_type_id]} {
    ad_return_complaint 1 "No standalone objects allowed for $object_type_plural!"
    return
}

if ![empty_string_p $object_id] {
    set object_name [km_conn object_name]
}

if ![empty_string_p $question_id] {
    ## so we are called from one-question-edit or object-edit
    if [empty_string_p $object_id_list] {
	ad_return_complaint 1 "Please select objects to be copied!"
	return
    }
    if ![km_check_object_id $parent_id] { return }
    set parent_type [km_static object_type_pretty_name [km_conn object_type_id]]
    set parent_name [km_conn object_name]
    set question_name [km_static question_pretty_name $question_id]
    set_the_usual_klib_context_bar $parent_id $category_id
    append_context_bar_data [list "object-copy?$url_vars" "Copy a $object_type"] [list [get_path_return_url $path_id] Edit] $title

} elseif {![empty_string_p $return_url] && ![info exists copy]} {
    ## so we are called from object-unarchive
    set_the_usual_klib_context_bar $object_id $category_id
    append_context_bar_data [list "object-unarchive?[export_url_vars object_id category_id return_url]" "Remove from Archive"] "Copy"

} elseif {[empty_string_p $copy_id]} {
    ## so we are called from object-view
    set_the_usual_klib_context_bar $object_id $category_id
    append_context_bar_data "Copy"

} else {
    ## so we are called from index, browse-one-type or browse-one-category
    set_context_bar_data [list "browse-one-type?[export_url_vars object_type_id]" $object_type_plural] [list "object-copy?$url_vars" "Copy a $object_type"] $title
}

template::multirow create objects object_id name object_type_id object_type public_p archived_p review_p

if {![empty_string_p $object_id_list]} {
    db_foreach check_objects_for_copy "
	select o.object_id as copy_object_id, o.object_type_id as type_id,
	decode(o.one_line_description,null,'Unnamed ' || ot.pretty_name,o.one_line_description) as copy_object_name,
	decode(o.public_p,'t',1,0) as public_p,
	decode(o.archived_p,'t',1,0) as archived_p,
	decode(o.in_review_p,'t',1,0) as review_p,
	ot.pretty_plural as type_plural
	from sn_objects o, sn_object_types ot
	where o.object_type_id = ot.object_type_id
	and o.object_id in ([join $object_id_list ,])
	and o.context_id = :package_id
	and o.expiration_date > sysdate
	and acs_permission.permission_p(o.object_id, :user_id, 'write') = 't'
	order by ot.pretty_plural
    " {
	template::multirow append objects $copy_object_id [sn_striphtml $copy_object_name] $type_id $type_plural $public_p $archived_p $review_p
    }
}

# Double click.
set dc_export [doubleclick::signature_html]

ad_return_template
