ad_page_contract {
    object-copy-3.tcl

    Copies selected objects.
    (a) if parent_id is set, the user can select several objects to be
        copied to parent_id under question_id (-> object_id_list)
    (b) if object_id is set, create a standalone object (check if possible).
        if object_id_list is also there, we came from object-unarchive:
        (c) copy all objects in object_id_list to object copied from object_id
            under the respective question_ids

    @author Timo Hentschel (timo@arsdigita.com)
    $Id: object-copy-3.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    {object_type_id:integer ""}
    {parent_id:integer ""}
    {question_id:integer ""}
    {category_id ""}
    {path_id:integer ""}
    {copy_id:integer ""}
    {object_id:integer ""}
    {object_id_list:integer,multiple ""}
    cancel:optional
    {return_url ""}
    new_object_id:integer
}
csrf::authenticate

set user_id [auth::require_login]
set package_id [ad_conn package_id]

if {[info exists cancel]} {
    if ![empty_string_p $question_id] {
	set return_url [get_path_return_url $path_id]
    } elseif {[empty_string_p $return_url]} {
	if [empty_string_p $copy_id] {
	    set return_url "object-view?[export_url_vars object_id category_id]"
	} elseif [empty_string_p $category_id] {
	    set return_url "browse-one-type?[export_url_vars object_type_id]"
	} else {
	    set return_url "browse-one-category?[export_url_vars object_type_id category_id]"
	}
    }
    ad_returnredirect $return_url
    return
}

if ![empty_string_p $question_id] {
    set target_url [get_path_return_url $path_id]
} else {
    set target_url "questions?object_id=$new_object_id&category_id=$category_id"
}

## have to deal with doubleclick here: (actually, only at object-copy-3)
## redirect to one-question-edit/object-edit or questions of new object
if {[doubleclick::check_all]} {
    ns_sleep 10
    ad_returnredirect $target_url
    return
}

if {[empty_string_p $object_type_id] && [empty_string_p $question_id] && [empty_string_p $object_id]} {
    ad_return_complaint 1 "Please select an object that should be copied!"
    return
}

if ![empty_string_p $question_id] {
    set object_type_id [km_static question_target_object_type_id $question_id]
} elseif ![empty_string_p $object_id] {
    ## check if object can be copied - correct permissions
    if ![km_check_object_id $object_id] { return }
    if {![km_conn admin_p] && ![km_conn write_p]} { return }
    set object_type_id [km_conn object_type_id]
}

if {![km_static object_type_copy_p $object_type_id]} {
    ad_return_complaint 1 "Not allowed to copy [km_static object_type_pretty_plural $object_type_id]!"
    return
}

if {[empty_string_p $question_id] && ![km_static object_type_create_p $object_type_id]} {
    ad_return_complaint 1 "No standalone objects allowed for [km_static object_type_pretty_plural $object_type_id]!"
    return
}

if ![empty_string_p $question_id] {
    ## so we are called from one-question-edit or object-edit
    ## copy all objects in object_id_list to parent_id under question_id
    if {[string equal [km_static question_abstract_data_type $question_id] "child_object"]} {
	set link_type "parent_child"
    } else {
	set link_type "uncle_nephew"
    }
    db_transaction {
	foreach child_id $object_id_list {
	    set new_object_id [km_new_acs_object_id]
	    km_copy_object $child_id $new_object_id
	    set result [km_link_objects -link_type $link_type $question_id $user_id $parent_id $new_object_id "" f]
	    km_check_public_status_after_edit $new_object_id
	}
    }

} elseif {![empty_string_p $return_url] || ![empty_string_p $object_id_list]} {
    ## so we are called from object-unarchive
    ## or object-view, index, browse-one-*
    ## copy object_id and object_id_list
    db_transaction {
	km_copy_object $object_id $new_object_id
	set parent_id $new_object_id

	if ![empty_string_p $object_id_list] {
	    db_foreach check_children "
	        select l.object_id_b as child_id, l.link_type, qlm.question_id
	        from sn_links l, sn_question_link_map qlm
	        where l.object_id_a = :object_id
	        and l.link_type in ('parent_child','uncle_nephew')
	        and l.object_id_b in ([join $object_id_list ,])
	        and l.link_id = qlm.link_id
	    " {
		lappend child_list [list $child_id $link_type $question_id]
	    }

	    foreach child $child_list {
		util_unlist $child child_id link_type question_id
		set new_object_id [km_new_acs_object_id]
		km_copy_object $child_id $new_object_id
		set result [km_link_objects -link_type $link_type $question_id $user_id $parent_id $new_object_id "" f]
	    }
	}
    }

} else {
    ## so we are called from object-view, index, browse-one-*
    ## copy object_id
    db_transaction {
	km_copy_object $object_id $new_object_id
    }
}

ad_returnredirect $target_url
