ad_page_contract {
    object-copy.tcl

    Displays a list of all objects the user can copy
    if parent_id is not set, let user select only one object as standalone

    @author Timo Hentschel (timo@arsdigita.com)
    $Id: object-copy.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    {object_type_id:integer ""}
    {parent_id:integer ""}
    {question_id:integer ""}
    {category_id ""}
    {path_id:integer ""}
    {order:oneof(name|date) "name"}
} -properties {
    title:onevalue
    form_vars:onevalue
    url_vars:onevalue
    objects:multirow
    object_type:onevalue
    object_type_plural:onevalue
    short_description:onevalue
    long_description:onevalue
    short_p:onevalue
    long_p:onevalue
    cancel_url:onevalue
    target:onevalue
}

set user_id [auth::require_login]

if {[empty_string_p $object_type_id] && [empty_string_p $question_id]} {
    ad_return_complaint 1 "Error in page call"
    return
}

if [empty_string_p $object_type_id] {
    set object_type_id [km_static question_target_object_type_id $question_id]
}

set object_type [km_static object_type_pretty_name $object_type_id]
set object_type_plural [km_static object_type_pretty_plural $object_type_id]
set title "Copy a $object_type"
set package_id [ad_conn package_id]
set user_id [ad_conn user_id]
set form_vars [export_form_vars object_type_id parent_id question_id category_id path_id]
set url_vars [export_url_vars object_type_id parent_id question_id category_id path_id]

set short_ids [km_static object_type_short_description $object_type_id]
set long_id [km_static object_type_long_description $object_type_id]
if {[empty_string_p $short_ids]} {
    set short_p 0
    set short_description ""
} else {
    set short_p 1
    set short_description [list]
    foreach short_id $short_ids {
	lappend short_description [km_static question_pretty_name $short_id]
    }
    set short_description [join $short_description ", "]
}
if {[empty_string_p $long_id]} {
    set long_p 0
    set long_description ""
} else {
    set long_p 1
    set long_description [km_static question_pretty_name $long_id]
}
if {!$short_p && !$long_p} {
    set short_p 1
    set short_description "Name"
}

if {![km_static object_type_copy_p $object_type_id]} {
    ad_return_complaint 1 "Not allowed to copy $object_type_plural!"
    return
}

if {[empty_string_p $question_id] && ![km_static object_type_create_p $object_type_id]} {
    ad_return_complaint 1 "No standalone objects allowed for $object_type_plural!"
    return
}

if ![empty_string_p $question_id] {
    ## so we are called from one-question-edit or object-edit
    if ![km_check_object_id $parent_id] { return }
    set cancel_url [get_path_return_url $path_id]
    set parent_type [km_static object_type_pretty_name [km_conn object_type_id]]
    set parent_name [km_conn object_name]
    set question_name [km_static question_pretty_name $question_id]
    set_the_usual_klib_context_bar $parent_id $category_id
    append_context_bar_data [list $cancel_url Edit] $title
    set target "object-copy-2"

} else {
    ## so we are called from index, browse-one-type or browse-one-category
    if {[empty_string_p $category_id]} {
	set cancel_url "browse-one-type?[export_url_vars object_type_id]"
    } else {
	set cancel_url "browse-one-category?[export_url_vars object_type_id category_id]"
    }
    set target "object-copy-1"
    set_context_bar_data [list "browse-one-type?[export_url_vars object_type_id]" $object_type_plural] $title
}

if {$order == "name"} {
    set order_sql "lower(object_name)"
} else {
    set order_sql "o.last_modified"
}

template::multirow create objects object_id name overview owner_id owner_name public_p archived_p review_p

set descendants_p 0
db_foreach get_copyable_objects "
    select o.object_id as copy_object_id, o.overview,
           decode(o.one_line_description,null,'Unnamed ' || ot.pretty_name,o.one_line_description) as object_name,
           decode(o.overview_html_p,'t',1,0) as overview_html_p,
           decode(o.public_p,'t',1,0) as public_p,
           decode(o.archived_p,'t',1,0) as archived_p,
           decode(o.in_review_p,'t',1,0) as review_p,
           o.creation_date, u.user_id as owner_id,
           u.first_names || ' ' || u.last_name as owner_name,
           acs_permission.permission_p(o.object_id, :user_id, 'write') as write_p
    from sn_objects o, users u, sn_object_types ot
    where ot.object_type_id = :object_type_id
    and o.object_type_id = ot.object_type_id
    and o.context_id = :package_id
    and o.expiration_date > sysdate
    and o.original_author_id = u.user_id
    order by $order_sql
" {
    set descendants_p 1
    if {$write_p == "t"} {
	if ![empty_string_p $overview] {
	    set overview [util_trim_string_with_hrefs [sn_striphtml $overview]]
	}
	template::multirow append objects $copy_object_id [sn_striphtml $object_name] $overview $owner_id $owner_name $public_p $archived_p $review_p
    }
}

# if there are no descendant objects the user could choose to copy too, redirect
# directly to confirmation page
if {!$descendants_p} {
    set target "object-copy-2"
}

ad_return_template
