ad_page_contract {
    packages/library/www/object-delete-2.tcl

    Delete an object.

    @cvs-id $Id: object-delete-2.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    object_id:notnull,integer
    {parent_object_id:integer 0}
    {uncle_object_id:integer 0}
    {question_id:integer 0}
    {category_id ""}
    {reason_for_deleting ""}
    {return_delete ""}
}
csrf::authenticate

set user_id [auth::require_login]

if {!$parent_object_id && !$uncle_object_id} {
    set parent [km_get_parent $object_id]
    if ![empty_string_p $parent] {
	set parent_object_id [fst $parent]
    }
    set uncle [km_get_uncle $object_id]
    if ![empty_string_p $uncle] {
	set uncle_object_id [fst $uncle]
    }
}

if { !$parent_object_id  && !$uncle_object_id && [empty_string_p $reason_for_deleting] } {
    ad_return_complaint 1 "<li>You have to enter a reason for deleting this object."
    return
}

if { $parent_object_id && $parent_object_id != [fst [km_get_parent $object_id]] } {
    ad_return_complaint 1 "The parent object given doesn't match the parent in the database."
    return
}

if { $uncle_object_id } {
    set parent_object_id [fst [km_get_uncle $object_id]]
    if { $parent_object_id != $uncle_object_id } {
	ad_return_complaint 1 "The uncle object given doesn't match the uncle in the database."
	return
    }
}

if { $parent_object_id && $question_id } {
    # We're deleting a child object, so we have to use a special return_url.
    append return_delete "?object_id=$parent_object_id&category_id=$category_id&question_id=$question_id"
}

if ![km_check_object_id -check_delete_p 1 $object_id] { return }
set object_type_id [km_conn object_type_id]

if [doubleclick::check_all $object_id] {
    ns_sleep 2
} else {
    db_transaction {
	db_dml object_delete_2_10 {
	    update sn_objects
	    set expiration_date = sysdate, last_modifying_user_id = :user_id
	    where object_id=:object_id 
		or object_id in (select child 
				 from km_flat_object_hierarchy 
				 where parent = :object_id)
	}
    
	if ![empty_string_p $reason_for_deleting] {
	    db_dml object_delete_2_20 {
		insert into sn_object_delete_reasons (object_id, reason_for_deleting)
		values (:object_id, :reason_for_deleting)
	    }
	}

	if { $uncle_object_id && [km_uncle_needs_private_status_p $uncle_object_id] } {
	    km_object_unpublish $uncle_object_id
	}
    }
}

if { [empty_string_p $return_delete] && ($parent_object_id || $uncle_object_id) } {
    if { $parent_object_id } {
	set object_id $parent_object_id
    } else {
	set object_id $uncle_object_id
    }
    set return_delete "object-view"
}

if {[empty_string_p $return_delete]} {
    if {[empty_string_p $category_id] || $category_id == "none"} {
	set return_delete "browse-one-type"
    } else {
	set return_delete "browse-one-category"
    }
}

switch $return_delete {
    "browse-one-type" { set return_delete "browse-one-type?[export_url_vars object_type_id]" }
    "browse-one-category" { set return_delete "browse-one-category?[export_url_vars object_type_id category_id]" }
    "object-view" { set return_delete "object-view?[export_url_vars object_id category_id]" }
    "one-question-edit" { set return_delete "one-question_id?[export_url_vars object_id category_id question_id]" }
}

ad_returnredirect $return_delete
