ad_page_contract {
    /www/library/object-edit-2.tcl

    First save the data from object-edit.tcl, then redirect the user
    to his destination.

    @cvs-id $Id: object-edit-2.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    object_type_id:integer
    object_id:integer
    {path_id:integer ""}
    {category_id ""}
    {parent_object_id:integer 0}
    {uncle_object_id:integer 0}
    {question_id:integer 0}
    gc_checksum:notnull
    mandatory_p:integer
}
csrf::authenticate

set last_question_id $question_id
set action ""
set_the_usual_form_variables

set form_size [ns_set size $Vform]

set user_id [auth::require_login]

set submit_buttons [km_get_button_names]

# Go through the form and pick out the data we need for the path from
# the data that needs to go to Oracle
set i 0
while {$i < $form_size} {
    
    set key [ns_set key $Vform $i]
    set value [ns_set value $Vform $i]
    # isolate the general name of this form field-- 
    # a submit button has the form general_name:question_id from the question_id 
    # that is is attached to
    regsub {\..*} $key {} button_name 

    # see if this key is a submit button
    set ix [lsearch [km_get_button_names] $button_name]
    if {$ix >= 0} {
	
	# This IS the form's submit button which sets the path action. 
	# We want to get the question_id out of it as well.
	set action $button_name
	regexp {.*\.([0-9]*)} $key match question_id
    

	# If it's the delete button for read-only questions,
	# get the question_id.
	regexp {delete\.([0-9]+)} $key match delete_question_id

    } elseif { [lsearch {object_type object_id category_id path_id} $key] == -1 } {
	
	# Must be a column that belongs to the object type.
	lappend keys $key 
	lappend values $value
    }
    incr i
}

# Create the object if it doesn't already exist
if {![km_object_exists_p $object_id]} {
    if ![km_check_object_type_id -check_view_p 0 -check_create_p 1 $object_type_id] { 
	return 
    }

    set package_id [ad_conn package_id]
    set peeraddr [ad_conn peeraddr]

    db_transaction {
	set dummy_object_id [db_exec_plsql create_object {
	    begin
	    :1 := object.insert_object (v_object_id => :object_id,
	                                v_object_type_id => :object_type_id,
					v_creation_user => :user_id,
					v_creation_ip => :peeraddr,
					v_context_id => :package_id);
	    end;
	}]

	db_exec_plsql grant_owner_permissions {
	    begin
	    acs_permission.grant_permission(:object_id, :user_id, 'read');
	    acs_permission.grant_permission(:object_id, :user_id, 'write');
	    acs_permission.grant_permission(:object_id, :user_id, 'delete');
	    end;
	}
    }

    km_flush_category_count -object_type_id $object_type_id

} else {
    if ![km_check_object_id -check_edit_p 1 $object_id] { return }
}

# Sort the form data
set key_values [km_sort_form [zip $keys $values]]

# Check the input for bad dates, etc.
set bad_input [km_check_input $object_id $key_values]

if ![null_p $bad_input] {
    ad_return_complaint [fst $bad_input] [snd $bad_input]
    return
} 

# Save the data
km_save_object_data $object_id $user_id $key_values
km_check_public_status_after_edit $object_id

# Have these already been linked during some path?
set ancestors [km_get_object_ancestors $object_id]
set result 1
if { $parent_object_id && ![elem_p $parent_object_id [fst [transpose $ancestors]]] } {
    # Create a parent_child_link
    set result [km_link_objects -link_type parent_child $last_question_id $user_id $parent_object_id $object_id "" f]
}
if { $uncle_object_id && ![elem_p $uncle_object_id [fst [transpose $ancestors]]] } {
    # Create a uncle_nephew_link
    set result [km_link_objects -link_type uncle_nephew $last_question_id $user_id $uncle_object_id $object_id "" f]
}
if {!$result} {
    ad_return_complaint 1 "This object couldnt be linked because it is already a child or a nephew of a knowledge object."
    return
}

# Figure out where to go from here.
if { [string compare $action "delete"] == 0 } {
    km_delete_object_data $object_id $delete_question_id
    if { [string equal $action "details"]} {
	set destination "questions?[export_url_vars object_id category_id]"
    } else {
	## action = "finish"
	set destination "object-view?[export_url_vars object_id category_id]"
    }

} elseif { ![string equal $action "finish"] && ![string equal $action "details"]} {
    # Some excursion is starting here.
    set path_vars [set_path_values -question_id $question_id $object_id $category_id]
    if [empty_string_p $path_id] {
	set parameters [export_url_vars object_id category_id mandatory_p]
    } else {
	set parameters [export_url_vars object_id category_id path_id mandatory_p]
    }

    if { $action == "add_user_ref" } {
	set return_url "object-edit?$parameters&foo=bar#user_link"
    } elseif { $action == "add_content_ref" } {
	set return_url "object-edit?$parameters&foo=bar#content_link$question_id"
    } elseif { $parent_object_id } {
	set return_url "object-edit?$parameters&parent_object_id=$parent_object_id&question_id=$last_question_id"
    } elseif { $uncle_object_id } {
	set return_url "object-edit?$parameters&uncle_object_id=$uncle_object_id&question_id=$last_question_id"
    } else {
	set return_url "object-edit?$parameters"
    }
    set path_id [init_path -last_path_id $path_id $path_vars $return_url $action]

    # Figure out where it is they want to go and redirect them.
    switch $action {
	"add_web_ref"      { set destination "add-web-ref?[export_url_vars path_id]" }
	"add_sn_ref"       { 
            set path_vars [list question_id $question_id object_id $object_id category_id $category_id]
            set path_id [init_path $path_vars $return_url add_sn_ref]
            array set node [site_node_closest_ancestor_site_node "acs-subsite"] 
            set community_id $node(package_id) 
            set destination "linking/?community_id=$community_id&source_id=$object_id&pass=$path_id"
        }
	"add_child" {
	    set parent_object_id $object_id
	    set destination "object-edit?[export_url_vars path_id question_id parent_object_id category_id]"
	 }
	"add_nephew" {
	    set uncle_object_id $object_id
	    set destination "object-edit?[export_url_vars path_id question_id uncle_object_id category_id]"
	 }
	"copy_child" {
	    set parent_id $object_id
	    set destination "object-copy?[export_url_vars path_id question_id parent_id category_id]"
	 }
	"copy_nephew" {
	    set parent_id $object_id
	    set destination "object-copy?[export_url_vars path_id question_id parent_id category_id]"
	 }
	"add_user_ref" {
	    set target "user-link-add"
	    set passthrough {path_id}
	    set keyword [set user_link.$question_id]
            set context_bar [list $return_url "Edit"]
	    set destination  "user-search?[export_url_vars keyword target passthrough path_id context_bar object_id category_id]"
	}
	"add_content_ref" {
	    set target "content-link-add"
	    set query_string [set content_query:$question_id]
	    set table_name [set content_table:$question_id]
	    set destination  "/shared/sw-search-2?[export_url_vars query_string table_name target path_id]"
	}
	default	{
            if { [string equal $action "details"]} {
                set destination "questions?[export_url_vars object_id category_id]"
	    } else {
		## action == "finish"
                set destination "object-view?[export_url_vars object_id category_id]"
            }
	}
    }
} elseif ![empty_string_p $path_id] {
    # We already are on an excursion, and the user has just created a new object. So instead of returning to
    # questions.tcl, let's take a look at the path's action.
    set values [get_path_values -action_only_p 0 $path_id]
    set action [value_from_tuples $values action]
    switch $action {
	"add_sn_ref" { 
	    set destination "add-sharenet-ref?[export_url_vars object_id path_id]" 
	}
	"link" { 
	    set destination "link-reason-add?[export_url_vars object_id path_id]" 
	}
	"add_child" - 
        "add_nephew" 
        {
	    set destination [get_path_return_url $path_id]
	}
	default {
            if { [string equal $action "details"]} {
                set destination "questions?[export_url_vars object_id category_id]"
	    } else {
		## action == "finish"
                set destination "object-view?[export_url_vars object_id category_id]"
            }
	}
    }
} elseif { $parent_object_id && $question_id } {
	set destination "one-question-edit?object_id=$parent_object_id&question_id=$question_id&category_id=$category_id"
} elseif { $uncle_object_id && $question_id } {
	set destination "one-question-edit?object_id=$uncle_object_id&question_id=$question_id&category_id=$category_id"
} else {
    if { [string equal $action "details"]} {
	set destination "questions?[export_url_vars object_id category_id]"
    } else {
	## action == "finish"
	set destination "object-view?[export_url_vars object_id category_id]"
    }
}

ad_returnredirect $destination
