ad_page_contract {
    /packages/library/www/object-add.tcl

    Create the form for adding an object.

    @author Carsten Clasohm (carsten@arsdigita.com)
    @author Dirk Gomez (dirk@arsdigita.com)

    @cvs-id $Id: object-edit.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    { object_type_id:integer 0 }
    { object_id:integer 0 }
    { category_id "" }
    { parent_object_id:integer 0 }
    { uncle_object_id:integer 0 }
    { path_id:integer "" }
    { link_new_p:boolean 0 }
    { mandatory_p:boolean 0 }
    { question_id:integer 0 }
    { short_name "" }
} -properties {
    title:onevalue
    graphic:onevalue
    graphic_width:onevalue
    form_vars:onevalue
    enctype:onevalue
    gc_checksum_export:onevalue
    ancestor_name:onevalue
    ancestor_object_type:onevalue
    ancestor_object_id:onevalue
    object_id:onevalue
    object_type_id:onevalue
    show_values_p:onevalue
    mandatory_p:onevalue
}

set user_id [auth::require_login]
set edit_p 0
set show_values_p 0

if { $object_id } {

    # Retrieve the infos for an existing object and display data in
    # the form.

    if ![km_check_object_id -check_edit_p 1 $object_id] { return }
    set object_name [km_conn object_name]
    set object_type_id [km_conn object_type_id]
    set show_values_p 1
    set edit_p 1

} elseif {$object_type_id || ![empty_string_p $short_name]} { 

    if {![empty_string_p $short_name]} {
	set context_id [ad_conn package_id]
	set object_type_id [db_string object_type_id {
	    select min(object_type_id) 
	    from sn_object_types
	    where short_name = :short_name and context_id = :context_id
	} -default "0"]
    }
    if ![km_check_object_type_id -check_view_p 0 -check_create_p 1 $object_type_id] { 
	return 
    }

    # This page is for a brand-new knowledge object, so there's no data.
    set object_id [km_new_acs_object_id]

} elseif { $question_id } {
   
    # This page has been called up for the creation of a child object
    set object_type_id [km_static question_target_object_type_id $question_id]

    # Set the new object_id here directly on the form
    set object_id [km_new_acs_object_id]

} else {
    set return_url [ad_conn url]?[ad_conn query]
    set path_vars [list question_id $question_id object_id $object_id category_id $category_id]
    set path_id [init_path $path_vars $return_url add_content_ref]
    array set node [site_node_closest_ancestor_site_node "acs-subsite"] 
    set community_id $node(package_id) 
    ad_returnredirect "linking/?community_id=$community_id&source_id=$object_id&pass=$path_id"
    return
}

set pretty_type [km_static object_type_pretty_name $object_type_id]
set pretty_plural [km_static object_type_pretty_plural $object_type_id]
set graphic [km_static object_type_graphic $object_type_id]

if { $edit_p } {
    set action "Edit"
    set title "Edit $object_name"
} else {
    set action "Create $pretty_type"
    set title $action
}

# Since this object may not yet exist, it might not have ancestors but
# it *might* already have a parent associated with it.

set ancestor_object_id 0
if { $parent_object_id > 0 || $uncle_object_id > 0 } {

    if { $parent_object_id } {
	set ancestor_object_id $parent_object_id
    } else {
	set ancestor_object_id $uncle_object_id
    }
    if ![km_check_object_id $ancestor_object_id] { return }
    set ancestor_object_type [km_static object_type_pretty_name [km_conn object_type_id]]
    set ancestor_name [km_conn object_name]
    set_the_usual_klib_context_bar $ancestor_object_id $category_id

    if { $edit_p } {
	append_context_bar_data [list "object-view?object_id=$object_id&category_id=$category_id" $object_name] $action
    } else {
	append_context_bar_data $action
    }
} elseif { $edit_p } { 
    set_the_usual_klib_context_bar $object_id $category_id
    append_context_bar_data $action
} else {
    set_context_bar_data [list "browse-one-type?[export_url_vars object_type_id]" $pretty_plural] $action
}


if ![km_check_object_type_id -check_view_p 0 -check_create_p 1 $object_type_id] { 
    return 
}

if [km_type_has_data_type_p $object_type_id "file"] {
    set enctype "enctype=multipart/form-data"
} else {
    set enctype ""
}

set form_vars [export_form_vars object_id category_id path_id object_type_id parent_object_id uncle_object_id question_id mandatory_p]
set graphic_width [library_icon_width]

set gc_checksum [doubleclick::generate_checksum $object_id]
set gc_checksum_export [export_form_vars gc_checksum]

ad_return_template
