ad_page_contract {
    /packages/library/www/object-propose-public-2.tcl

    Submit object for approval.

    @cvs-id $Id: object-propose-public-2.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    object_id:notnull,integer
    {category_id ""}
    {button_name "Confirm"}
    {return_url ""}
    {nephew_list:multiple,optional ""}
}
csrf::authenticate

if {[empty_string_p $return_url]} {
    set return_url "object-view?[export_url_vars object_id category_id]"
}

if {[doubleclick::check_all $nephew_list]} {
    ns_sleep 2
    ad_returnredirect $return_url
    return
}

set user_id [auth::require_login]
set package_id [ad_conn package_id]
if ![km_check_object_id $object_id] { return }

set author_id [km_conn original_author_id]
set write_p [km_conn write_p]
set admin_p [km_conn admin_p]
set archived_p [km_conn archived_p]
set publish_p [km_conn publish_p]
set approval_p [km_static approval_p $package_id]
set creation_ip [ad_conn peeraddr]
# the sweeper will make sure that descendants will have the same publisher
# than the ancestors
set publisher_id [km_conn publisher_id]

if { !$approval_p || $publish_p || $admin_p } {
    ad_returnredirect "object-publish?[export_url_vars object_id category_id return_url]"
    return
}

if { [km_conn public_p] } {
    ad_return_complaint 1 "This [km_static object_type_pretty_name [km_conn object_type_id]] is already public."
    return
}

if { $button_name == "Cancel" } {
    ad_returnredirect $return_url
    return
}

if { ![km_conn delete_p] } {
    ad_return_complaint 1 "Only owners can propose an object for publication."
    return
}

set uncle_p [km_is_mandatory_uncle_p $object_id]
set child_p [km_is_child_p $object_id]
set private_nephew_p [km_is_nephew_with_private_uncle_p $object_id]

if {$child_p} {
    ad_return_complaint 1 "Not allowed to publish descendent objects separately."
    return
}

if {$private_nephew_p} {
    ad_return_complaint 1 "Not allowed to publish descendent object without public main object"
    return
}

if {$uncle_p && ([empty_string_p $nephew_list] || ![km_valid_nephew_publish_list_p $object_id $nephew_list])} {
    ad_return_complaint 1 "You have to select at least one linked object of each question for publishing!"
    return
}

if {[empty_string_p $nephew_list]} {
    set descendents [concat $object_id [map fst [km_get_descendents $object_id]]]
} else {
    set descendents [concat $object_id $nephew_list [km_get_objects_descendents $nephew_list] [km_get_child_descendents $object_id]]
}

set complete_p [km_object_complete_p $object_id]

if {!$complete_p} {
    ad_return_warning "Required Data Missing" \
	    "<p>At least one mandatory question has not been answered. Please
	     <a href=\"object-edit?[export_url_vars object_id]&mandatory_p=1\">edit</a>
	     the object and add any missing data."
    return
}

# have to compose workflow comment if nephew_list supplied
if ![empty_string_p $nephew_list] {
    set msg_list [list]
    set last_nephew_type_id ""

    db_foreach get_nephew_data "
	select object_id as nephew_id, one_line_description as nephew_name,
	       object_type_id as nephew_type_id
	from sn_objects
	where object_id in ([join $nephew_list ,])
	order by object_type_id, one_line_description
    " {
	if [empty_string_p $nephew_name] {
	    set nephew_name "Unnamed [km_static object_type_pretty_name $nephew_type_id]"
	}
	if {$last_nephew_type_id != $nephew_type_id} {
	    lappend msg_list "[km_static object_type_pretty_plural $nephew_type_id]: $nephew_name"
	    set last_nephew_type_id $nephew_type_id
	} else {
	    lappend msg_list $nephew_name
	}
    }
    set msg "The following linked objects should be made public as well: [join $msg_list ,]"
}

db_transaction {
    # mark objects as in_review_p
    db_dml mark_objects_in_review_p "
	update sn_objects
	set in_review_p = 't'
	where object_id in ([join $descendents ,])
    "

    # Actually submit objects for publication.
    set case_id [db_exec_plsql object_queue_for_review {
	begin
	   :1 := library_workflow.new_workflow_case (
              p_object_id     => :object_id,
              p_creation_user => :user_id,
              p_creation_ip   => :creation_ip,
	      p_publisher_id  => :publisher_id
	   );
	end;
    }]

    # add workflow comment for selected nephews
    if ![empty_string_p $nephew_list] {
	db_exec_plsql add_workflow_comment {
	    begin
	       :1 := journal_entry.new(
                  object_id     => :case_id,
                  action        => 'case start',
                  action_pretty => 'Case started',
                  creation_user => :user_id,
                  creation_ip   => :creation_ip,
                  msg           => :msg
        );
	    end;
	}
    }
}

ad_returnredirect $return_url


