ad_page_contract {
    /www/library/object-publish-2.tcl

    Publish the object.

    @cvs-id $Id: object-publish-2.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    object_id:notnull,integer
    {category_id ""}
    cancel:optional
    confirm:optional
    release_date:array,optional
    {return_url ""}
    {nephew_list:multiple,optional ""}
    task_id:optional
    msg:optional
}
csrf::authenticate

if {[empty_string_p $return_url]} {
    set return_url "object-view?[export_url_vars object_id category_id]"
}

if {[doubleclick::check_all $nephew_list]} {
    ns_sleep 2
    ad_returnredirect $return_url
    return
}

set user_id [auth::require_login]
set package_id [ad_conn package_id]
if ![km_check_object_id $object_id] { return }

set author_id [km_conn original_author_id]
set write_p [km_conn write_p]
set admin_p [km_conn admin_p]
set archived_p [km_conn archived_p]
set publish_p [km_conn publish_p]
set approval_p [km_static approval_p $package_id]
set in_review_p [km_conn in_review_p]

if { !$admin_p && (!$write_p || $archived_p) } { return }

if { [km_conn public_p] } {
    ad_return_complaint 1 "This [km_static object_type_pretty_name [km_conn object_type_id]] is already public."
    return
}

if { $approval_p && !$publish_p && !$admin_p } {
    ad_returnredirect "object-propose-public?[export_url_vars object_id category_id return_url]"
    return
}

if {[info exists cancel]} {
    ad_returnredirect $return_url
    return
}

set uncle_p [km_is_mandatory_uncle_p $object_id]
set child_p [km_is_child_p $object_id]
set private_nephew_p [km_is_nephew_with_private_uncle_p $object_id]

if {$child_p} {
    ad_return_complaint 1 "Not allowed to publish descendent objects separately."
    return
}

if {$private_nephew_p} {
    ad_return_complaint 1 "Not allowed to publish descendent object without public main object"
    return
}

if {$uncle_p && ([empty_string_p $nephew_list] || ![km_valid_nephew_publish_list_p $object_id $nephew_list])} {
    ad_return_complaint 1 "You have to select at least one linked object of each question for publishing!"
    return
}

if {[empty_string_p $nephew_list]} {
    set descendents [concat $object_id [map fst [km_get_descendents $object_id]]]
} else {
    set descendents [concat $object_id $nephew_list [km_get_objects_descendents $nephew_list] [km_get_child_descendents $object_id]]
}

set object_name [km_conn object_name]
set object_type_id [km_conn object_type_id]
set write_p [km_conn write_p]
set original_author_id [km_conn original_author_id]
set complete_p [km_object_complete_p $object_id]

if { [info exists release_date] } {
    set release_date_sql "$release_date(year)-$release_date(month)-$release_date(day)"
    if { $release_date_sql == "--" || ![date_p $release_date_sql] } {
	set release_date_sql "sysdate"
    } else {
	set release_date_sql "'$release_date_sql'"
    }
} else {
    set release_date_sql "sysdate"
}

if {!$complete_p} {
    ad_return_warning "Required Data Missing" \
	    "<p>At least one mandatory question has not been answered. Please
	     <a href=\"object-edit?[export_url_vars object_id]&mandatory_p=1\">edit</a>
	     the object and add any missing data."
    return
}

db_transaction {

    # Check if a user already got shares for this entry (by making
    # it public).  If not, give him the shares.  This is ignoring
    # the issue of different banks, but that's OK because we just
    # want to know if he was rewarded for publishing the object,
    # not where and how the reward took place.
    set awardedp [db_string object_awarded_shares {
	select count(*) from ir_share_transactions 
	where cc_name = 'kl_object_publish' and object_id = :object_id
    }]

    if {!$awardedp && ![km_is_nephew_p $object_id]} {
	foreach bank_id [incentives::bank_list -user_id $original_author_id $package_id] {
	    set amount [incentives::charge_value -key $object_type_id \
		    kl_object_publish $bank_id]
	    incentives::award_shares -user_id $original_author_id \
		    kl_object_publish $amount $bank_id $object_id
	}
    }

    # Actually publish the object.
    km_object_publish $descendents

    # Generate alerts.
    set published_objects_list [db_list_of_lists get_published_objects "
	select object_id, object_type_id
	from sn_objects
	where public_p = 't'
	and object_id in ([join $descendents ,])
    "]

    foreach published_object $published_objects_list {
	util_unlist $published_object published_object_id published_object_type_id

	alerts::generate_alert -package_id $package_id -content_type_id $published_object_type_id -action insert -link_only $published_object_id
    }

    ## but what if the task is at a clarify state? can we close it then
    ## with this code?
    if ![info exists task_id] {
	db_0or1row get_workflow_case {
	    select case_id
	    from wf_cases
	    where object_id = :object_id
	    and state = 'active'
	    and rownum = 1
	    order by case_id desc
	}
	if [info exists case_id] {
	    db_dml delete_task_assignments {
		delete from wf_task_assignments
		where task_id in (select task_id from wf_tasks where case_id = :case_id)
	    }
	    db_dml finish_tasks {
		update wf_tasks
		set state = 'finished'
		where case_id = :case_id
	    }
	    db_dml delete_tokens {
		delete from wf_tokens
		where case_id = :case_id
	    }
	    db_1row get_task_id {
		select wf_task_id_seq.nextval as task_id from dual
	    }
	    db_dml insert_transition_key {
		insert into wf_tasks
		      (task_id, case_id, workflow_key, transition_key, state, started_date, holding_user)
		values (:task_id, :case_id, 'library_approval_wf', 'library_review', 'started', sysdate, :user_id)
	    }
	    set msg "Object Published"
	}
    }

    if [info exists task_id] {
	# update publisher_id
	db_dml update_publisher_id "
	    update sn_objects
	    set publisher_id = :user_id
	    where object_id in ([join $descendents ,])
	"

	#finish workflow task
	set journal_id [wf_task_action -user_id $user_id -msg $msg -attributes [list library_object_needs_clarification f] $task_id finish]
    }
}

ad_returnredirect $return_url
