ad_page_contract {
    /packages/library/www/object-publish.tcl

    Ask the user for confirmation before publishing an object.

    @cvs-id $Id: object-publish.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    object_id:notnull,integer
    {category_id ""}
    {return_url ""}
    task_id:optional
    msg:optional
} -properties {
    title:onevalue
    form_vars:onevalue
    object_id:onevalue
    object_type:onevalue
    object_type_plural:onevalue
    ancestor_object_type:onevalue
    ancestor_object_type_plural:onevalue
    author_id:onevalue
    author_name:onevalue
    complete_p:onevalue
    nephews:multirow
    nephews_p:onevalue
    nephew_p:onevalue
    data_missing_questions:multirow
    error:onevalue
    in_review_p:onevalue
    awarded_p:onevalue
    shares:multirow
    user_id:onevalue
}

set user_id [auth::require_login]
if ![km_check_object_id $object_id] { return }

set checkoff_date [db_string object_publish_10 "
    select decode(user_checkoff_date,'9999-12-31',sysdate,user_checkoff_date)
    from sn_objects where object_id=:object_id"]
set release_year [string range $checkoff_date 0 3]
set release_month [string range $checkoff_date 5 6]
set release_day [string range $checkoff_date 8 9]

set object_name [km_conn object_name]
set object_type_id [km_conn object_type_id]
set object_type [km_static object_type_pretty_name $object_type_id]
set object_type_plural [km_static object_type_pretty_plural $object_type_id]
set author_id [km_conn original_author_id]
set author_name [km_conn original_author_name]
set complete_p [km_object_complete_p $object_id]
set nephews_p [km_object_nephews_p $object_id]
set uncle_p [km_is_mandatory_uncle_p $object_id]
set child_p [km_is_child_p $object_id]
set private_nephew_p [km_is_nephew_with_private_uncle_p $object_id]
set nephew_p [km_is_nephew_p $object_id]
set write_p [km_conn write_p]
set admin_p [km_conn admin_p]
set archived_p [km_conn archived_p]
set publish_p [km_conn publish_p]
set package_id [ad_conn package_id]
set approval_p [km_static approval_p $package_id]
set in_review_p [km_conn in_review_p]

if { !$admin_p && (!$write_p || $archived_p) } { return }

if { [km_conn public_p] } {
    ad_return_complaint 1 "This $object_type is already public."
    return
}

if { $approval_p && !$publish_p && !$admin_p } {
    ad_returnredirect "object-propose-public?[export_url_vars object_id category_id return_url]"
    return
}

set form_vars [export_form_vars object_id category_id return_url task_id msg]

set error 0
template::multirow create nephews object_id object_name in_review_p question_id question_name object_type object_type_plural
template::multirow create data_missing_questions question_id question_name

set ancestor_object_type ""
set ancestor_object_type_plural ""

if {$child_p} {
    set parent_object_type_id [thd [km_get_parent $object_id]]
    set ancestor_object_type [km_static object_type_pretty_name $parent_object_type_id]
    set ancestor_object_type_plural [km_static object_type_pretty_plural $parent_object_type_id]
    set title "It is not possible to publish $object_type_plural separately from the linked $ancestor_object_type"
    set error -1
} elseif {$private_nephew_p} {
    set uncle_object_type_id [thd [km_get_uncle $object_id]]
    set ancestor_object_type [km_static object_type_pretty_name $uncle_object_type_id]
    set ancestor_object_type_plural [km_static object_type_pretty_plural $uncle_object_type_id]
    set title "Not allowed to publish $object_type_plural without linked public $ancestor_object_type"
    set error -2
} elseif {$complete_p && $uncle_p && $nephews_p} {
    set nephew_question_list [km_uncle_nephew_questions $object_id]
    foreach nephew_question $nephew_question_list {
	set nephew_list [km_publishable_nephews $object_id [fst $nephew_question]]
	if {[llength $nephew_list] == 0} {
	    set error 1
	    lappend error_questions $nephew_question
	} else {
	    set target_object_type_id [km_static question_target_object_type_id [fst $nephew_question]]
	    set target_pretty_name [km_static object_type_pretty_name $target_object_type_id]
	    set target_pretty_plural [km_static object_type_pretty_plural $target_object_type_id]
	    set count 0
	    foreach nephew $nephew_list {
		if {!$in_review_p || $admin_p || [thd $nephew] == "t"} {
		    eval template::multirow append nephews $nephew $nephew_question $target_pretty_name $target_pretty_plural]
		    incr count
		}
	    }
	    if {$count == 0} {
		set error 1
		lappend error_questions $nephew_question
	    }
	}
    }
    if {!$error} {
	set title "Select Linked Objects for Publishing"
    } else {
	set title "Required Data Missing in Linked Objects"
	foreach nephew_question $error_questions {
	    eval template::multirow append data_missing_questions $nephew_question
	}
    }
} elseif {$complete_p} {
    set title "Confirm Object Publishing"
} else {
    set title "Required Data Missing"
    set error 2
}

set awarded_p [db_string object_awarded_shares {
    select count(*) from ir_share_transactions 
    where cc_name = 'kl_object_publish' and object_id = :object_id
}]

template::multirow create shares bank_id bank_name currency amount balance
if {!$awarded_p && !$nephew_p} {
    foreach bank_id [incentives::bank_list -user_id $author_id $package_id] {
	set amount [incentives::charge_value -key $object_type_id kl_object_publish $bank_id]
	if {$amount > 0} {
	    db_1row get_currency_name {
		select pretty_name as bank_name, currency_name
		from ir_banks
		where bank_id = :bank_id
	    }
	    template::multirow append shares $bank_id $bank_name $currency_name $amount [expr $amount+[thd [incentives::account_balance -user_id $author_id $bank_id]]]
	}
    }
}

set_the_usual_klib_context_bar $object_id $category_id
append_context_bar_data "Make public"

# Double click.
set dc_export [doubleclick::signature_html]

ad_return_template
