ad_page_contract {
    object-view.tcl

    Display the completed content of an object.

    @author Timo Hentschel (timo@arsdigita.com)
    $Id: object-view.tcl,v 1.1 2003/07/02 12:19:43 peterm Exp $
} {
    object_id:integer
    {category_id ""}
    {path_id:integer ""}
} -properties {
    title:onevalue
    write_feedback_of_object:onevalue
    object_id:onevalue
    object_type_id:onevalue
    pretty_type:onevalue
    date_created:onevalue
    public_p:onevalue
    checkoff_date:onevalue
    pretty_checkoff_date:onevalue
    date_last_modified:onevalue
    author_id:onevalue
    author_name:onevalue
    length_linked_objects:onevalue
    object_list:onevalue
    read_only_child_p:onevalue
    object_access_toolbar:onevalue
    display_object_data:multirow
    linked_urls_list:onevalue
    workspace_toolbar:onevalue
    access_count:onevalue
    access_count_month:onevalue
    edit_p:onevalue
    admin_p:onevalue
    delete_p:onevalue
    change_owner_p:onevalue
    change_publisher_p:onevalue
    user_id:onevalue
    reuse_p:onevalue
    comments:multirow
    path_id:onevalue
    category_id:onevalue
    subsite_url:onevalue
    subsite_name:onevalue
    community_id:onevalue
    public_p:onevalue
    bookmarked_p:onevalue
    shopping_cart_p:onevalue
    child_p:onevalue
    nephew_p:onevalue
    application_p:onevalue
    application_form_vars:onevalue
    ancestor_object_id:onevalue
    ancestor_object_type:onevalue
    ancestor_name:onevalue
    ancestor_public_p:onevalue
    ancestor_archived_p:onevalue
    ancestor_review_p:onevalue
    publisher_id:onevalue
    publisher_name:onevalue
    ot_create_p:onevalue
    ot_copy_p:onevalue
    ot_archive_p:onevalue
    review_p:onevalue
    task_id:onevalue
    archived_p:onevalue
    approval_p:onevalue
    review_state:onevalue
    create_p:onevalue
}

set user_id [ad_conn user_id]
set package_id [ad_conn package_id]
set instance_name [ad_conn instance_name]

if ![km_check_object_id $object_id] { return }

set object_type_id [km_conn object_type_id]
set read_only_child_p 0
set pretty_type [km_static object_type_pretty_name $object_type_id]
set pretty_type_plural [km_static object_type_pretty_plural $object_type_id]
set graphic [km_static object_type_graphic $object_type_id]

array set node [site_node_closest_ancestor_site_node "acs-subsite"] 
set community_id $node(package_id) 
set subsite_url $node(url)
set subsite_name $node(instance_name)
set public_p [km_conn public_p]
set review_p [km_conn in_review_p]
set review_state [km_conn review_state]
set original_author_id [km_conn original_author_id]
set object_name	[km_conn object_name]
set stripped_name [sn_striphtml $object_name]
set approval_p [km_static approval_p $package_id]
set ancestor_object_id 0
set publisher_id [km_conn publisher_id]
if ![empty_string_p $publisher_id] {
    set publisher_name [db_string get_publisher_name {select first_names || ' ' || last_name from users where user_id = :publisher_id}]
} else {
    set publisher_name ""
}

# Determine which permissions the viewing user has.
set admin_p [km_conn admin_p]
set publish_p [km_conn publish_p]
set edit_p [km_conn write_p]
set delete_p [km_conn delete_p]
set create_p [km_conn create_p]
set archived_p [km_conn archived_p]
set ot_archive_p [km_static object_type_archive_p $object_type_id]
set ot_create_p [km_static object_type_create_p $object_type_id]
set ot_copy_p [km_static object_type_copy_p $object_type_id]
set publisher_id [km_conn publisher_id]
set title "$pretty_type: $stripped_name"

if { ($user_id == $original_author_id && !$archived_p) || $admin_p } {
    set change_owner_p 1
} else {
    set change_owner_p 0
}

if { $admin_p || $publish_p || $publisher_id == $user_id} {
    set change_publisher_p 1
} else {
    set change_publisher_p 0
}

## get the task_id of the workflow task if object is in review
if { $review_p } {
    set task_id [db_string get_workflow_task {
	select distinct(ut.task_id)
	from wf_cases c, wf_user_tasks ut
	where c.object_id = :object_id
	and c.case_id = ut.case_id
	and c.state = 'active'
	and rownum = 1
	order by ut.task_id desc
    } -default ""]
} else {
    set task_id ""
}

# Determine how often the object has been viewed in total, and this month.
if { $user_id } { km_count_object_view $user_id $object_id }

set access_count [km_conn access_total]
set access_count_month [km_conn access_month]

# This object may have some parents.
# The proc returns pairs of parent_object_id and the object name
set ancestors [reverse [km_get_object_ancestors $object_id]]

if ![null_p $ancestors] {
    set ancestor [fst $ancestors]
    set ancestor_object_id [fst $ancestor]
    set ancestor_object_type [km_static object_type_pretty_name [thd $ancestor]]
    set ancestor_name [snd $ancestor]
    db_1row get_public_archived_status {
	select decode(public_p,'t',1,0) as ancestor_public_p,
	       decode(archived_p,'t',1,0) as ancestor_archived_p,
	       decode(in_review_p,'t',1,0) as ancestor_review_p
	from sn_objects
	where object_id = :ancestor_object_id
    }

    if { !$admin_p } {
	set ancestor_object_type_id [thd $ancestor]
	if {[llength $ancestors] > 1} {
	    set ancestor_linked_object_id [fst [snd $ancestors]]
	} else {
	    set ancestor_linked_object_id $object_id
	}
	set ancestor_question_state [db_string object_view_10 "
	    select qm.question_state
	    from sn_question_object_type_map qm, sn_links l, sn_question_link_map map
	    where l.object_id_a = :ancestor_object_id
	    and l.object_id_b = :ancestor_linked_object_id
	    and l.link_id = map.link_id
	    and map.question_id = qm.question_id
	    and qm.object_type_id = :ancestor_object_type_id"]
	if { $ancestor_question_state == "read-only" } {
	    set edit_p 0
	    set read_only_child_p 1
	}
    }
    set child_p 1
} else {
    set child_p 0
}

set nephew_p [km_is_nephew_p $object_id]

if {$child_p && !$nephew_p} {
    if {$ot_archive_p} {
	set return_url "object-view?[export_url_vars object_id]"
	if { $archived_p && $edit_p } {
	    set object_access_toolbar "<font color=red>\[Archived\]</font> <a href=\"object-unarchive?[export_url_vars object_id category_id return_url]\">Remove from archive</a>"
	} else {
	    set object_access_toolbar "<a href=\"object-archive?[export_url_vars object_id category_id return_url]\">Put in archive</a>"
	}
    } else {
	set object_access_toolbar ""
    }
} else {
    set object_access_toolbar [km_object_access_toolbar $object_id $category_id]
}

if {$nephew_p && [string equal [km_get_presentation_type_of_nephew_question $object_id] "application"]} {
    set application_p 1
    set resource_req_id [fst [km_get_uncle $object_id]]
    set return_url "object-view?[ad_conn query]"
    set application_form_vars [export_form_vars object_id category_id resource_req_id return_url]
} else {
    set application_p 0
    set application_form_vars ""
}

if { ![empty_string_p $path_id] && [path_valid_p $path_id] } {
    set path_values [get_path_values -action_only_p 0 $path_id]
    set last_path_id [get_last_path $path_id]
    set browse_type [snd $path_values]
    set browse_category [lindex $path_values 3]

    set browse_type "browse-one-type?[export_url_vars object_type_id]&path_id=$last_path_id&$browse_type"
    set browse_category "browse-one-category?[export_url_vars object_type_id category_id]&path_id=$last_path_id&$browse_category"
} else {
    set browse_type "browse-one-type?[export_url_vars object_type_id]"
    set browse_category "browse-one-category?[export_url_vars object_type_id category_id]"
}

# Set up the context bar. This is so complicated because we
# not only provide a link to browse-one-type.tcl, but also
# to browse-one-category.tcl (if needed).

## if {$child_p && !$nephew_p} {
##    set workspace_toolbar ""
##} else {
    set workspace_toolbar [km_workspace_toolbar $object_id $category_id $child_p]
##}
set_the_usual_klib_context_bar $object_id $category_id

if { $public_p } {
    set write_feedback_of_object [get_feedback_statistics_for_object $object_id feedback_statistics]
} else {
    set write_feedback_of_object 0
    set feedback_statistics ""
}

# Display questions and answers (omitting unanswered questions).
set date_created [util_AnsiDatetoPrettyDate [km_conn creation_date]]
set date_last_modified [util_AnsiDatetoPrettyDate [km_conn last_modified]]
set checkoff_date [km_conn user_checkoff_date]
set pretty_checkoff_date [util_AnsiDatetoPrettyDate [km_conn user_checkoff_date]]

set author_name [km_conn original_author_name]
set author_id [km_conn original_author_id]

# Retrieve all data for this object as a list of lists in this Special Form:
# question_id, pretty_question, abstract_data_type, value
set object_data [km_get_object_data -answers_only_p 0 $object_id]

template::multirow create display_object_data object_id question_id abstract_data_type branch_p root_branch_p pretty_question value edit_p

foreach item $object_data {
    set value [fst $item]
    set question_id [snd $item]
    set pretty_question [thd $item]
    set abstract_data_type [lindex $item 3]
    set branch_p [lindex $item 4]
    set root_branch_p [lindex $item 5]

    if {[empty_string_p $branch_p]} {
        set branch_p 0
}
    if { !$branch_p || [km_active_path_p $question_id $object_id $object_type_id] } {
        template::multirow append display_object_data $object_id $question_id $abstract_data_type $branch_p $root_branch_p $pretty_question $value $edit_p
    }
}

set linked_objects [km_get_linked_objects -to_p 1 -reference_links_p 0 -family_members_p 0 $user_id $object_id]
if {[llength $linked_objects] <= 1} {
    set linked_objects ""
}

set reuse_p 0
if { $public_p && $original_author_id != $user_id } {
    # Prevent private objects from getting shares.
    set reuse_p 1
}

get_feedback_for_object $object_id comments

if {![empty_string_p $category_id] && ![string equal $category_id none]} {
    query get_category_tree_info tree_info onerow {
       select gt.tree_name, gt.tree_id, c.short_name
         from sw_category_dim scd, generic_trees gt, categories c
        where scd.object_id = :category_id
          and scd.tree_id = gt.tree_id
          and c.category_id = :category_id
    }
    set search_contexts [list \
        [list "lcat:$object_type_id,$tree_info(tree_id),$category_id" "$tree_info(short_name)"] \
        [list "ltype:$object_type_id" "$pretty_type_plural"] \
        [list "lib:$package_id" "$instance_name"] \
        [list "sc:$community_id" "$subsite_name"] \
                             [sws_km_site_search_context]]
} else {
    set search_contexts [list \
        [list "ltype:$object_type_id" "$pretty_type_plural"] \
        [list "lib:$package_id" "$instance_name"] \
        [list "sc:$community_id" "$subsite_name"] \
                             [sws_km_site_search_context]]
}

ad_return_template
