ad_page_contract {
    /www/library/one-question-edit-2.tcl

    First save the data from one-question-edit.tcl, then redirect the user
    to his destination.

    @cvs-id $Id: one-question-edit-2.tcl,v 1.2 2005/03/01 00:01:14 jeffd Exp $
} {
    question_id:integer
}
csrf::authenticate

set key_values {}
set_the_usual_form_variables
set button_name ""
set new_question ""
set branch_id ""
set action ""

# We need some information about this question
set pretty_name [km_static question_pretty_name $question_id]
set abstract_data_type [km_static question_abstract_data_type $question_id]

# expecting: object_id path_id 
# In the form: the submit button name, the question's column_name 
# and its value to be stored in the database

set form_size [ns_set size $Vform]

# Go through the form and pick out the data we need for the path from
# the data that needs to go to Oracle.
set counter 0

while { $counter < $form_size } {
    
    set key [ns_set key $Vform $counter]
    set value [ns_set value $Vform $counter]
    incr counter

    # Isolate the general name of this form field.
    # A submit button has the form general_name.question_id 

    regexp {(.*)\.} $key match button_name

    # See if this is the form's submit button.
    if { [lsearch [km_get_button_names] $button_name] >= 0 } {
	regexp {\.(.*)} $key match button_target

	# This IS the form's submit button which sets the path action. 
	# We want to get the question_id out of it as well.

	set action $button_name
	
	# The second half of the submit button name is the question_id
	# for the next/previous question to be edited
	if { $button_name == "next_question" || $button_name == "previous_question" } {
	    set new_question_id $button_target
	}

    } else {
	
	switch $key {
	    "path_id" { set path_id $value }
            "object_id" { set object_id $value }
	    "object_type_id" { set object_type_id $value }
	    "parent_object_id" { set parent_object_id $value }
	    "uncle_object_id" { set uncle_object_id $value }
	    "question_id" { set question_id $value }
	    "category_id" { set category_id $value }
	    "default" { lappend key_values [list $key $value] }
	}
    }
}

if {![km_check_object_id -check_edit_p 1 $object_id]} { 
    return 
}

set user_id [auth::require_login]
set archived_p [km_conn archived_p]
set admin_p [km_conn admin_p]
set all_types_p 0
if {$admin_p && $archived_p} {
    set all_types_p 1
}

set question [km_get_question -all_types_p $all_types_p $question_id $object_type_id]
set question_state [lindex $question 3]

if ![info exists path_id] { set path_id "" }

if {[lsearch [list category other_category option] $abstract_data_type] != -1} {
    set valid_keys [km_filter_valid_keys $key_values]
    if {![null_p $valid_keys] && [empty_string_p [snd [fst $valid_keys]]]} {
	set key_values [list]
    }
}

if { $action == "delete" } {
    set results [km_delete_object_data $object_id $button_target]
}

if { $action == "delete" || ($question_state == "read-only" && !$admin_p) } {
    # If the user cannot edit or has deleted the content, 
    # then there won't be bad input, and nothing should be saved.
    set results ""
} elseif {![null_p [km_filter_valid_keys $key_values]] || [string equal $abstract_data_type date]} {

    # Do some error checking
    set key_values [km_sort_form $key_values]
    set bad_input [km_check_input -required_p 1 $object_id $key_values]

    if ![null_p $bad_input] {
	ad_return_complaint [fst $bad_input] [snd $bad_input]
	return
    } else {
	# Save the data
	set results [km_save_object_data $object_id $user_id $key_values]
    }

} elseif { [string equal $abstract_data_type "category"] || [string equal $abstract_data_type "other_category"] } {

    # We have to handle this differently because it is possible that
    # no variable is carried thru from the one-question-edit.tcl, and
    # yet this variable may be required.

    if { [elem_p $question_id [km_required_questions $object_id]] } {
        # Some important variable has not passed that is required
	ad_return_complaint 1 "<li>You must supply a value for $pretty_name"
	return
    } else {
	# this isn't a required field
	km_uncategorize_object $object_id $question_id
	if { $abstract_data_type == "other_category" } {
	    km_save_object_content -html_p 0 $question_id $object_id $user_id ""
	}
	set results {}
    }
} elseif { $abstract_data_type == "option" } {
     
    if { [elem_p $question_id [km_required_questions $object_id]] } {
        # Some important variable has not passed that is required
	ad_return_complaint 1 "<li>You must supply a value for $pretty_name"
	return
    } else {
	# this isn't a required field so we need to save an empty string
	set results [km_unoptionize_object $object_id $question_id]
    }

} else {
    # If the form has no input fields then there won't be bad input.
    set results ""
}

km_check_public_status_after_edit $object_id

if [empty_string_p [snd $results]] {
    # If there was no message then we can assume the editing was a success. 
    # So where do we go now?

    switch $action {
	"previous_question" -
	"next_question" {
	    
	    # First, this question might lead off to a branch
	    if { [lsearch {"option" "category" "other_category" "integer" "text"} $abstract_data_type] >= 0 && 
		 $action == "next_question" } {
		if { $question_state == "read-only" && !$admin_p } {
		    # There was no form on one-question-edit.tcl.
		    # So we have to retrieve the answer from the database.

		    set question_fields {question_id pretty_name abstract_data_type question_state}
		    set question [list $question_fields [list $question_id $pretty_name $abstract_data_type $question_state]]
		    set answer [fst [km_get_object_data -questions $question $object_id]]
		} else {
		    set answer [snd [fst $key_values]]
		}
		set branch_id [km_next_branch -abstract_data_type $abstract_data_type $question_id $answer $object_type_id]
		if ![empty_string_p $branch_id] { 
		    set new_question_id $branch_id 
		} elseif { [km_root_branch_p $question_id] } {
		    set new_question_id [fst [km_get_question -next_p 1 -all_types_p $all_types_p $question_id $object_type_id]]
		}
	    }
	    
	    # Set the destination
	    if ![empty_string_p $new_question_id] {
		set question_id $new_question_id
		set new_abstract_data_type [km_static question_abstract_data_type $question_id]   
		
		if { $new_abstract_data_type == "object_link" } {
		    set destination "object-link?[export_url_vars object_id category_id question_id]"
		} elseif ![empty_string_p $question_id] {
		    set destination "one-question-edit?[export_url_vars object_id category_id question_id parent_object_id]"
		} else {
		    set destination "object-view?[export_url_vars object_id category_id]"
		}
	    } else {
		# redirect to answered questions if the answer to this branch
		# question doesn't lead to a branch and this question
		# is the last question
		set destination "object-view?edit_p=1&[export_url_vars object_id category_id parent_object_id]"
	    }
	}
	"all_questions" { 
	    set destination "questions?view_questions=all&[export_url_vars object_id category_id parent_object_id]"
	}
	"answered" { 
	    set destination "object-view?edit_p=1&[export_url_vars object_id category_id parent_object_id]"
	}
	"unanswered" {
	    set destination "questions?view_questions=unanswered&[export_url_vars object_id category_id parent_object_id]"
	}
	"add_web_ref" {
	    set return_url "one-question-edit?[export_url_vars object_id category_id question_id]"
	    set path_vars [list question_id $button_target object_id $object_id category_id $category_id]
	    set path_id [init_path $path_vars $return_url $action]
	    set destination "add-web-ref?[export_url_vars path_id]"
	}
	"add_sn_ref" { 
	    set return_url "one-question-edit?[export_url_vars object_id category_id question_id]"
	    set path_vars [list question_id $button_target object_id $object_id category_id $category_id]
	    set path_id [init_path $path_vars $return_url $action]
            array set node [site_node_closest_ancestor_site_node "acs-subsite"] 
            set community_id $node(package_id) 
            set destination "linking/?community_id=$community_id&source_id=$object_id&pass=$path_id"
	}
	"add_user_ref" {
	    set target "user-link-add"
	    set passthrough {path_id}
	    set keyword [set user_link.$button_target]
	    set return_url "one-question-edit?[export_url_vars object_id category_id question_id]"
	    set path_vars [list question_id $button_target object_id $object_id category_id $category_id]
	    set path_id [init_path $path_vars $return_url $action]

            set context_bar [list "one-question-edit?[export_url_vars object_id category_id question_id]" "Edit"]
	    set destination "user-search?[export_url_vars keyword target passthrough path_id context_bar object_id category_id]"
	}
	"add_content_ref" {
	    set target "content-link-add"
	    set query_string $content_query
	    set table_name $content_table
	    set return_url "one-question-edit?[export_url_vars object_id category_id question_id]"
	    set path_vars [list question_id $button_target object_id $object_id category_id $category_id]
	    set path_id [init_path $path_vars $return_url $action]
	    set destination "/shared/sw-search-2?[export_url_vars query_string table_name target path_id]"
	}
	"add_nephew" {
	    set return_url "one-question-edit?[export_url_vars object_id category_id question_id path_id]"
	    set path_vars [list question_id $button_target uncle_object_id $object_id category_id $category_id]
	    set path_id [init_path $path_vars $return_url $action]
	    set destination "object-edit?[export_url_vars path_id]&uncle_object_id=$object_id&question_id=$button_target"
	}
	"add_child" {
	    set return_url "one-question-edit?[export_url_vars object_id category_id question_id path_id]"
	    set path_vars [list question_id $button_target parent_object_id $object_id category_id $category_id]
	    set path_id [init_path $path_vars $return_url $action]
	    set destination "object-edit?[export_url_vars path_id]&parent_object_id=$object_id&question_id=$button_target"
	}
	"copy_nephew" {
	    set return_url "one-question-edit?[export_url_vars object_id category_id question_id path_id]"
	    set path_vars [list question_id $button_target uncle_object_id $object_id category_id $category_id]
	    set path_id [init_path $path_vars $return_url $action]
	    set destination "object-copy?[export_url_vars path_id category_id]&parent_id=$object_id&question_id=$button_target"
	}
	"copy_child" {
	    set return_url "one-question-edit?[export_url_vars object_id category_id question_id path_id]"
	    set path_vars [list question_id $button_target parent_object_id $object_id category_id $category_id]
	    set path_id [init_path $path_vars $return_url $action]
	    set destination "object-copy?[export_url_vars path_id category_id]&parent_id=$object_id&question_id=$button_target"
	}
	default { set destination "questions?view_questions=all&[export_url_vars object_id category_id]" }
    }
} else {
    set destination "/global/error.html"
}

ad_returnredirect $destination
