ad_page_contract {
    /www/library/questions.tcl

    Let the user edit an object. Display all or only the unanswered questions.

    @cvs-id $Id: questions.tcl,v 1.2 2005/03/01 00:01:15 jeffd Exp $
} {
    object_id:notnull,integer
    {path_id:integer ""}
    {view_questions all}
    {category_id ""}
} -properties {
    title:onevalue
    graphic:onevalue
    graphic_width:onevalue
    question_state_deprecated:onevalue
    question_state_read_only:onevalue
    view_question:onevalue
    object_id:onevalue
    author_id:onevalue
    author_name:onevalue
    questions:multirow
    child_p:onevalue
    object_access_toolbar:onevalue
    public_p:onevalue
    pretty_type_name:onevalue
    create_p:onevalue
    public_p:onevalue
    pretty_type_name:onevalue
    user_id:onevalue
    admin_p:onevalue
    ancestor_name:onevalue
    ancestor_object_type:onevalue
    ancestor_object_id:onevalue
    ancestor_public_p:onevalue
    ancestor_archived_p:onevalue
    ancestor_review_p:onevalue
    publisher_id:onevalue
    publisher_name:onevalue
    change_owner_p:onevalue
    change_publisher_p:onevalue
    ot_create_p:onevalue
    ot_copy_p:onevalue
    ot_archive_p:onevalue
    review_p:onevalue
    task_id:onevalue
    archived_p:onevalue
    approval_p:onevalue
    review_state:onevalue
    delete_p:onevalue
}

set user_id [auth::require_login]
set package_id [ad_conn package_id]
set dots ""
if ![km_check_object_id -check_edit_p 1 $object_id] { return }

set object_type_id [km_conn object_type_id]
set object_name [km_conn object_name]
set create_p [km_conn create_p]
set public_p [km_conn public_p]
set admin_p [km_conn admin_p]
set publish_p [km_conn publish_p]
set delete_p [km_conn delete_p]
set archived_p [km_conn archived_p]
set edit_p [km_conn write_p]
set review_p [km_conn in_review_p]
set review_state [km_conn review_state]
set approval_p [km_static approval_p $package_id]
set ot_archive_p [km_static object_type_archive_p $object_type_id]
set author_name [km_conn original_author_name]
set ot_create_p [km_static object_type_create_p $object_type_id]
set ot_copy_p [km_static object_type_copy_p $object_type_id]
set author_id [km_conn original_author_id]
set publisher_id [km_conn publisher_id]

if ![empty_string_p $publisher_id] {
    set publisher_name [db_string get_publisher_name {select first_names || ' ' || last_name from users where user_id = :publisher_id}]
} else {
    set publisher_name ""
}

if { ($user_id == $author_id && !$archived_p) || $admin_p } {
    set change_owner_p 1
} else {
    set change_owner_p 0
}

if { $admin_p || $publish_p || $publisher_id == $user_id} {
    set change_publisher_p 1
} else {
    set change_publisher_p 0
}

## get the task_id of the workflow task if object is in review
if { $review_p } {
    set task_id [db_string get_workflow_task {
	select distinct(ut.task_id)
	from wf_cases c, wf_user_tasks ut
	where c.object_id = :object_id
	and c.case_id = ut.case_id
	and c.state = 'active'
	and rownum = 1
	order by ut.task_id desc
    } -default ""]
} else {
    set task_id ""
}

set graphic [km_static object_type_graphic $object_type_id]
set graphic_width [library_icon_width]
set question_state_deprecated [km_pretty_question_state "deprecated"]
set question_state_read_only [km_pretty_question_state "read-only"]

set title "[km_static object_type_pretty_name $object_type_id]: $object_name "
set check_img "<img src=\"graphics/check.gif\" width=16 height=16>"
set check_gray_img "<img src=\"graphics/check-gray.gif\" width=16 height=16>"
set pretty_type_name [km_static object_type_pretty_name $object_type_id]

# This object may have some parents.
# The proc returns pairs of parent_object_id and the object name
set ancestors [reverse [km_get_object_ancestors $object_id]]
if ![null_p $ancestors] {
    set direct_ancestor [fst $ancestors]
    set ancestor_object_id [fst $direct_ancestor]
    set ancestor_name [snd $direct_ancestor]
    set ancestor_object_type [km_static object_type_pretty_name [thd $direct_ancestor]]

    db_1row get_public_archived_status {
	select decode(public_p,'t',1,0) as ancestor_public_p,
	       decode(archived_p,'t',1,0) as ancestor_archived_p,
	       decode(in_review_p,'t',1,0) as ancestor_review_p
	from sn_objects
	where object_id = :ancestor_object_id
    }
    set child_p 1
} else {
    set child_p 0
}

set_the_usual_klib_context_bar $object_id $category_id
append_context_bar_data "Questions"
set nephew_p [km_is_nephew_p $object_id]

if {$child_p && !$nephew_p} {
    if {$ot_archive_p} {
	set return_url "object-view?[export_url_vars object_id category_id]"
	if { $archived_p && $edit_p } {
	    set object_access_toolbar "<font color=red>\[Archived\]</font> <a href=\"object-unarchive?[export_url_vars object_id category_id return_url]\">Remove from archive</a>"
	} else {
	    set object_access_toolbar "<a href=\"object-archive?[export_url_vars object_id category_id return_url]\">Put in archive</a>"
	}
    } else {
	set object_access_toolbar ""
    }
} else {
    set object_access_toolbar [km_object_access_toolbar $object_id $category_id]
}

# Get the questions that apply to this object type and indicate if
# they have been answered or are links
set question_states [list active deprecated read-only]
if {$archived_p} {
    lappend question_states invisible
}
set question_list [km_get_questions -question_states $question_states -all_properties_p 1 -root_node_p 1 -branch_children_p 1 -object_type_id $object_type_id -answered_p 1 -object_id $object_id]

set key [head $question_list]
set question_list [tail $question_list]
set question_ids [fst [transpose [tail $question_list]]]

# Go thru each question and decide to display it or not

set abstract_data_type_ix [lsearch $key abstract_data_type]
set question_state_ix [lsearch $key question_state]
set branch_p_ix [lsearch $key branch_p]
set root_branch_p_ix [lsearch $key root_branch_p]
set entry_explanation_ix [lsearch $key entry_explanation]
set question_id_ix [lsearch $key question_id]
set question_name_ix [lsearch $key pretty_name]
set mandatory_p_ix [lsearch $key mandatory_p]
set answered_p_ix [lsearch $key answered_p]

set questions_with_links [db_list_of_lists questions_10 {select unique question_id from (
        select map.question_id,
        acs_permission.permission_p(l.object_id_b, :user_id, 'sw_read') sw_read_p, 
        (select decode(count(*),0,'f','t') from dual where exists     
        (select decode(count(*),'f','t') from dual where exists
        (select 1
        from membership_rels mr_lm
        where mr_lm.person_id = :user_id
        and sn_group.group_list_members_p(mr_lm.group_id, l.object_id_b) > 0 ))) as user_list_p
        from sn_links l, sn_question_link_map map, acs_objects ao
        where l.object_id_a = :object_id
        and l.object_id_b = ao.object_id
        and l.link_id = map.link_id
        ) where sw_read_p='t' or user_list_p='t'}]

set i 0
template::multirow create questions check destination_url question_name star arrow dots short_explanation question_state answered
foreach question $question_list {
    set abstract_data_type [lindex $question $abstract_data_type_ix]
    set question_state [lindex $question $question_state_ix]
    set branch_p [lindex $question $branch_p_ix]
    set root_branch_p [lindex $question $root_branch_p_ix]
    set arrow ""
    set destination "one-question-edit?"
    set explanation [lindex $question $entry_explanation_ix]
    set question_id [lindex $question $question_id_ix]
    set question_name [lindex $question $question_name_ix]
    set mandatory_p [lindex $question $mandatory_p_ix]

    # I think for questions of abstract data type options and categories are
    # better checked in single queries.
    set answered_p 0
    switch $abstract_data_type {
        "text" -
        "file" -
        "date" - 
        "integer" {
            set answered_p [lindex $question $answered_p_ix]
        }
        "object_link" - 
        "user_link" - 
        "content_link" {
            if {[lsearch $questions_with_links $question_id] < 0} {
                set answered_p 0
            } else {
                set answered_p 1
            }
        }
        "category" - "other_category" {
            set answered_p [db_string questions_20 {select decode(count(*),0,0,1) 
            from dual 
            where exists
            (select 'x'
            from categories c, sw_flat_cat fc, sw_object_category_map cm, sn_questions q
            where cm.object_id = :object_id
	    and q.question_id = :question_id
	    and fc.parent = q.node_id
	    and c.category_id = fc.child_category_id
	    and cm.category_id = c.category_id)}]
        }
        "option" { 
            set answered_p [db_string questions_20 {select decode(count(*),0,0,1) 
            from dual 
            where exists
            (select 'x'
            from sn_answer_options a, sn_object_option_map m
            where a.option_id =m.option_id
            and m.object_id = :object_id
            and a.question_id = :question_id)}]
        }
        "child_object" {
            set answer ""
            set child_ids [km_get_child_objects $object_id $question_id]
            foreach child_id $child_ids {
                lappend answer [km_get_object_summary $child_id]
            }
            set answered_p [km_answered_p $abstract_data_type $answer]
        }
        "nephew_object" {
            set answer ""
            set nephew_ids [km_get_nephew_objects $object_id $question_id]
            foreach nephew_id $nephew_ids {
                lappend answer [km_get_object_summary $nephew_id]
            }
            set answered_p [km_answered_p $abstract_data_type $answer]
        }
        "composite" {
            set child_ids [km_get_child_questions $question_id]
            if {![null_p $child_ids]} {
                set child_questions [km_get_questions -question_states {active deprecated read-only} -root_node_p 0 -question_ids $child_ids]
                set answer [km_get_object_data -questions $child_questions -answers_only_p 0 $object_id]
                set answered_p [km_answered_p $abstract_data_type $answer]
            } else {
                set answered_p 0
            }
        }
        default {
        ns_log Notice "Error in questions.tcl. I'm in the default branche and I shouldn't be there. Testing abstract_data_type $abstract_data_type of question $question_name"
        set answer [km_get_object_data -questions [list question_id $question_id] $object_id]
        set answered_p [km_answered_p $abstract_data_type $answer]
        }
    }


    if { $abstract_data_type == "object_link" || \
         $abstract_data_type == "user_link" || \
         $abstract_data_type == "content_link" || \
         $abstract_data_type == "child_object" || \
         $abstract_data_type == "nephew_object"} {
	set arrow "<img width=13 height=9 src=\"graphics/arrow.gif\">"
	set destination "one-question-edit?"
##	set destination "object-link?"
    }

    if { $answered_p } {
	if { $question_state == "active" } {
	    if { $abstract_data_type == "composite" } {
		# If it's a composite question, we only mark it
		# as answered if all children have been answered.

		set all_answered_p 1
		foreach item $answer {
		    set child_abstract_datatype [lindex $item 3]
		    set child_answer [fst $item]

		    if ![km_answered_p $child_abstract_datatype $child_answer] {
			set all_answered_p 0
			break
		    }
		}

		set check [ad_decode $all_answered_p 1 $check_img $check_gray_img]
		set answered [ad_decode $all_answered_p 1 2 1]

	    } else {
		set check $check_img
                set answered 2
	    }
	} else {
	    set check "X"
            set answered 0
	}
    } else {
	set check "&nbsp;"
        set answered 0
    }
    
    # Set the question attributes using the key for the list variables
    if { $mandatory_p == "t" } { 
	set star "*" 
    } else {
	set star ""
    }
    if { $root_branch_p || $abstract_data_type == "composite" } { 
	set dots " ..."
    } else {
	set dots ""
    }

    set short_explanation [km_shorten_question $explanation]

    if {[empty_string_p $branch_p]} {set branch_p 0}
    if {$branch_p && [km_active_path_p $question_id $object_id $object_type_id]} {
	set branch_p 0
	set question_name "... $question_name"
    }

    if { ($question_state == "active" || $answered_p) &&
	 (($view_questions == "all" && !$branch_p) ||
	  ($view_questions == "unanswered" && !$answered_p && !$branch_p)) } {
              # Display the row

              template::multirow append questions $check "$destination[export_url_vars object_id category_id path_id question_id]" $question_name $star $arrow $dots $short_explanation $question_state $answered
    }

    incr i
}

ad_return_template
