ad_page_contract {
    /packages/library/www/resource-application-add-2.tcl

    Show a form where a user can volunteer for an XchangeTask 

    @author Timo Hentschel (timo@arsdigita.com)
    @cvs-id $Id: resource-application-add-2.tcl,v 1.2 2005/03/01 00:01:15 jeffd Exp $
} {
    object_id:notnull,integer
    resource_req_id:integer
    {category_id ""}
    {return_url "."}
    apply:optional
    cancel:optional
} -properties {
    title:onevalue
    graphic:onevalue
    graphic_width:onevalue
    form_vars:onevalue
    recipient:onevalue
    role:onevalue
    pretty_project_type:onevalue
    subject:onevalue
    system_url:onevalue
    user_id:onevalue
    contact_data:onevalue
    nationality:onevalue
    manager_email:onevalue
    role_select:onevalue
    role_other:onevalue
    working_area:onevalue
    first_language:onevalue
    second_language_id:onevalue
    third_language_id:onevalue
    second_language_prof_id:onevalue
    third_language_prof_id:onevalue
    other_language:onevalue
    languages:multirow
    language_proficiencies:multirow
    strengths:onevalue
    leadership:onevalue
    intercultural:onevalue
    comments:onevalue
    conditions:onevalue
    attachments:multirow
    dc_export:onevalue
    edit_p:onevalue
}
csrf::authenticate

set user_id [auth::require_login]
if ![km_check_object_id $object_id] { return }

if ![exists_and_not_null resource_req_id] {
    ad_return_complaint 1 "For some reason, resource_req_id has not been supplied with this request."
    return
}

if {[info exists cancel]} {
    ad_returnredirect $return_url
}

set demand_type_id [km_conn object_type_id]
set pretty_demand_type [km_static object_type_pretty_name $demand_type_id]
set demand_name [km_conn object_name]
set role [snd [fst [fst [km_get_object_data $resource_req_id]]]]
set graphic [km_static object_type_graphic $demand_type_id]
set graphic_width [library_icon_width]

set_the_usual_klib_context_bar $object_id $category_id
append_context_bar_data "Apply"

if ![km_check_object_id $resource_req_id] { return }
set project_type_id [km_conn object_type_id]
set pretty_project_type [km_static object_type_pretty_name $project_type_id]
set project_name [km_conn object_name]

set title "Volunteer for the $pretty_project_type \"$project_name\""
set subject "I apply for the $pretty_demand_type \"$demand_name\" of the $pretty_project_type \"$project_name\""

# All users that are linked to this Xchange task object
# will get a copy of the mail.

set recipient [sn_people_responsible $user_id $object_id]

set form_vars [export_form_vars object_id resource_req_id category_id return_url recipient]
set system_url [ad_url]

# Now let's get some info about the volunteer.

db_1row get_user_data {
    select email as user_email, first_names || ' ' || last_name as user_name
    from users
    where user_id = :user_id
}

set contact_data "Name: $user_name\nEmail: $user_email\n"
db_foreach get_global_questions {
    select q.question_id, q.question,
           decode ( q.category_tree_id, null, a.answer, c.short_name ) as answer
    from sn_reg_questions q, sn_reg_answers a, categories c
    where q.scope = 'site_wide'
       and q.deleted_p = 'f'
       and q.question_id = a.question_id(+)
       and a.category_id = c.category_id(+)
       and a.user_id(+) = :user_id
} {
    append contact_data "$question: $answer\n"
}

set edit_p 0
util_unlist [km_break_date [db_string get_sysdate {select sysdate from dual}]] from_year from_month from_day
set to_year $from_year
set to_month $from_month
set to_day $from_day
set nationality ""
set manager_email ""
set working_area ""
set first_language ""
set second_language_id ""
set third_language_id ""
set second_language_prof_id ""
set third_language_prof_id ""
set other_language ""
set strengths ""
set leadership ""
set intercultural ""
set comments ""
set conditions ""
template::multirow create attachments attachment_id filename title size

set package_id [ad_conn package_id]
set magic_ids_exist_p [db_0or1row get_magic_ids_for_category_trees {
    select role_magic_id, language_magic_id, proficiency_magic_id
    from psn_category_trees
    where package_id = :package_id
}]

if {!$magic_ids_exist_p} {
    ad_return_complaint 1 "Error in library setup. No magic ids for category trees for the application form found."
    return
}

if {!$role_magic_id} {
    ad_return_complaint 1 "No category tree assigned for roles. Please contact a library administrator."
    return
}
if {!$language_magic_id} {
    ad_return_complaint 1 "No category tree assigned for languages. Please contact a library administrator."
    return
}
if {!$proficiency_magic_id} {
    ad_return_complaint 1 "No category tree assigned for language proficiencies. Please contact a library administrator."
    return
}


set role_category_tree_assigned_p [db_0or1row role_category_tree {
    select octr.subtree_root_node_id as role_subtree_root_id,
           gt.root_node_id as role_orig_root_id
    from generic_trees gt,
    acs_rels ar,
    object_category_tree_rels octr
    where ar.object_id_one = :role_magic_id
    and   ar.object_id_two = gt.tree_id
    and   ar.rel_id = octr.rel_id
    and rel_type = 'object_category_tree_rel'}] 

if {!$role_category_tree_assigned_p} {
    ad_return_complaint 1 "No category tree assigned for roles. Please contact a library administrator."
    return
}

if { $role_subtree_root_id == $role_orig_root_id } {
    set role_level_correction 1
} else {
    set role_level_correction 0
}

set language_category_tree_assigned_p [db_0or1row language_category_tree {
    select octr.subtree_root_node_id as language_subtree_root_id,
           gt.root_node_id as language_orig_root_id
    from generic_trees gt,
    acs_rels ar,
    object_category_tree_rels octr
    where ar.object_id_one = :language_magic_id
    and   ar.object_id_two = gt.tree_id
    and   ar.rel_id = octr.rel_id
    and rel_type = 'object_category_tree_rel'}] 

if {!$language_category_tree_assigned_p} {
    ad_return_complaint 1 "No category tree assigned for languages. Please contact a library administrator."
    return
}

if { $language_subtree_root_id == $language_orig_root_id } {
    set language_level_correction 1
} else {
    set language_level_correction 0
}

set proficiency_category_tree_assigned_p [db_0or1row proficiency_category_tree {
    select octr.subtree_root_node_id as proficiency_subtree_root_id,
           gt.root_node_id as proficiency_orig_root_id
    from generic_trees gt,
    acs_rels ar,
    object_category_tree_rels octr
    where ar.object_id_one = :proficiency_magic_id
    and   ar.object_id_two = gt.tree_id
    and   ar.rel_id = octr.rel_id
    and rel_type = 'object_category_tree_rel'}] 

if {!$proficiency_category_tree_assigned_p} {
    ad_return_complaint 1 "No category tree assigned for language proficiencies. Please contact a library administrator."
    return
}

if { $proficiency_subtree_root_id == $proficiency_orig_root_id } {
    set proficiency_level_correction 1
} else {
    set proficiency_level_correction 0
}

set role_select "<select multiple name=role_id>"
set role_results [util_memoize [list swc_get_category_tree $role_subtree_root_id 0 {}]]
foreach role $role_results {
    util_unlist $role category_id category_name category_description
    append role_select "<option value=\"$category_id\">$category_name"
}
append role_select "</select>"

template::multirow create languages language_id name
set language_results [util_memoize [list swc_get_category_tree $language_subtree_root_id 0 {}]]
foreach language $language_results {
    util_unlist $language category_id category_name category_description
    template::multirow append languages $category_id $category_name
}

template::multirow create language_proficiencies prof_id name
set prof_results [util_memoize [list swc_get_category_tree $proficiency_subtree_root_id 0 {}]]
foreach prof $prof_results {
    util_unlist $prof category_id category_name category_description
    template::multirow append language_proficiencies $category_id $category_name
}

set role_other ""

set dc_export [doubleclick::signature_html]

db_release_unused_handles

ad_return_template resource-application
