ad_page_contract {
    /packages/library/www/resource-application-edit.tcl

    Show a form where a user can volunteer for an XchangeTask 

    @author Timo Hentschel (timo@arsdigita.com)
    @cvs-id $Id: resource-application-edit.tcl,v 1.2 2005/03/01 00:01:15 jeffd Exp $
} {
    application_id:notnull,integer
    {category_id ""}
    {return_url "."}
} -properties {
    title:onevalue
    graphic:onevalue
    graphic_width:onevalue
    form_vars:onevalue
    recipient:onevalue
    role:onevalue
    pretty_project_type:onevalue
    subject:onevalue
    system_url:onevalue
    user_id:onevalue
    contact_data:onevalue
    nationality:onevalue
    manager_email:onevalue
    role_select:onevalue
    role_other:onevalue
    working_area:onevalue
    first_language:onevalue
    second_language_id:onevalue
    third_language_id:onevalue
    second_language_prof_id:onevalue
    third_language_prof_id:onevalue
    languages:multirow
    language_proficiencies:multirow
    other_language:onevalue
    from_date_widget:onevalue
    to_date_widget:onevalue
    strengths:onevalue
    leadership:onevalue
    intercultural:onevalue
    comments:onevalue
    conditions:onevalue
    attachments:multirow
    dc_export:onevalue
    edit_p:onevalue
}

set user_id [auth::require_login]

set application_exists [db_0or1row get_application_data {
    select object_id, resource_req_id, recipient, subject, contact_data, nationality,
           manager_email, working_area, first_language, second_language_id,
           third_language_id, second_language_prof_id, role_other,
           third_language_prof_id, other_language, from_date, to_date, strengths,
           leadership, intercultural, comments, conditions, user_id as creator_id
    from psn_res_applications
    where application_id = :application_id
}]

if !$application_exists {
    ad_return_complaint 1 "There is no such application."
    return
}
if {$user_id != $creator_id} {
    ad_return_complaint 1 "This is not your application, therefore you are not allowed to see it."
    return
}

if ![km_check_object_id $object_id] { return }

util_unlist [km_break_date $from_date] from_year from_month from_day
util_unlist [km_break_date $to_date] to_year to_month to_day

set role_id_list [db_list get_role_list_of_application {
    select role_id from psn_res_application_roles where application_id = :application_id
}]

## attachments

set demand_type_id [km_conn object_type_id]
set pretty_demand_type [km_static object_type_pretty_name $demand_type_id]
set demand_name [km_conn object_name]
set role [snd [fst [fst [km_get_object_data $resource_req_id]]]]
set graphic [km_static object_type_graphic $demand_type_id]
set graphic_width [library_icon_width]

set_the_usual_klib_context_bar $object_id $category_id
append_context_bar_data "Apply"

if ![km_check_object_id $resource_req_id] { return }
set project_type_id [km_conn object_type_id]
set pretty_project_type [km_static object_type_pretty_name $project_type_id]
set project_name [km_conn object_name]

set title "Volunteer for a $pretty_project_type: $project_name"

set form_vars [export_form_vars application_id category_id return_url]
set system_url [ad_url]

set package_id [ad_conn package_id]
set magic_ids_exist_p [db_0or1row get_magic_ids_for_category_trees {
    select role_magic_id, language_magic_id, proficiency_magic_id
    from psn_category_trees
    where package_id = :package_id
}]

if {!$magic_ids_exist_p} {
    ad_return_complaint 1 "Error in library setup. No magic ids for category trees for the application form found."
    return
}
if {!$role_magic_id} {
    ad_return_complaint 1 "No category tree assigned for roles. Please contact a library administrator."
    return
}
if {!$language_magic_id} {
    ad_return_complaint 1 "No category tree assigned for languages. Please contact a library administrator."
    return
}
if {!$proficiency_magic_id} {
    ad_return_complaint 1 "No category tree assigned for language proficiencies. Please contact a library administrator."
    return
}

set role_category_tree_assigned_p [db_0or1row role_category_tree {
    select octr.subtree_root_node_id as role_subtree_root_id,
           gt.root_node_id as role_orig_root_id
    from generic_trees gt,
    acs_rels ar,
    object_category_tree_rels octr
    where ar.object_id_one = :role_magic_id
    and   ar.object_id_two = gt.tree_id
    and   ar.rel_id = octr.rel_id
    and rel_type = 'object_category_tree_rel'}] 

if {!$role_category_tree_assigned_p} {
    ad_return_complaint 1 "No category tree assigned for roles. Please contact a library administrator."
    return
}

if { $role_subtree_root_id == $role_orig_root_id } {
    set role_level_correction 1
} else {
    set role_level_correction 0
}

set language_category_tree_assigned_p [db_0or1row language_category_tree {
    select octr.subtree_root_node_id as language_subtree_root_id,
           gt.root_node_id as language_orig_root_id
    from generic_trees gt,
    acs_rels ar,
    object_category_tree_rels octr
    where ar.object_id_one = :language_magic_id
    and   ar.object_id_two = gt.tree_id
    and   ar.rel_id = octr.rel_id
    and rel_type = 'object_category_tree_rel'}] 

if {!$language_category_tree_assigned_p} {
    ad_return_complaint 1 "No category tree assigned for languages. Please contact a library administrator."
    return
}

if { $language_subtree_root_id == $language_orig_root_id } {
    set language_level_correction 1
} else {
    set language_level_correction 0
}

set proficiency_category_tree_assigned_p [db_0or1row proficiency_category_tree {
    select octr.subtree_root_node_id as proficiency_subtree_root_id,
           gt.root_node_id as proficiency_orig_root_id
    from generic_trees gt,
    acs_rels ar,
    object_category_tree_rels octr
    where ar.object_id_one = :proficiency_magic_id
    and   ar.object_id_two = gt.tree_id
    and   ar.rel_id = octr.rel_id
    and rel_type = 'object_category_tree_rel'}] 

if {!$proficiency_category_tree_assigned_p} {
    ad_return_complaint 1 "No category tree assigned for language proficiencies. Please contact a library administrator."
    return
}

if { $proficiency_subtree_root_id == $proficiency_orig_root_id } {
    set proficiency_level_correction 1
} else {
    set proficiency_level_correction 0
}

set role_select "<select multiple name=role_id>"
set role_results [util_memoize [list swc_get_category_tree $role_subtree_root_id 0 {}]]
foreach role $role_results {
    util_unlist $role category_id category_name category_description
    append role_select "<option value=\"$category_id\""
    if {[lsearch -exact $role_id_list $category_id] != -1} {
	append role_select " selected"
    }
    append role_select ">$category_name"
}
append role_select "</select>"

template::multirow create languages language_id name
set language_results [util_memoize [list swc_get_category_tree $language_subtree_root_id 0 {}]]
foreach language $language_results {
    util_unlist $language category_id category_name category_description
    template::multirow append languages $category_id $category_name
}

template::multirow create language_proficiencies prof_id name
set prof_results [util_memoize [list swc_get_category_tree $proficiency_subtree_root_id 0 {}]]
foreach prof $prof_results {
    util_unlist $prof category_id category_name category_description
    template::multirow append language_proficiencies $category_id $category_name
}

template::multirow create attachments attachment_id filename title size
db_foreach get_attachments {
    select attachment_id, filename, title as file_title, dbms_lob.getlength(attachment) as bytes
    from psn_attachments
    where application_id = :application_id
} {
    template::multirow append attachments $attachment_id $filename $file_title $bytes
}    

set edit_p 1
set dc_export [doubleclick::signature_html]

db_release_unused_handles

ad_return_template resource-application
