ad_page_contract {
    /packages/library/www/resource-application-save.tcl

    Save the input the user entered in the form and either add an attachment,
    sent the email or redirect to the return_url.

    @author Timo Hentschel (timo@arsdigita.com)
    @cvs-id $Id: resource-application-save.tcl,v 1.2 2005/03/01 00:01:15 jeffd Exp $
} {
    {application_id:integer ""}
    {object_id:integer ""}
    {resource_req_id:integer ""}
    {category_id ""}
    {return_url:notnull "."}
    {recipient:string_length(max|4000) ""}
    subject:string_length(max|1000)
    contact_data:string_length(max|4000)
    nationality:string_length(max|4000)
    manager_email:string_length(max|4000)
    {role_id:integer,multiple ""}
    role_other:string_length(max|1000)
    working_area:string_length(max|4000)
    first_language:string_length(max|1000)
    second_language_id:integer
    second_language_prof_id:integer
    third_language_id:integer
    third_language_prof_id:integer
    other_language:string_length(max|4000)
    from_date:array
    to_date:array
    strengths:string_length(max|4000)
    leadership:string_length(max|4000)
    intercultural:string_length(max|4000)
    comments:string_length(max|4000)
    conditions:string_length(max|4000)
    send:optional
    save:optional
    attach:optional
    cancel_form:optional
    dont_apply:optional
}
csrf::authenticate

set user_id [auth::require_login]
set ip_address [ad_conn peeraddr]
set package_id [ad_conn package_id]

if {![empty_string_p $resource_req_id] && ![km_check_object_id $resource_req_id]} { return }

if {[doubleclick::check_all] && [empty_string_p $application_id]} {
    set application_id [db_string get_application_id {
	select application_id
	from psn_res_applications
	where object_id = :object_id
	and resource_req_id = :resource_req_id
    } -default "0"]
}

if {[info exists cancel_form]} {
    ad_returnredirect $return_url
    return
}

if {[info exists dont_apply]} {
    db_transaction {
	db_dml delete_application_10 {delete from psn_res_application_roles where application_id = :application_id}
	db_dml delete_application_20 {delete from psn_attachments where application_id = :application_id}
	db_dml delete_application_30 {delete from psn_res_applications where application_id = :application_id}
    }
    ad_returnredirect $return_url
    return
}

if {![empty_string_p $application_id]} {
    set application_exists [db_0or1row check_existing_application {
	select user_id as application_user_id, sent_p
	from psn_res_applications
	where application_id = :application_id
    }]
    if !$application_exists {
	ad_return_complaint 1 "There is no such application."
	return
    }
    if {![string equal $user_id $application_user_id]} {
	ad_return_complaint 1 "This is not your application. Your are not allowed to edit it."
	return
    }
    if {$sent_p == "t"} {
	ad_return_complaint 1 "This application already got sent. There is no way to edit it later, but you can send a new application."
	return
    }
}

set form [ns_conn form]
set form_size [ns_set size $form]

set i 0
while {$i < $form_size} {
    set key [ns_set key $form $i]
    set value [ns_set value $form $i]
    if {[regexp {^delete([0-9]*)} $key match delete_attachment_id]} {
	break
    }
    incr i
}

set from_date_sql "$from_date(year)-$from_date(month)-$from_date(day)"
set to_date_sql "$to_date(year)-$to_date(month)-$to_date(day)"

if {![date_p $from_date_sql]} {
    set from_date_sql [db_string get_sysdate {select sysdate from dual}]
}
if {![date_p $to_date_sql]} {
    set to_date_sql [db_string get_sysdate {select sysdate from dual}]
}

if {![empty_string_p $role_other]} {
    set magic_ids_exist_p [db_0or1row get_magic_ids_for_category_trees {
	select role_magic_id
	from psn_category_trees
	where package_id = :package_id
    }]

    if {!$magic_ids_exist_p} {
	ad_return_complaint 1 "Error in library setup. No magic ids for category trees for the application form found."
	return
    }
    if {!$role_magic_id} {
	ad_return_complaint 1 "No category tree assigned for roles. Please contact a library administrator."
	return
    }

    set role_category_tree_assigned_p [db_0or1row role_category_tree {
	select octr.subtree_root_node_id as role_subtree_root_id,
	gt.root_node_id as role_orig_root_id
	from generic_trees gt,
	acs_rels ar,
	object_category_tree_rels octr
	where ar.object_id_one = :role_magic_id
	and   ar.object_id_two = gt.tree_id
	and   ar.rel_id = octr.rel_id
	and rel_type = 'object_category_tree_rel'}] 

    if {!$role_category_tree_assigned_p} {
	ad_return_complaint 1 "No category tree assigned for roles. Please contact a library administrator."
	return
    }

    if {[db_0or1row check_other_category_id {
	select  /*+INDEX(sw_category_dim sw_category_dim_long_name) */ category.long_name(object_id)   as name,
	object_id as other_category_id
	from  sw_category_dim
	where category.long_name(object_id) = 'Other'
	start with node_id = :role_subtree_root_id
	connect by prior node_id = parent_node_id
    }]} {
	if {[lsearch -exact $role_id $other_category_id] == -1} {
	    set role_other ""
	}
    }
}

db_transaction {

    if {![empty_string_p $resource_req_id]} {

	set application_id [db_exec_plsql create_new_application {
	    begin
	      :1 := acs_object.new (
	        object_type	   => 'psn_application',
	        creation_date	   => sysdate,
	        creation_user	   => :user_id,
	        creation_ip	   => :ip_address,
	        context_id	   => :resource_req_id
	      );
	    end;
	}]

	if ![km_check_object_id $object_id] { return }
	set project_type_id [km_conn object_type_id]
	set project_type [km_static object_type_pretty_name $project_type_id]
	set project_name [km_conn object_name]

	if ![km_check_object_id $resource_req_id] { return }
	set demand_type_id [km_conn object_type_id]
	set demand_type [km_static object_type_pretty_name $demand_type_id]
	set demand_name [km_conn object_name]

	set shortname [string range "Application for '$demand_name'" 0 199]
	set overview "Application for the $demand_type '$demand_name' of the $project_type '$project_name'"

	db_exec_plsql set_application_object_description {
	    begin
	    acs_object.new_description (
	      v_object_id         => :application_id,
	      v_shortname         => :shortname,
	      v_overview          => :overview,
	      v_overview_html_p   => 'f',
	      v_package_id        => :package_id
	    );
	    end;
	}

	db_dml insert_new_application {
	    insert into psn_res_applications
	    (application_id, object_id, resource_req_id, user_id, creation_date,
	    application_date, recipient, subject, contact_data, nationality, manager_email,
	    working_area, first_language, second_language_id, third_language_id,
	    second_language_prof_id, third_language_prof_id, role_other,
	    other_language, from_date, to_date, strengths, leadership, intercultural,
	    comments, conditions)
	    values
	    (:application_id, :object_id, :resource_req_id, :user_id, sysdate,
	    null, :recipient, :subject, :contact_data, :nationality, :manager_email,
	    :working_area, :first_language, :second_language_id, :third_language_id,
	    :second_language_prof_id, :third_language_prof_id, :role_other,
	    :other_language, to_date(:from_date_sql), to_date(:to_date_sql), :strengths,
	    :leadership, :intercultural, :comments, :conditions)
	}
    } else {
	db_dml update_application {
	    update psn_res_applications
	    set subject = :subject,
	    contact_data = :contact_data,
	    nationality = :nationality,
	    manager_email = :manager_email,
	    working_area = :working_area,
	    role_other = :role_other,
	    first_language = :first_language,
	    second_language_id = :second_language_id,
	    third_language_id = :third_language_id,
	    second_language_prof_id = :second_language_prof_id,
	    third_language_prof_id = :third_language_prof_id,
	    other_language = :other_language,
	    from_date = to_date(:from_date_sql),
	    to_date = to_date(:to_date_sql),
	    strengths = :strengths,
	    leadership = :leadership,
	    intercultural = :intercultural,
	    comments = :comments,
	    conditions = :conditions
	    where application_id = :application_id
	}
    }

    db_dml delete_old_roles {
	delete from psn_res_application_roles where application_id = :application_id
    }
    foreach single_role_id $role_id {
	db_dml insert_new_roles {
	    insert into psn_res_application_roles (application_id, role_id)
	    values (:application_id, :single_role_id)
	}
    }
}

db_release_unused_handles

if {[info exists attach]} {
    ad_returnredirect "resource-application-attach?[export_url_vars application_id category_id return_url]"
}

if {[info exists delete_attachment_id]} {
    db_dml delete_attachment {
	delete from psn_attachments where attachment_id = :delete_attachment_id and application_id = :application_id
    }
    ad_returnredirect "resource-application-edit?[export_url_vars application_id category_id return_url]"
}

if {[info exists send]} {
    ad_returnredirect "resource-application-send?[export_url_vars application_id category_id return_url]"
}
    
ad_returnredirect $return_url
