ad_page_contract {
    /packages/library/www/resource-application-send-2.tcl

    Mime encode attachments, send the email and mark the application as sent.
    send the email or redirect to the return_url.

    @author Timo Hentschel (timo@arsdigita.com)
    @cvs-id $Id: resource-application-send-2.tcl,v 1.2 2005/03/01 00:01:15 jeffd Exp $
} {
    application_id:integer,notnull
    {category_id ""}
    return_url:notnull
    send:optional
    edit:optional
}
csrf::authenticate

set user_id [auth::require_login]
set package_id [ad_conn package_id]

if {[info exists edit]} {
    ad_returnredirect "resource-application-edit?[export_url_vars application_id category_id return_url]"
    return
}

set boundary_string "------=_NextPart_000_0064_01C160A3.5C6A4A50"

set application_exists [db_0or1row get_application_data {
    select user_id as application_user_id, object_id, resource_req_id, sent_p,
           recipient, subject, contact_data, nationality, manager_email,
           working_area, first_language, second_language_id, role_other,
           third_language_id, second_language_prof_id,
           third_language_prof_id, other_language, from_date, to_date,
           strengths, leadership, intercultural, comments, conditions
    from psn_res_applications
    where application_id = :application_id
}]

if !$application_exists {
    ad_return_complaint 1 "There is no such application."
    return
}
if {![string equal $user_id $application_user_id]} {
    ad_return_complaint 1 "This is not your application. Your are not allowed to edit or sent it."
    return
}
if {$sent_p == "t"} {
    ad_return_complaint 1 "This application already got sent. There is no way to edit it later, but you can sent a new application."
    return
}

db_1row get_username {
    select first_names || ' ' || last_name as user_name, email as user_email
    from users
    where user_id = :user_id
}

db_1row get_project_and_demand_names {
    select o1.one_line_description as demand_name,
           o2.one_line_description as project_name,
           ot1.pretty_name as demand_type, ot2.pretty_name as project_type
    from sn_objects o1, sn_objects o2, sn_object_types ot1, sn_object_types ot2
    where o1.object_id = :object_id
    and o2.object_id = :resource_req_id
    and o1.object_type_id = ot1.object_type_id
    and o2.object_type_id = ot2.object_type_id
}

db_1row package_location {
    select site_node.url(sn.node_id) as library_url
    from site_nodes sn, acs_objects_description dsc
    where dsc.object_id = :object_id
    and sn.object_id = dsc.package_id
    and rownum = 1
}

set selected_role_ids [db_list get_application_roles {
    select role_id
    from psn_res_application_roles
    where application_id = :application_id
}]

set magic_ids_exist_p [db_0or1row get_magic_ids_for_category_trees {
    select role_magic_id, language_magic_id, proficiency_magic_id
    from psn_category_trees
    where package_id = :package_id
}]

if {!$magic_ids_exist_p} {
    ad_return_complaint 1 "Error in library setup. No magic ids for category trees for the application form found."
    return
}
if {!$role_magic_id} {
    ad_return_complaint 1 "No category tree assigned for roles. Please contact a library administrator."
    return
}
if {!$language_magic_id} {
    ad_return_complaint 1 "No category tree assigned for languages. Please contact a library administrator."
    return
}
if {!$proficiency_magic_id} {
    ad_return_complaint 1 "No category tree assigned for language proficiencies. Please contact a library administrator."
    return
}
set role_category_tree_assigned_p [db_0or1row role_category_tree {
    select octr.subtree_root_node_id as role_subtree_root_id
    from generic_trees gt,
    acs_rels ar,
    object_category_tree_rels octr
    where ar.object_id_one = :role_magic_id
    and   ar.object_id_two = gt.tree_id
    and   ar.rel_id = octr.rel_id
    and rel_type = 'object_category_tree_rel'
}]

if {!$role_category_tree_assigned_p} {
    ad_return_complaint 1 "No category tree assigned for roles. Please contact a library administrator."
    return
}

set language_category_tree_assigned_p [db_0or1row language_category_tree {
    select octr.subtree_root_node_id as language_subtree_root_id
    from generic_trees gt,
    acs_rels ar,
    object_category_tree_rels octr
    where ar.object_id_one = :language_magic_id
    and   ar.object_id_two = gt.tree_id
    and   ar.rel_id = octr.rel_id
    and rel_type = 'object_category_tree_rel'
}]

if {!$language_category_tree_assigned_p} {
    ad_return_complaint 1 "No category tree assigned for languages. Please contact a library administrator."
    return
}

set proficiency_category_tree_assigned_p [db_0or1row proficiency_category_tree {
    select octr.subtree_root_node_id as proficiency_subtree_root_id
    from generic_trees gt,
    acs_rels ar,
    object_category_tree_rels octr
    where ar.object_id_one = :proficiency_magic_id
    and   ar.object_id_two = gt.tree_id
    and   ar.rel_id = octr.rel_id
    and rel_type = 'object_category_tree_rel'
}]

if {!$proficiency_category_tree_assigned_p} {
    ad_return_complaint 1 "No category tree assigned for language proficiencies. Please contact a library administrator."
    return
}

set role_list [db_list_of_lists get_role_categories {
    select  /*+INDEX(sw_category_dim sw_category_dim_long_name) */ category.long_name(object_id)   as name,
    object_id as category_id
    from  sw_category_dim
    start with node_id = :role_subtree_root_id
    connect by prior node_id = parent_node_id
}]

set roles [list]
foreach single_role $role_list {
    if {[lsearch $selected_role_ids [snd $single_role]] > -1} {
	if {[string equal [fst $single_role] "Other"]} {
	    lappend roles "[fst $single_role] ($role_other)"
	} else {
	    lappend roles [fst $single_role]
	}
    }
}
set roles [join $roles ", "]


set language_list [db_list_of_lists get_language_categories {
    select  /*+INDEX(sw_category_dim sw_category_dim_long_name) */ category.long_name(object_id)   as name,
    object_id as category_id
    from  sw_category_dim
    start with node_id = :language_subtree_root_id
    connect by prior node_id = parent_node_id
}]

set language_id_list [map snd $language_list]
set second_language [fst [lindex $language_list [lsearch $language_id_list $second_language_id]]]
set third_language [fst [lindex $language_list [lsearch $language_id_list $third_language_id]]]


set language_prof_list [db_list_of_lists get_language_prof_categories {
    select  /*+INDEX(sw_category_dim sw_category_dim_long_name) */ category.long_name(object_id)   as name,
    object_id as category_id
    from  sw_category_dim
    start with node_id = :proficiency_subtree_root_id
    connect by prior node_id = parent_node_id
}]

set language_prof_id_list [map snd $language_prof_list]
set second_language_prof [fst [lindex $language_prof_list [lsearch $language_prof_id_list $second_language_prof_id]]]
set third_language_prof [fst [lindex $language_prof_list [lsearch $language_prof_id_list $third_language_prof_id]]]

set replacement_list [list \
	user_name               $user_name \
	user_email              $user_email \
	library_url             [ad_url]$library_url \
	project_id              $resource_req_id \
	project_name            $project_name \
	project_type            $project_type \
	demand_id               $object_id \
	demand_name             $demand_name \
	demand_type             $demand_type \
	recipient               $recipient \
	subject                 $subject \
	contact_data            $contact_data \
	nationality             $nationality \
	manager_email           $manager_email \
	roles                   $roles \
	working_area            $working_area \
	first_language          $first_language \
	second_language         $second_language \
	third_language          $third_language \
	second_language_prof    $second_language_prof \
	third_language_prof     $third_language_prof \
	other_language          $other_language \
	from_date               [util_AnsiDatetoPrettyDate $from_date] \
	to_date                 [util_AnsiDatetoPrettyDate $to_date] \
	strengths               $strengths \
	leadership              $leadership \
	intercultural           $intercultural \
	comments                $comments \
	conditions              $conditions ]

set community_id [db_string get_community_id {
    select community_id
    from sn_community_instances_all
    where package_id = :package_id
}]


set email_set [et_process -community_id $community_id "psn_application" $replacement_list]

## generate my email in a file - to prevent having all huge attachments in a tcl variable
## first generate header
set email_filename [ns_tmpnam]
set fd [open $email_filename w]
fconfigure $fd -encoding iso8859-1
puts $fd "This is a multi-part message in MIME format.

--$boundary_string
Content-Type: text/plain; charset=\"iso-8859-1\"
Content-Transfer-Encoding: quoted-printable

"
close $fd

## cut out all carriage returns out of the email form
regsub -all {\r} [ns_set get $email_set "body"] {} body

## now append encoded cleartext-email (->input form)
set tmp_filename [ad_mime_encode -string $body "quoted-printable"]
exec /bin/sh -c "cat $tmp_filename >> $email_filename"
ns_unlink $tmp_filename

set attachment_list [db_list_of_lists get_attachment_id_list {
    select attachment_id, mime_type, filename
    from psn_attachments
    where application_id = :application_id
}]

## now append all attachments
foreach attachment $attachment_list {
    util_unlist $attachment attachment_id mime_type filename
    set tmp_filename [ns_tmpnam]
    db_blob_get_file get_actual_attachment "
	select attachment from psn_attachments where attachment_id = $attachment_id
    " -file $tmp_filename

    ## append attachment separator
    set fd [open $email_filename a]
    fconfigure $fd -encoding iso8859-1
    puts $fd "
--$boundary_string
Content-Type: $mime_type; name=\"[ad_quotehtml $filename]\"
Content-Transfer-Encoding: base64
Content-Disposition: attachment; filename=\"[ad_quotehtml $filename]\"

"
    close $fd

    ## append encoded attachment
    set tmp_filename_enc [ad_mime_encode -file $tmp_filename base64]
    exec /bin/sh -c "cat $tmp_filename_enc >> $email_filename"
    ns_unlink $tmp_filename_enc
}

## append closing string at the end
set fd [open $email_filename a]
fconfigure $fd -encoding iso8859-1
puts $fd "

--$boundary_string\--
"
close $fd

## now get the complete email in just one chunk in a tcl variable (yeah....)
set fd [open $email_filename r]
fconfigure $fd -encoding iso8859-1
ns_set update $email_set "body" [read $fd]
close $fd
ns_unlink $email_filename

db_transaction {
    et_queue_email -content_type "multipart/mixed; boundary=\"$boundary_string\"" $email_set
    ns_set update $email_set "to" $user_email
    et_queue_email -content_type "multipart/mixed; boundary=\"$boundary_string\"" $email_set
    db_dml set_object_state {
	update acs_objects_description
	set state = 'a'
	where object_id = :application_id
    }

    db_dml set_sent_state {
	update psn_res_applications
	set sent_p = 't',
	application_date = sysdate
	where application_id = :application_id
    }
}

ad_returnredirect $return_url
