ad_library {

    Photo album helper procedures.
    @author Andrew Grumet <aegrumet@arsdigita.com>
    @creation-date 12 Dec 2000
    @cvs-id $Id: photo-album-lite-init.tcl,v 1.3 2009/02/13 20:40:39 jeffd Exp $

}

ad_proc pl_photo_cleanup_filename {
    filename
} {
    <pre>
    # Regsub out spaces and other characters from the client filename.
    </pre>
} {
    set root [file rootname $filename]
    regsub -all {[^0-9a-zA-Z_\-]} $root {} result
    return "$result[file extension $filename]"
}

ad_proc pl_photo_commentable_p {
    photo_id
} {
    <pre>
    # Returns 1 if both general comments is installed
    # and GeneralCommentsEnabledP is 1, otherwise returns 0.
    # HINT: This would be a good place to add fancy permission checks.
    </pre>
} {

    if ![ad_parameter GeneralCommentsEnabledP photo-album-lite pl_photos] {
	# general-comments is disabled for this package
	return 0
    }

    if [empty_string_p [namespace eval :: {info procs general_comments_get_comments}]] {
	# general-comments is not installed
	return 0
    }

    # If we made it here, comments are okay.
    return 1
}

ad_proc pl_base_directory {
    {-read_only:boolean}
    photo_id
} {
    <pre>
    # returns a destination directory for this photo.
    # creates the directory if necessary
    </pre>
} {

    regexp {(.*)/[^/]+} [ns_info pageroot] match server_base
    set pl_photo_base $server_base/[ad_parameter PhotoBaseDirectory photo-album-lite pl_photos]
    set first_subdir $pl_photo_base/[expr $photo_id % 10]
    set second_subdir $first_subdir/[expr [expr $photo_id / 10] % 10]

    if { !$read_only_p } {
	foreach dir [list $pl_photo_base $first_subdir $second_subdir] {
	    if ![file exists $dir] {
		ns_mkdir $dir
	    }
	}
    }
    return $second_subdir
}

ad_proc pl_delete_photo photo_id {
    <pre>
    # Deletes a photo.  First, flag the table row as deleted,
    # then nuke the associated files.
    </pre>
} {

    db_transaction {

	db_dml pl_photo_delete {
	    update pl_photos
	    set deleted_p = 't'
	    where photo_id in ( select photo_id
                                from pl_v_photos
                                where photo_id = :photo_id )
	}

	set n_deleted [db_resultrows]

	# note this will be somewhat inefficient for deleting an
        # entire folder.
	db_exec_plsql pl_reshuffle {begin pl_reshuffle; end;}
    }

    if $n_deleted {
	set file_extension [db_string pl_photo_file_ext { select file_extension from pl_photos where photo_id = :photo_id }]
	set base_directory [pl_base_directory -read_only $photo_id]
	foreach size [list thumb med orig] {
	    ns_unlink -nocomplain $base_directory/$photo_id-$size$file_extension
	}
    }
}

ad_proc pl_delete_folder folder_id {
    <pre>
    # Deletes a folder and all of its contents.
    </pre>
} {
    set user_id [ad_conn user_id]

    db_dml pl_folder_delete {
	update pl_folders
	set deleted_p = 't'
	where folder_id in ( select folder_id
                             from pl_v_folders
                             where folder_id = :folder_id )
    }
    if [db_resultrows] {
	db_foreach pl_folder_photo_delete {
	    select photo_id from pl_photos where folder_id = :folder_id
	} {
	    pl_delete_photo $photo_id
	}
    }
}

ad_proc pl_photo_insert {
    {-use_default_date 0}
    {-use_default_prefix 0}
    folder_id
    server_tempnam
    {full_client_filename ""}
} {
    <pre>
    # insert a row for the pic; copy and scale the tempfile
    </pre>
} {

    set caller_id [ad_conn user_id]
    set caller_ip [ns_conn peeraddr]
    set im_home [ad_parameter ImageMagickBinDir photo-album-lite /usr/X11R6/bin]

    if [empty_string_p $im_home] {
	ns_log Notice "photo-album-lite: warning: ad_parameter has set the ImageMagickBinDir to the empty string!"
    }

    if [empty_string_p $full_client_filename] {
	set full_client_filename $server_tempnam
    }

    set file_extension [string tolower [file extension $full_client_filename]]

    # strip off the C:\directories... crud and just get the file name
    if ![regexp {([^/\\]+)$} $full_client_filename match client_filename] {
	# couldn't find a match
	set client_filename $full_client_filename
    }

    set client_filename [pl_photo_cleanup_filename $client_filename]
    db_transaction {
	set photo_id [db_exec_plsql pl_photo_add "
begin
  :1 := pl_photo.new (
    folder_id          => :folder_id,
    client_filename    => :client_filename,
    file_extension     => :file_extension,
    object_type        => 'pl_photo',
    creation_date      => sysdate,
    creation_user      => :caller_id,
    creation_ip        => :caller_ip,
    context_id         => :folder_id );
end;
"]
        # note this will be somewhat inefficient when bulk uploading
        # but we need it to guarantee sort_n integrity.
	db_exec_plsql pl_reshuffle {begin pl_reshuffle; end;}
    }

    set base_directory [pl_base_directory $photo_id]
    set orig_name $base_directory/$photo_id-orig$file_extension
    set med_name $base_directory/${photo_id}-med$file_extension
    set thumb_name $base_directory/${photo_id}-thumb$file_extension

    ns_cp $server_tempnam $orig_name

    # get info for original image
    if [catch {set identify_output [exec $im_home/identify -ping -format %h,%w,%b $orig_name]} errMsg] {
	pl_delete_photo $photo_id
	return -code error "photo-album-lite: identify failed for photo $photo_id with error message: $errMsg"
    }
    foreach {orig_height orig_width orig_kbytes_kb} [split $identify_output ,] {}
    regsub {[^0-9]+$} $orig_kbytes_kb {} orig_kbytes

    # scale down and get info
    if [catch {set convert_output [exec $im_home/convert -verbose -geometry ${orig_width}x[ad_parameter MediumHeight photo-album-lite 400] -sharpen 3 -quality 90 $orig_name $med_name 2>/dev/null]} errMsg] {
	pl_delete_photo $photo_id
	return -code error "photo-album-lite: convert failed for photo $photo_id with error message: $errMsg"
    }

    if [catch {set identify_output [exec $im_home/identify -ping -format %h,%w,%b $med_name]} errMsg] {
	pl_delete_photo $photo_id
	return -code error "photo-album-lite: identify failed for photo $photo_id with error message: $errMsg"
    }
    foreach {med_height med_width med_kbytes_kb} [split $identify_output ,] {}
    regsub {[^0-9]+$} $med_kbytes_kb {} med_kbytes

    if [catch {set convert_output [exec $im_home/convert -verbose -geometry ${orig_width}x[ad_parameter ThumbHeight photo-album-lite 100] -sharpen 3 -quality 90 $orig_name $thumb_name 2>/dev/null]} errMsg] {
	pl_delete_photo $photo_id
	return -code error "photo-album-lite: convert failed for photo $photo_id with error message: $errMsg"
    }

    if [catch {set identify_output [exec $im_home/identify -ping -format %h,%w,%b $thumb_name]} errMsg] {
	pl_delete_photo $photo_id
	return -code error "photo-album-lite: identify failed for photo $photo_id with error message: $errMsg"
    }
    foreach {thumb_height thumb_width thumb_kbytes_kb} [split $identify_output ,] {}
    regsub {[^0-9]+$} $thumb_kbytes_kb {} thumb_kbytes

    db_dml pl_photos_update {
	update pl_photos
	set orig_width = :orig_width,
            orig_height = :orig_height,
            orig_kbytes = :orig_kbytes,
            med_width = :med_width,
            med_height = :med_height,
            med_kbytes = :med_kbytes,
            thumb_width = :thumb_width,
            thumb_height = :thumb_height,
            thumb_kbytes = :thumb_kbytes
        where photo_id = :photo_id
    }

    if $use_default_date {
	db_dml pl_photo_default_date_dml {
	    update pl_photos
               set exposure_date = (select default_exp_date
                                    from pl_folders
                                    where folder_id = :folder_id)
               where photo_id = :photo_id
	}
    }

    if $use_default_prefix {
	db_exec_plsql pl_photo_default_name_dml {
	    begin pl_photo.apply_default_name(:photo_id); end; 
        }
    }
}
