ad_page_contract {

    @author Andrew Grumet <aegrumet@arsdigita.com>
    @cvs-id $Id: folder-list.tcl,v 1.2 2002/09/10 22:23:14 jeffd Exp $

    Show a list of folders, in various contexts:

    1. me, looking at a list of folders to add photos to,
       with an option to create a new one.
    2. anyone, looking at a list of one photographer's folders
    3. anyone, looking at a list of all folders by anyone

} {
    pl_user_id:optional,naturalnum
    all_p:optional,boolean
    {mode:optional all}
}

set user_id [ad_conn user_id]
set package_id [ad_conn package_id]
set return_url [ad_urlencode folder-list?mode=mine]

set sql "
select folder_id,
       title,
       person.name(creation_user) as creator,
       pl_photo_date_format(creation_date) as pretty_creation_date,
       pl_folder.photo_count(folder_id) as photo_count
from pl_v_folders
where context_id = :package_id "

switch $mode {
    mine {
        if !$user_id {
	    ad_redirect_for_registration
	    ad_script_abort
	}
	append sql "
and creation_user = :user_id"
        set folders_by_id $user_id

        # Also query for folders this user can write to.
        db_multirow writable writable_q {
	    select folder_id,
                   title,
                   creation_user,
                   person.name(creation_user) as creator,
                   pl_photo_date_format(creation_date) as pretty_creation_date,
                   pl_folder.photo_count(folder_id) as photo_count,
                   decode(acs_permission.permission_p(folder_id,:user_id,'admin'),'t',1,0) as admin_p
           from pl_v_folders
           where context_id = :package_id
             and acs_permission.permission_p(folder_id,:user_id,'write') = 't'
             and creation_user not in (:user_id)
           order by creation_date desc
	}
    }
    user {
	if ![info exists pl_user_id] {
	    ad_return_error "Application error." "mode=user requires pl_user_id"
	    ad_script_abort
	}
	append sql "
and creation_user = :pl_user_id"
        set folders_by_id $pl_user_id
    }
}

db_multirow folder pl_folder_list "$sql order by creation_date desc"

# just in case there are no rows
if [info exists folders_by_id] {
    set person_name [db_string pl_folders_by_name {select person.name(:folders_by_id) from dual}]}

# determine whether to show extra writable rows.
if { [string equal $mode mine] && ![info exists all_p] } {
    if { [template::multirow size writable] > 10 } {
	set all_p 0
    } else {
	set all_p 1
    }
}

set context_bar [ad_context_bar Folders]
