ad_page_contract {

    @author Andrew Grumet <aegrumet@arsdigita.com>
    @cvs-id $Id: folder-view.tcl,v 1.2 2002/09/10 22:23:14 jeffd Exp $

    Display contents of one folder.

} {
    folder_id:notnull,naturalnum
}

set user_id [ad_conn user_id]
set package_id [ad_conn package_id]
set return_url [ad_urlencode [ns_conn url]?[ns_conn query]]
set default_height [ad_parameter ThumbHeight photo-album-lite 100]
set preload_p [ad_parameter ImagePreloadingEnabledP photo-album-lite 1]

set write_p [ad_permission_p $folder_id write]
set admin_p [ad_permission_p $folder_id admin]

db_1row folder_info {
    select f.title as folder_title,
           f.descr,
           to_char(f.creation_date,'DD Mon YYYY') as post_date,
           person.name(f.creation_user) as photographer,
           f.creation_user,
           pl_folder.photo_count(:folder_id) as photo_count,
           pl_photo_date_format(default_exp_date) as default_exp_date
    from pl_v_folders f
    where f.folder_id = :folder_id
      and f.context_id = :package_id
}

set context_bar [ad_context_bar Folder]

set line_break [ad_parameter RightMargin photo-album-lite 450]
set running_width 0

set photo_table "<table border=0 cellpadding=5 cellspacing=0><tr>"

set first_p 1
db_foreach folder_photos {
    select client_filename,
           photo_id,
           decode(thumb_width,NULL,'','width="' || thumb_width || '"') as width_tag,
           'height="' || nvl(thumb_height,:default_height) || '"' as height_tag,
           nvl(thumb_width,round(:default_height * 1.33)) as thumb_width
    from pl_v_photos
    where folder_id = :folder_id
    order by sort_n
} {
    if $first_p { set first_photo_id $photo_id; set first_client_filename $client_filename; set first_p 0 }

    if { $running_width > $line_break } {
	append photo_table "</tr></table><table border=0 cellpadding=5 cellspacing=0><tr>"
	set running_width 0
    }

    append photo_table "<td><a href=\"photo-view?photo_id=$photo_id\"><img src=\"photo/thumb/$photo_id/$client_filename\" $width_tag $height_tag border=\"0\" alt=\"$client_filename\"></a></td>"

    incr running_width $thumb_width

} if_no_rows { unset photo_table }

 
if [info exists photo_table] {
    append photo_table "</tr></table>"
} else {
    set photo_table "<em>Empty folder.</em>"
    set creation_user [db_string pl_folder_creation_user "select creation_user from pl_v_folders where folder_id = :folder_id"]
    if { $creation_user == $user_id } {
	append photo_table "You may <a href=photo-add?folder_id=$folder_id>add photos</a> to this folder if you'd like."
    }
}
