ad_page_contract {
    
    @author Andrew Grumet <aegrumet@arsdigita.com>
    @cvs-id $Id: index.tcl,v 1.2 2002/09/10 22:23:14 jeffd Exp $

    Show recent photos and summary info.
    
} {}

# Give a friendly warning for fresh installs, or set a parameter
# for use below.
if [catch {set default_height [ad_parameter ThumbHeight photo-album-lite 100]}] {
    ad_return_error "Error calling ad_parameter." "There was an error <code>calling ad_parameter</code>.  If this is a fresh install, please restart your server."
}

set context_bar [ad_context_bar]

set package_id [ad_conn package_id]

set instance_name [db_string pl_instance_name {
    select instance_name from apm_packages where package_id = :package_id
}]

set photo_count [db_string pl_photo_count {
    select count(p.photo_id)
    from pl_v_photos p,
         pl_v_folders f
    where p.folder_id = f.folder_id
      and f.context_id = :package_id
}]

set last_upload [db_string pl_last_upload {
    select pl_relative_date(max(p.creation_date))
    from pl_v_photos p,
         pl_v_folders f
    where p.folder_id = f.folder_id
      and f.context_id = :package_id
}]

set line_break [ad_parameter RightMargin photo-album-lite 450]
set line_count 1
set running_width 0

set photo_table "<table border=0 cellpadding=5 cellspacing=0><tr>"

# assumption: 20 photos is enough to fill the page.
# don't think you can do this with pure templating logic.
# we do some goofy nvl'ing and arithmetic to handle
# missing heights and widths
db_foreach new_photos {
    select * from (
      select p.client_filename,
             p.photo_id,
             decode(p.thumb_width,NULL,'','width="' || p.thumb_width || '"') as width_tag,
            'height="' || nvl(p.thumb_height,:default_height) || '"' as height_tag,
             nvl(p.thumb_width,round(:default_height * 1.33)) as thumb_width
      from pl_v_photos p,
           pl_v_folders f
      where p.folder_id = f.folder_id
        and f.context_id = :package_id
      order by p.creation_date desc )
    where rownum < 20
} {
    if { $running_width > $line_break } {
	incr line_count
	set running_width 0
	if { $line_count < 4 } {
	    append photo_table "</tr></table><table border=0 cellpadding=5 cellspacing=0><tr>"
	}
    }
    if { $line_count < 4} {

	append photo_table "<td><a href=\"photo-view?photo_id=$photo_id\"><img src=\"photo/thumb/$photo_id/$client_filename\" $width_tag $height_tag border=\"0\" alt=\"$client_filename\"></a></td>"

	incr running_width $thumb_width
    }
}
 
append photo_table "</tr></table>"

set new_folder_table {<table width=100%>
 <tr><th bgcolor=e0e0e0>New Folders</th></tr>}

db_multirow new_folders pl_new_folders {
  select * from (
    select title, folder_id
    from pl_v_folders
    where sysdate < creation_date + 7
      and context_id = :package_id
    order by creation_date desc
  ) where rownum < 5
}

set user_id [ad_conn user_id]

db_multirow your_folders pl_your_folders {
   select * from (
    select title, folder_id
    from pl_v_folders
    where creation_user = :user_id
      and context_id = :package_id
    order by creation_date desc
  ) where rownum < 5
}
