ad_page_contract {} {
    comment_id:integer,notnull
    ticket_id:integer,optional
    message:allhtml,trim,notnull
    message_format:notnull
}

ttl_restrict_to_ttl_users

set caller_id [ad_conn user_id]

db_dml update_comment {
    update ttl_comments set
        message = :message,
        message_format = :message_format
    where comment_id = :comment_id
}

if {[db_resultrows] == 0} {

    db_dml insert_comment {
        insert into ttl_comments (
            comment_id, ticket_id, creation_user,
            message, message_format
        ) values (
            :comment_id, :ticket_id, :caller_id,
            :message, :message_format
        )
    }
    
    db_foreach get_email {
        select p.email
        from parties p, ttl_assignments ta
        where ta.ticket_id = :ticket_id
        and ta.user_id = p.party_id
        UNION
        select p.email
        from ttl_v_tickets t, parties p
        where t.ticket_id = :ticket_id
        and t.creation_user = p.party_id
    } {
        lappend email_recipiants $email
    }

    if [info exists email_recipiants] {

        set formatted_message [ttl_format_text_for_display -format $message_format -wrap 60 $message]

        db_1row get_recipient_info {
            select
                name as ticket_name,
                person.name(:caller_id) as sender_fullname,
                (select email from parties
                 where party_id = :caller_id) as sender_email
            from ttl_v_tickets
            where ticket_id = :ticket_id
        }

        ns_sendmail "[join $email_recipiants ,]" "$sender_email" "Re: $ticket_name" \
"$sender_fullname wrote:

$message

[util_current_location][ad_conn package_url]ticket?ticket_id=$ticket_id"

    }
} else {
    # We need to get the ticket_id so we know where to redirect
    # the user

    db_1row get_ticket {
        select ticket_id from ttl_comments where comment_id = :comment_id
    }
} 

ad_returnredirect ticket?ticket_id=$ticket_id

 