ad_page_contract {
    lets set the default page when not available
} {
    {page:naturalnum 1}
}

set caller_privilege [ttl_restrict_to_ttl_users]

if [string equal admin $caller_privilege] {
    set admin_url "[ad_conn package_url]admin"
} else {
    set admin_url ""
}

set package_id [ad_conn package_id]
set caller_id  [ad_conn user_id]

set master_template [ttl_master_template]

#set context [ttl_context -nohome]
set context [list]

db_0or1row get_prefs {
    select 
        sort,
        mine_p,
        status,
        area_id,
        (select name from ttl_va_areas a
         where a.area_id = p.area_id) as area_name
    from ttl_prefs p
    where user_id = :caller_id
}

#
# 1) Whose tickets we're viewing
#

set mine_p [ttl_util_default mine_p f]

if [ttl_util_boolean mine_p] {
    set user_id_clause "
and exists (select 1 from ttl_assignments ta
            where ta.user_id = :caller_id
            and ta.ticket_id = t.ticket_id)"

} else {
    set user_id_clause ""
}

#
# 2) Which states we're viewing
#

set status [ttl_util_default status all]

if [string equal $status active] {
    set state_list {queued dev staging revision}
} elseif [string equal $status all] {
    set state_list {}
} else {
    set state_list $status
}

if ![empty_string_p $state_list] {
    set status_clause "\nand s.status in ('[join $state_list ',']')"
} else {
    set status_clause ""
}

#
# 3) How we're going to sort the results
#

set sort [ttl_util_default sort priority]

switch $sort {
    ticket_id {
	set order_clause "order by ticket_id"
    }
    deadline {
        set order_clause "[db_map order_deadline]"
    } 
    activity {
        set order_clause "[db_map order_activity]"
    }
    name {
        set order_clause "order by upper(t.name)"
    }
    area {
        set order_clause "[db_map order_area]"
    } 
    status {
        set order_clause "[db_map order_status]"
    } 
    default {
        set order_clause "[db_map order_default]"
    }
}

#
# 4) Which area we're viewing
#

set area_id [ttl_util_default area_id all]

if [string equal all $area_id] {
    set area_clause ""
} else {
    set area_clause "\nand t.area_id = :area_id"
}


# 5) Figure the page title

if [ttl_util_boolean mine_p] {
    set title "Your tickets"
} else {
    set title "Tickets"
}

if [exists_and_not_null area_name] {
    append title " in \"$area_name\""
}

#
# For dimensional sliders
#
set id_slider       [ttl_pref_choices sort ticket_id "Ticket Id"]
set name_slider     [ttl_pref_choices sort name Subject]
set status_slider   [ttl_pref_choices sort status Status]
set activity_slider [ttl_pref_choices sort activity "Last Activity"]
set area_slider     [ttl_pref_choices sort area "Area"]
set priority_slider [ttl_pref_choices sort priority Priority]
set deadline_slider [ttl_pref_choices sort deadline Deadline]

set option_list [list "f {All [string repeat &nbsp 25]}" {t Me}]
set ticket_owner_slider [ad_db_select_widget -default $mine_p -option_list $option_list mine_list "" mine_p]
set option_list [list "all {All [string repeat &nbsp 25]}" {queued Queued} {active Active}]
set ticket_state_slider [ad_db_select_widget -default $status -option_list $option_list status_list "" status]


set area_optionlist [ttl_area_optionlist -all_option $area_id]


# build the paginated tickets
set p_name tickets_paginator

paginator create get_tickets $p_name "" -pagesize 20 -timeout 1
paginator get_data get_paged_tickets $p_name tickets "" ticket_id $page

set group [paginator get_group $p_name $page]
paginator get_pages_info $p_name pages [paginator get_pages $p_name $group]
