ad_page_contract {} {
    ticket_id:integer,notnull
    area_id:integer,notnull
    type:trim,optional
    name:trim,notnull
    message:trim,optional,allhtml
    message_format:trim,notnull
    priority:trim,notnull
    deadline:date,array
    {status queued}
}

ttl_restrict_to_ttl_users

set caller_id [ad_conn user_id]
set caller_ip [ad_conn peeraddr]
set package_id [ad_conn package_id]

set deadline_date $deadline(date)

db_transaction {

    db_dml add_ticket {
        declare
            v_object_id     integer;
        begin
            v_object_id := acs_object.new (
                object_id     => :ticket_id,
                object_type   => 'ticket_tracker_lite_ticket',
                creation_user => :caller_id,
                creation_ip   => :caller_ip,
                context_id    => :package_id
            );

            insert into ttl_tickets (
                ticket_id, area_id, name, type, status, deadline, 
                message, message_format, priority
            ) values (
                :ticket_id, :area_id, :name, :type, :status, :deadline_date, 
                :message, :message_format, :priority
            );
        end;
    }

    db_dml insert_assignment {
        insert into ttl_assignments (user_id, ticket_id)
        select user_id, :ticket_id
        from ttl_area_assignments
        where area_id = :area_id
    }
}

# now that the transaction is over, let's send out the email

db_foreach get_recipient_email {
    select p.email 
    from parties p, ttl_assignments a
    where a.ticket_id = :ticket_id
    and a.user_id = p.party_id
} {
    lappend people_to_email $email
}

if [exists_and_not_null people_to_email] {
    db_1row get_sender_email {
        select 
            p1.first_names || ' ' || p1.last_name as ticket_creator_name,
            p2.email as ticket_creator_email,
            (select name from ttl_va_areas where area_id = :area_id) as area_name
        from persons p1, parties p2
        where p1.person_id = p2.party_id
        and p1.person_id = :caller_id
    }

    set recipients "[join $people_to_email ,]"

    set notif_message "Ticket added by $ticket_creator_name in '$area_name'

       $message

       [util_current_location][ad_conn package_url]ticket?ticket_id=$ticket_id"


    if { [catch {ns_sendmail \
                     $recipients \
                     $ticket_creator_email \
                     $name \
                     $notif_message } errmsg ] } {
        ad_return_error \
            "Email failed" \
            "ticket-tracker-lite - ticket-add-2.tcl: \n
             The email could not be sent. Error was : $errmsg"
    }
}

ad_returnredirect ticket?ticket_id=$ticket_id
