ad_page_contract {} {
    ticket_id:integer,optional
}

ttl_restrict_to_ttl_users

set caller_id  [ad_conn user_id]
set package_id [ad_conn package_id]

set master_template [ttl_master_template]

set edit_url "ticket-edit?ticket_id=$ticket_id"

#
# 1) Get the ticket info
#

if ![db_0or1row get_one_ticket {
    select 
        t.area_id,
        a.name as area_name,
        t.name,
        t.type,
        t.status,
        t.message,
        t.message_format,
        t.priority,
        t.deadline,
        t.url,
        ttl_relative_date(t.creation_date) as posting_date,
        person.name(t.creation_user) as posting_name
    from ttl_v_tickets t, ttl_va_areas a
    where a.context_id = :package_id 
    and t.ticket_id = :ticket_id
    and t.area_id = a.area_id
}] then {
    ad_returnredirect "ticket-none" 
    ad_script_abort
}

set title "Ticket $ticket_id"
set context [list [list "prefs-ae?area_id=$area_id" "$area_name"] $title]

#set context [list [list [list "prefs-ae?area_id=$area_id" "$area_name"] $title]]

set message [ttl_format_text_for_display -format $message_format -wrap 60 $message]

#
# 2) Get people assigned to this ticket
#

db_foreach get_people {
  select * from (
    select 
        p.first_names || ' ' || p.last_name as full_name,
        t.user_id,
        decode(a.user_id, null, 0, 1) as already_assigned_p
    from persons p, ttl_v_users t, 
            (select user_id
             from ttl_assignments
             where ticket_id = :ticket_id) a
    where t.package_id = :package_id 
    and p.person_id = t.user_id
    and t.user_id = a.user_id(+)
    order by upper(full_name)
  ) where rownum < 50
} {

    if $already_assigned_p {
        lappend assigned_users "$full_name <a class=tiny href=assignment-ae?user_id=$user_id&ticket_id=$ticket_id&action=remove>remove</a>"
    } else {
        append unassigned_options "<option value=$user_id>$full_name\n"
    }

    if [info exists unassigned_options] {
        set unassigned_optionlist "
        <form action=assignment-ae>
        <input type=hidden name=ticket_id value=$ticket_id>
        <select name=user_id>\n$unassigned_options</select>
        <input type=submit value=Assign>
        </form>"
    } else {
        set unassigned_optionlist ""
    }
}

#
# 4) Create the widgets
#

set type_optionlist     [ttl_type_optionlist $type]
set area_optionlist     [ttl_area_optionlist $area_id]
set priority_optionlist [ttl_priority_optionlist $priority]
set status_optionlist   [ttl_status_optionlist $status]
set deadline_widget     [ad_dateentrywidget deadline $deadline]

db_multirow files get_files "select i.name, r.mime_type, r.revision_id, i.item_id from cr_items i, cr_revisions r where i.parent_id = :ticket_id and i.item_id = r.item_id"

set return_url "ticket?ticket_id=$ticket_id"
