# /packages/ticket-tracker/tcl/ttracker-procs.tcl

ad_library {

    Procs for ticket tracker.

    @author Phong Nguyen (phong@arsdigita.com)
    @author Tony Tseng (tony@arsdigita.com)
    @creation-date 2000-11-27
    @cvs-id $Id: ttracker-procs.tcl,v 1.3 2001/10/31 20:42:07 donb Exp $
}

ad_page_contract_filter ttracker_ticket_id { name value } {
    Checks whether the value (assumed to be an integer) is the id of an ticket 
    that belongs to the current ttracker instance.
} {
    set package_id [ad_conn package_id]
    db_1row check_ticket_id {
        select decode(ttracker_ticket.id_valid_p(:value, :package_id), 't', 1, 0) as id_valid_p from dual
    }

    if { !$id_valid_p } {
        ad_complain "$value is not a valid ticket id for this ticket tracker instance"
        return 0
    }
    
    return 1
}

ad_page_contract_filter ttracker_option_id { name value } {
    Checks whether the value (assumed to be an integer) is the id of an option
    that belongs to the current ttracker instance.
} {
    set package_id [ad_conn package_id]
    db_1row check_option_id {
	select decode(count(option_id), 0, 0, 1) as id_valid_p
          from ttracker_options
         where option_id = :value
           and package_id = :package_id
    }

    if { !$id_valid_p } {
        ad_complain "$value is not a valid option id for this ticket tracker instance"
        return 0
    }

    return 1 
    
}

    
ad_page_contract_filter ttracker_task_id { name value } {
    Checks whether the value (assumed to be an integer) is the id of a task 
    that relates to the current ttracker instance.
} {
    set package_id [ad_conn package_id]
    db_1row check_task_id {
        select decode(count(wt.task_id), 0, 0, 1) as id_valid_p 
          from wf_tasks wt
         where exists (select wc.case_id
                         from wf_cases wc,
                              ttracker_tickets tt,
                              ttracker_categories tc 
                        where wt.case_id = wc.case_id
                          and wc.object_id = tt.ticket_id
                          and tt.category_id = tc.category_id
                          and tc.package_id = :package_id) 
           and wt.task_id = :value
    }

    if { !$id_valid_p } {
        ad_complain "$value is not a valid task id for this ticket tracker instance"
        return 0
    }
    
    return 1
}

ad_proc -public ttracker_workflow_url {
    -absolute:boolean
} {
    If absolute flag is set, return the absolute url to acs-workflow.
    Otherwise return the relative url.
} {
    if { $absolute_p } {
	set absolute_p "t"
    } else {
	set absolute_p "f"
    }

    return [db_string get_url { select ttracker_util.workflow_url(:absolute_p) from dual }]
}

ad_proc -public ttracker_category_exists_p {
    -package_id 
} {
    Check if the ttracker instance has at least one category
} {
    if { ![info exists package_id] } { 
        set package_id [ad_conn package_id]
    }

    return [db_string check_category {
        select decode(count(1), 0, 0, 1)
        from ttracker_categories
        where package_id = :package_id
    }]
}

ad_proc -public ttracker_install {
    -package_id
} {
    Installs the default category and severity/priority options for this ttracker instance.
} {
    if { ![info exists package_id] } {
        set package_id [ad_conn package_id] 
    }

    db_exec_plsql install {
        begin
            ttracker_util.install(:package_id);
        end;
    }
}




