# /packages/ticket-tracker/www/admin/option-delete.tcl

ad_page_contract {
    Delete a name-value option pair.

    @author Phong Nguyen (phong@arsdigita.com)
    @author Tony Tseng (tony@arsdigita.com)
    @creation-date 2001-01-08
    @cvs-id $Id: option-delete.tcl,v 1.1 2001/04/20 20:51:25 donb Exp $
} {
    option_id:integer,ttracker_option_id
} -validate {
    is_in_use -requires {option_id:ttracker_option_id} {
	# check if this option is not being used by any of the tickets
	db_1row ticket_count {
	    select count(*) as in_use 
	      from dual
	     where exists (select tt.ticket_id
	                     from ttracker_options ts,
	                          ttracker_tickets tt,
	                          ttracker_categories tc
	                    where ts.option_id = :option_id
	                      and tt.category_id = tc.category_id
	                      and tc.package_id = ts.package_id
	                      and ((ts.function = 'severity' and ts.value = tt.severity) or
	                           (ts.function = 'priority' and ts.value = tt.priority)))
	}

	
	if { $in_use } {
	    ad_complain
	}
    }
} -errors {
    is_in_use { This option is being used right now. \
	    You must make sure none of the tickets is using this option before deleting it }
}

db_dml delete_option {
    delete from ttracker_options 
     where option_id = :option_id
       and not exists (select tt.ticket_id
	                 from ttracker_options ts,
	                      ttracker_tickets tt,
	                      ttracker_categories tc
	                where ts.option_id = :option_id
	                  and tt.category_id = tc.category_id
	                  and tc.package_id = ts.package_id
	                  and ((ts.function = 'severity' and ts.value = tt.severity) or
	                       (ts.function = 'priority' and ts.value = tt.priority)))
}

ad_returnredirect ""

