# /packages/ticket-tracker/www/category-summary.tcl

ad_page_contract {
    Displays a summary of tickets by category

    @author Phong Nguyen (phong@arsdigita.com)
    @author Tony Tseng (tony@arsdigita.com)
    @creation-date 2000-11-20
    @cvs-id $Id: category-summary.tcl,v 1.3 2002/09/06 06:57:03 jeffd Exp $
} {
    {orderby {category}}
    {order {}}
} -properties {
    context:onevalue
    summary_table:onevalue
} 

set category_name [ad_parameter CategoryName "ticket-tracker" "Category"]
set page_title "$category_name Summaries"
set context [list "$category_name Summaries"]    
set package_id [ad_conn package_id]

# dimensional slider definition
set dimensional {
    {posted "In the last" any {
        {year "year" { where "[db_map last_1_year]" }}
        {3month "3 months" { where "[db_map last_3_months]" }}
        {month "month" { where "[db_map last_1_month]" }}
        {any "all" {}}
    }}
}

set dimensional_bar [ad_dimensional $dimensional]

# this is the table defn we'll feed to ad_table
set table_def [list]
lappend table_def [list  category "$category_name" {} \
        {<td align=center><a href="category?category_id=$category_id">$category</a></td>} ]
lappend table_def [list  active "Active" {} c ]
lappend table_def [list  suspended "Suspended" {} c ]
lappend table_def [list  canceled "Canceled" {} c ]
lappend table_def [list  finished "Closed" {} c ]
lappend table_def [list  total "Total" {} c ]

set sql "
    select *
      from (select tc.category_id, 
                   tc.name as category,
                   count(tt.ticket_id) as total,
                   sum(decode(lower(state),'active',1,0)) as active,
                   sum(decode(lower(state),'suspended',1,0)) as suspended,
                   sum(decode(lower(state),'canceled',1,0)) as canceled,
                   sum(decode(lower(state),'finished',1,0)) as finished
              from ttracker_categories tc, ttracker_tickets tt, wf_cases wfc
             where tc.package_id = :package_id and
                   tt.category_id = tc.category_id and
                   tt.ticket_id = wfc.object_id
             [ad_dimensional_sql $dimensional]
             group by tc.category_id, tc.name                          
             union all
            select tc1.category_id, 
                   tc1.name as category,
                   0 as total,
                   0 as active,
                   0 as suspended,
                   0 as canceled,
                   0 as finished
              from ttracker_categories tc1
             where tc1.package_id = :package_id
               and not exists (select tt1.ticket_id
                                 from ttracker_tickets tt1 
                                where tt1.category_id = tc1.category_id
                                [ad_dimensional_sql $dimensional]))
    [ad_order_by_from_sort_spec $orderby $table_def]    
"

set bind_ns_set [ad_tcl_vars_to_ns_set package_id]
set summary_table [ad_table -Torderby $orderby \
                            -Tmissing_text {<i>No tickets available.</i>} \
                            -Ttable_extra_html "width=100%" \
                            -bind $bind_ns_set \
                            tasks_select $sql $table_def]

ad_return_template "summary"

