# /packages/ticket-tracker/www/category.tcl

ad_page_contract {
    Lists all tickets in this category

    @author Phong Nguyen (phong@arsdigita.com)
    @author Tony Tseng (tony@arsdigita.com)
    @creation-date 2000-11-26
    @cvs-id $Id: category.tcl,v 1.3 2002/09/06 06:57:03 jeffd Exp $
} {
    category_id:integer,notnull    
    {orderby {ticket_id}}
    {order {}}
} -properties {
    page_title:onevalue
    context:onevalue
    name:onevalue
    description:onevalue
    category_id:onevalue
    assignee_name:onevalue
    category_name:onevalue
    dimensional_bar:onevalue
    ticket_table:onevalue   
}

set package_id [ad_conn package_id]

# get the category info
db_1row category_info_select {
    select name, 
           description, 
           nvl(acs_object.name(default_assignee), 'N/A') as assignee_name
      from ttracker_categories 
     where category_id = :category_id
}

set page_title $name
set context [list "$name"]
set category_name [ad_parameter CategoryName "ticket-tracker" "Category"]

# dimensional slider definition
set dimensional {
    {created "Creation Time" any {
        {1d "last 24hrs" {where "[db_map last_1_day]"}}
        {1w "last week" {where "[db_map last_7_days]"}}
        {1m "last month" {where "[db_map last_30_days]"}}
        {any "all" {}}
    }}
}

set dimensional_bar [ad_dimensional $dimensional]

# this is the table defn we'll feed to ad_table
set table_def [list]
lappend table_def [list ticket_id "Ticket ID" {} \
        {<td align=center><a href="ticket-view?ticket_id=$ticket_id">$ticket_id</a></td>} ]
lappend table_def [list pretty_priority "Priority" {priority $order} c ]
lappend table_def [list pretty_severity "Severity" {severity $order} c ]
lappend table_def [list state "Status" {} c ]
lappend table_def [list subject "Subject" {} c ]
lappend table_def [list creator_id "Submitted&nbsp;by" {upper(creator_name) $order} \
        {<td align=center><a href="/shared/community-member?user_id=$creator_id">$creator_name</a></td>} ]
lappend table_def [list created "Created" {creation_date $order} c ]

set sql "
    select ttracker_option.option_name(:package_id, 'severity', tt.severity) as pretty_severity,
           ttracker_option.option_name(:package_id, 'priority', tt.priority) as pretty_priority,
           tt.ticket_id,
           tt.subject,
           to_char(tt.creation_date, 'MM/DD/YYYY HH24:MI') as created,
           c.state,
           tt.creation_user as creator_id,
           acs_object.name(tt.creation_user) as creator_name
      from wf_cases c,
           ttracker_tickets tt
     where tt.category_id = :category_id
       and c.object_id = tt.ticket_id
           [ad_dimensional_sql $dimensional]
     [ad_order_by_from_sort_spec $orderby $table_def]
"

# render the table
set bind_ns_set [ad_tcl_vars_to_ns_set category_id  package_id]
set ticket_table [ad_table -Torderby $orderby \
                           -Tmissing_text "<i>No tickets available.</i>" \
                           -Ttable_extra_html "width=100%" \
                           -bind $bind_ns_set \
                           tasks_select $sql $table_def]

ad_return_template
