# /packages/ticket-tracker/www/index.tcl

ad_page_contract {
    Display task list, categories, and options to view tickets

    @author Tony Tseng (tony@arsdigita.com)
    @author Phong Nguyen (phong@arsdigita.com)
    @creation-date 2000-11-20
    @cvs-id $Id: index.tcl,v 1.4 2002/09/18 13:04:24 jeffd Exp $
} {
    {orderby {ticket_id}}
    {order {}}
} -properties {
    page_title:onevalue
    context:onevalue
    admin_p:onevalue
    category_name:onevalue
    category_name_lower:onevalue
    user_task_table:onevalue
    category_plural_name:onevalue
    category:multirow    
}

# authenticate user
set user_id [ad_conn user_id]

# check for admin priviledges
set package_id [ad_conn package_id]
set admin_p [ad_permission_p $package_id admin]

# template variables
set package_name [db_string package_name {select apm_package.name(:package_id) from dual}]
set page_title "$package_name"
set context {}

# this is the return_url we'll give to the task page under acs-workflow
set return_url [ad_conn url]

set category_name [ad_parameter CategoryName "ticket-tracker"  "Category"]
set category_name_lower [string tolower $category_name]
set category_plural_name [ad_parameter CategoryPluralName "ticket-tracker" "Categories"]
set description [ad_quotehtml [ad_parameter Description "ticket-tracker" ""]]

# checking if at least one category exists
# if not, redirects to the install directory
if { ![ttracker_category_exists_p] } {
    ad_returnredirect "admin/install/"
    set user_task_table ""
    set unassigned_task_table ""
    ad_script_abort
}

# table definition for user's outstanding tasks
# NOTE: [list]'s are used here instead of {}'s because ad_table 
#       does not evaluate $category_name
set table_def [list]
lappend table_def [list  ticket_id "Ticket ID" {} \
        {<td align=center><a href="ticket-view?ticket_id=$ticket_id">$ticket_id</a></td>} ]
lappend table_def [list  category "$category_name" {} c ]
lappend table_def [list  pretty_priority "Priority" {priority $order} c ]
lappend table_def [list  pretty_severity "Severity" {severity $order} c ]
lappend table_def [list  subject "Subject" {} c ]
lappend table_def [list  created "Created" {creation_date $order} c ]
lappend table_def [list  task_id "To do" {} \
        {<td align=center><a href="[ttracker_workflow_url]task?[export_vars -url -override \
	{{return_url "[ad_conn url]"}} {task_id return_url}]">$to_do</a></td>} ]

# get the user's outstanding tasks
set user_task_sql "
             select tt.ticket_id, 
                    tt.subject, 
                    ttracker_option.option_name(:package_id, 'severity', tt.severity) as pretty_severity,
                    ttracker_option.option_name(:package_id, 'priority', tt.priority) as pretty_priority,
                    tt.creation_date as created, 
                    ca.case_id, 
                    tc.name as category, 
                    ta.task_id,
                    ta.transition_key as to_do
               from ttracker_tickets tt, 
                    wf_cases ca, 
                    ttracker_categories tc,
                    wf_task_assignments w, 
                    wf_tasks ta, party_approved_member_map map
              where ca.object_id = tt.ticket_id
	        and ca.state = 'active'
                and tt.category_id = tc.category_id
                and tc.package_id = :package_id
                and w.party_id = map.party_id
                and map.member_id = :user_id 
                and ta.case_id = ca.case_id 
                and ta.task_id = w.task_id
                and (ta.state='enabled' or (ta.state='started' and ta.holding_user=:user_id))
               [ad_order_by_from_sort_spec $orderby $table_def]      
"
set bind_ns_set [ad_tcl_vars_to_ns_set package_id user_id]
set user_task_table [ad_table -Torderby $orderby \
                         -Tmissing_text "<i>No tasks available.</i>" \
                         -Ttable_extra_html "width=100%" \
                         -bind $bind_ns_set \
                         tasks_select $user_task_sql $table_def]

# get unassigned tasks for the admin to assign
set unassigned_task_sql "
    select tt.ticket_id,
           ttracker_option.option_name(:package_id, 'severity', tt.severity) as pretty_severity,
           ttracker_option.option_name(:package_id, 'priority', tt.priority) as pretty_priority,
           tt.subject,
           to_char(tt.creation_date, 'MM/DD/YYYY HH24:MI') as created,
           tc.name as category,
           ta.task_id
      from ttracker_tickets tt,
           ttracker_categories tc,
           wf_cases ca,
           wf_tasks ta
     where tc.package_id = :package_id
       and tt.category_id = tc.category_id
       and ca.object_id = tt.ticket_id
       and ca.state = 'active'
       and ta.case_id = ca.case_id 
       and not exists (select tasn.task_id 
                         from wf_task_assignments tasn
                         where tasn.task_id = ta.task_id)           
    [ad_order_by_from_sort_spec $orderby $table_def]
"

if { $admin_p } {
    # this is the table def for unassigned tasks
    # NOTE: [list]'s are used here instead of {}'s because ad_table 
    #       does not evaluate $category_name
    set table_def2 [list]
    lappend table_def2 [list ticket_id "Ticket ID" {} \
            {<td align=center><a href="ticket-view?ticket_id=$ticket_id">$ticket_id</a></td>} ]
    lappend table_def2 [list  category "$category_name" {} c ]
    lappend table_def2 [list  pretty_priority "Priority" {priority $order} c ]
    lappend table_def2 [list  pretty_severity "Severity" {severity $order} c ]
    lappend table_def2 [list  subject "Subject" {} c ]
    lappend table_def2 [list  created "Created" {creation_date $order} c ]
    lappend table_def2 [list  to_do "To do" no_sort \
        {<td align=center><a href="[ttracker_workflow_url]task?[export_vars -url -override \
	{{return_url "[ad_conn url]"}} {task_id return_url}]">assign</a></td>} ]

    set unassigned_task_table [ad_table -Torderby $orderby \
                               -Tmissing_text "<i>There are no unassigned tasks.</i>" \
                               -Ttable_extra_html "width=100%" \
                               -bind $bind_ns_set \
                               unassigned_tasks_select $unassigned_task_sql $table_def2]
}

# get categories
db_multirow category categories_select {
    select category_id, name 
    from ttracker_categories
    where package_id = :package_id
    order by upper(name)
}

ad_return_template

