# /packages/ticket-tracker/www/ticket-view.tcl

ad_page_contract {
    Detailed view of a ticket

    @author Phong Nguyen (phong@arsdigita.com)
    @author Tony Tseng   (tony@arsdigita.com)
    @creation-date 2000-11-28
    @cvs-id $Id: ticket-view.tcl,v 1.5 2005/03/01 00:01:16 jeffd Exp $
} {
    ticket_id:trim,integer,ttracker_ticket_id
    {orderby {task_id*}}
    {order {}}
} -properties {
    context:onevalue
    option_bar:onevalue
    category_name:onevalue
    progress_table:onevalue
    ticket:onerow
    assignee:multirow
}
# The trim filter is used on ticket_id since the ticket_id may
# be entered manually from the index page.
 
set user_id [ad_conn user_id]
set package_id [ad_conn package_id]
set category_name [ad_parameter CategoryName "ticket-tracker" "Category"]
set context [list "Ticket #$ticket_id"]

# get the comments from general comments
set option_bar [general_comments_create_link -object_name "Ticket #$ticket_id" \
        "$ticket_id"  "[ad_conn package_url]ticket-view?ticket_id=$ticket_id"]

# get permission info
# instead of calling ad_permission_p twice and query db twice, use one query instead
db_1row get_permission {
    select decode(acs_permission.permission_p(:ticket_id, :user_id, 'write'), 't', 1, 0) as write_p,
           decode(acs_permission.permission_p(:ticket_id, :user_id, 'admin'), 't', 1, 0) as admin_p 
      from dual
}

# if user has write privilege, allow them edit the ticket
if { $write_p } {
    append option_bar " | <a href=ticket-edit?ticket_id=$ticket_id>Edit</a>"
}

# gather the ticket info
db_1row ticket_info_select {
    select tt.subject,
           cr.content as description,
           cr.mime_type,
           ttracker_option.option_name(:package_id, 'severity', tt.severity) as pretty_severity,
           ttracker_option.option_name(:package_id, 'priority', tt.priority) as pretty_priority,
           to_char(tt.creation_date, 'MM/DD/YYYY HH24:MI') as created,
           tc.name as category,
           tc.category_id,
           acs_object.name(tt.creation_user) as submitted_by,
           wc.state, 
           wc.case_id    
      from ttracker_tickets tt,
           ttracker_categories tc,
           wf_cases wc,
           cr_revisions cr
     where tt.ticket_id = :ticket_id 
       and tc.category_id = tt.category_id
       and wc.object_id = :ticket_id
       and cr.revision_id = content_item.get_live_revision(tt.ticket_id)
} -column_array {ticket}

# if it's plain text, escape the html stuff
if [string equal $ticket(mime_type) "text/plain"] {
    set ticket(description) [ad_quotehtml $ticket(description)]
}

# if user is an admin, display the action stuff
if { $admin_p } {
    if { [string equal $ticket(state) "suspended"] || [string equal $ticket(state) "canceled"] } {
        append option_bar " | <a href=state-change?ticket_id=$ticket_id&action=resume>Resume</a>"
    } 
    if [string equal $ticket(state) "active"] {
        append option_bar " | <a href=state-change?ticket_id=$ticket_id&action=suspend>Suspend</a>"
    }
    if { [string equal $ticket(state) "active"] || [string equal $ticket(state) "suspended"] } {
        append option_bar " | <a href=state-change?ticket_id=$ticket_id&action=cancel>Cancel</a>"
    }
}       


# get all the assignees
db_multirow assignee assignees_select {
    select acs_object.name(wta.party_id) as name
      from wf_task_assignments wta
     where wta.task_id = (select max(wt.task_id) 
                            from wf_tasks wt,
                                 wf_cases wc
                           where wc.object_id = :ticket_id
                             and wt.case_id = wc.case_id
                             and wt.transition_key = 'resolve')
}

# get the comments from general comments
set comments [general_comments_get_comments "$ticket_id" "[ad_conn package_url]ticket-view?ticket_id=$ticket_id"]

ad_return_template

