# /packages/ticket-tracker/www/tickets.tcl

ad_page_contract {
    Displays a list of tickets

    @author Tony Tseng (tony@arsdigita.com)
    @author Phong Nguyen (phong@arsdigita.com)
    @creation-date 11/22/2000
    @cvs-id $Id: tickets.tcl,v 1.6 2005/03/01 00:01:16 jeffd Exp $
} { 
    {orderby {ticket_id}}
    {view {}}
    {assign {me}}
} -properties {
    page_title:onevalue
    context:onevalue
    view:onevalue
    dimensional_bar:onevalue
    ticket_table:onevalue
    tickets:multirow
    category_name:onevalue
}

# template variables
set package_id [ad_conn package_id]
set user_id [ad_conn user_id]
set category_name [ad_parameter CategoryName "ticket-tracker" "Category"]
set page_title "Tickets"
set context {"Tickets"}

# dimensional slider definition
set dimensional {
    {submitby "Submitted by" any {
        {me "me" {where "[db_map creation_user]"}}
        {any "anyone" {}}
    }}
    {assign "Assigned to" me {
        {me "me" {}}
        {any "anyone" {}}
        {noass "unassigned" {}}
    }}      
    {state "Status" any {
        {created "created" {where "[db_map status_created]"}}
        {active "active" {where "[db_map status_active]"}}
        {suspended "suspended" {where "[db_map status_suspended]"}}
        {canceled "canceled" {where "[db_map status_canceled]"}}
        {finished "finished" {where "[db_map status_finished]"}}
        {any "all" {}}
    }}
    {created "Creation Time" any {
        {1d "last 24hrs" {where "[db_map last_1_day]"}}
        {1w "last week" {where "[db_map last_7_days]"}}
        {1m "last month" {where "[db_map last_30_days]"}}
        {any "all" {}}
    }}
}

# dynamically generating the dimensional list for priority options
set priority_slider [list priority "Priority" any]
set priority_list [list]
db_foreach get_priorities {
    select name,
           value
      from ttracker_options
     where package_id = :package_id
       and function = 'priority'
} {
    lappend priority_list [list $value $name "where \"tt.priority = $value\""]
}
lappend priority_list [list any all [list]]
lappend priority_slider $priority_list

# dynamically generating the dimensional list for severity options
set severity_slider [list severity "Severity" any]
set severity_list [list]
db_foreach get_severities {
    select name,
           value
      from ttracker_options
     where package_id = :package_id
       and function = 'severity'
} {
    lappend severity_list [list $value $name "where \"tt.severity = $value\""]
}
lappend severity_list [list any all [list]]
lappend severity_slider $severity_list 

# adding the priority and severity options to the opion bar
lappend dimensional $priority_slider $severity_slider

set dimensional_bar [ad_dimensional $dimensional]

# ad_table definition
# NOTE: 1) [list]'s are used here instead of {}'s because ad_table 
#          does not evaluate $category_name
#       2) The link generated for submitted_by does not work if
#          it references a group. An SDM ticket has been created
#          requesting this feature.
set table_def [list]
lappend table_def [list line_number "Num" {} {<td align=right>$Tcount</td>} ]
lappend table_def [list ticket_id "ID#" {} \
        {<td align=center><a href="ticket-view?ticket_id=$ticket_id">$ticket_id</a></td>} ]
lappend table_def [list category "$category_name" {} c ]
lappend table_def [list pretty_priority "Priority" {priority $order} c ]
lappend table_def [list pretty_severity "Severity" {severity $order} c ]
lappend table_def [list state "Status" {} c ]
lappend table_def [list subject "Subject" {} c ]
# vinodk: kludge - since you can't order by alias names in postgres
#         have to order by item #
lappend table_def [list creator_id "Submitted by" {upper(11) $order} \
        {<td align=center><a href="/shared/community-member?user_id=$creator_id">$creator_name</a></td>} ]
# vinodk: kludge again
lappend table_def [list assignee_id "Assigned to" {upper(14) $order} \
        {<td align=center><a href="/shared/community-member?user_id=$assignee_id">$assignee_name</a></td>} ]
lappend table_def [list created "Created" {created $order} c ]


# sql to retrieve selected tickets
set view_select ""
set view_from ""
set view_where ""
# only getting the content and mime type if it's a full view
if [string equal $view "full"] {
    set view_select ", cr.mime_type, cr.content"
    set view_from ", cr_revisions cr "
    set view_where "[db_map live_revision]"
}


# this is the ugly part
# 3 different queries for 3 different assignee types

# openacs change - vinodk
# old code conditionally assigned huge query to var 'sql'
# and then ran the query sql (after the if statement)
# 
# I've moved those queries to .xql files with different statement names
# I then conditionally assignthe var 'statement_name' to the 
# appropriate statement name from the .xql file

set sql ""
set dimensional [lreplace $dimensional 1 1]
if {[string equal $assign "me"]} {
	set statement_name ticket_select_me

    # this query is specifically for tickets assigned to the user 
    # the "exists" clause asks if there's any 'resolved' tasks assigned to the user
} elseif {[string equal $assign "noass"]} {
	set statement_name ticket_select_noass

    # the "not exists" clause specifies that there must be no assignees
    # assigned to the "resolve" task for this case
    # the reason to use "max" is that we want to find the most recent task
    # there might be many tasks associated with a case
} else {
	set statement_name ticket_select_all

    # the first query is for tickets with assignees
    # the second one is for tickets with no assignees
    # the reason to use "union all" is for performance issue
    # tried to compress them into one query using outer join 
	# with a view, but it's too slow
}

# ad_table manipulating code
set remove_column {}
set rowcode {}
switch [ad_sort_primary_key $orderby] {
    creator_id {
        lappend remove_column submitby
        set rowcode {
            [if {![ad_table_same creator_id]} {
                ad_table_span [subst {Submitted by: <a href="/shared/community-member?user_id=$creator_id">$creator_name</a>}] {bgcolor=cccccc}
            }]
			}
    }
    assignee_id {
        lappend remove_column assignee_id
        set rowcode {
            [if {![ad_table_same assignee_id]} {
                ad_table_span [switch $assignee_id {
                    "" {subst {Assigned to: <font color=red>unassigned</font>}}
                    default {subst {Assigned to: <a href="/shared/community-member?user_id=$assignee_id">$assignee_name</a>}}
                }] {bgcolor=cccccc}
            }]
        }
    }
    category {
        lappend remove_column category
        set rowcode {
            [if {![ad_table_same category]} {
                ad_table_span [subst {<strong>$category</strong>}] {bgcolor=cccccc}
            }]
        }
    }
}

# set which columns we really want to be displayed
set all_columns {line_number ticket_id category pretty_priority pretty_severity \
		state subject creator_id assignee_id created}
set col [list]
foreach acol $all_columns { 
    if {[lsearch -exact $remove_column $acol] < 0} { 
        lappend col $acol
    }
}

# create the table to display the tickets 
if { [empty_string_p $view] } {
    set bind_ns_set [ad_tcl_vars_to_ns_set package_id user_id]
    # this is ugly, ad_table should have some option of turning off
    # the audit if the primary sort key is the column that changes
    if { [ad_sort_primary_key $orderby] == "assignee_id" } {
        set ticket_table [ad_table -Torderby $orderby \
                                   -Tpre_row_code $rowcode \
                                   -Tcolumns $col \
                                   -Tmissing_text {<i>No tickets available.</i>} \
                                   -Ttable_extra_html "width=100%" \
                                   -bind $bind_ns_set \
                                   $statement_name $sql $table_def]
    } else {
        set ticket_table [ad_table -Torderby $orderby \
                                   -Taudit {ticket_id} \
                                   -Tpre_row_code $rowcode \
                                   -Tcolumns $col \
                                   -Tmissing_text {<i>No tickets available.</i>} \
                                   -Ttable_extra_html "width=100%" \
                                   -bind $bind_ns_set \
                                   $statement_name $sql $table_def]
    }
}

# creating the data source for report view
if { ![empty_string_p $view] } {
    set old_ticket_id ""
    set assignees ""
    set i 1
    # create a custom datasource for easier processing in the template page
    template::multirow create tickets ticket_id pretty_severity pretty_priority \
			subject created category_id  category creator_id creator_name state \
			assignee_id assignee_name content assignee_list comments

    db_foreach $statement_name $sql {
        # add a row in the datasource for new tickets
        # if we sorted by assignee_id, then always create a new row 
        if { $old_ticket_id != $ticket_id || [ad_sort_primary_key $orderby] == "assignee_id"} {
            if { ![empty_string_p $old_ticket_id] } {
                # we have already transitioned into another ticket
                # set the accumulated assignee names into the old row
                # before we start adding the new row
                template::multirow set tickets $i assignee_list $assignees
                incr i
            }	    

			template::multirow append tickets $ticket_id $pretty_severity \
					$pretty_priority $subject $created $category_id $category \
					$creator_id $creator_name $state $assignee_id $assignee_name

			# quote the content if it's plain text
			if [string equal $view "full"] {
				if [string equal $mime_type "text/plain"] {
					template::multirow set tickets $i content \
							[ad_quotehtml $content] 
				} else {
					template::multirow set tickets $i content $content
				}

				# full view also shows comments
				template::multirow set tickets $i comments \
						[general_comments_get_comments -print_content_p 1 $ticket_id ""]
			}
		    
            if { ![empty_string_p $assignee_name] } {
                set assignees "<strong><a href=\"/shared/community-member?user_id=$assignee_id\">$assignee_name</a></strong>"
            } else {
                set assignees ""
            }
         
			# accumulate the assignee_name otherwise     
        } else {
            append assignees ", <strong><a href=\"/shared/community-member?user_id=$assignee_id\">$assignee_name</a></strong>"
        }
        set old_ticket_id $ticket_id
    }
    template::multirow set tickets $i assignee_list $assignees
}

set view_print_bar "
View/Print: 
<a href=\"tickets?view=full&[export_ns_set_vars]\">Full Report</a> | 
<a href=\"tickets?view=summary&[export_ns_set_vars]\">Summary Report (no comments/descriptions)</a>
"

ad_return_template
