# /packages/ticket-tracker/www/user-assign-summary.tcl

ad_page_contract {
    Displays a summary of tickets by category

    @author Phong Nguyen (phong@arsdigita.com)
    @author Tony Tseng (tony@arsdigita.com)
    @creation-date 2000-11-20
    @cvs-id $Id: user-assign-summary.tcl,v 1.4 2002/09/06 06:57:03 jeffd Exp $
} {
    {orderby {party_id}}
    {order {}}
} -properties {
    context:onevalue
    summary_table:onevalue
} 

set page_title "Ticket Assignment Summaries"
set context { "Ticket Assignment Summaries" }
set category_name [ad_parameter CategoryName "ticket-tracker" "Category"]
set package_id [ad_conn package_id]

# dimensional slider definition
set dimensional {
    {posted "In the last" any {
        {year "year" {where "[db_map posted_last_year]"}}
        {3month "3 months" {where "[db_map posted_last_3mos]"}}
        {month "month" {where "[db_map posted_last_month]"}}
        {any "all" {}}
    }}
}

set dimensional_bar [ad_dimensional $dimensional]

# this is the table defn we'll feed to ad_table
set table_def [list]
lappend table_def [list party_name "Party" {upper(2) $order} \
        {<td align=center><a href="/shared/community-member?user_id=$party_id">$party_name</a></td>} ]
lappend table_def [list total "Total" {} c ]
lappend table_def [list active "Active" {} c ]
lappend table_def [list suspended "Suspended" {} c ]
lappend table_def [list canceled "Canceled" {} c ]
lappend table_def [list finished "Closed" {} c ]
lappend table_def [list latest "Latest" {} c ]
lappend table_def [list oldest "Oldest" {} c ]

set sql " 
    select wta.party_id, 
           acs_object.name(wta.party_id) as party_name,
           count(ticket_id) as total,
           sum(decode(lower(wc.state),'active',1,0)) as active,
           sum(decode(lower(wc.state),'suspended',1,0)) as suspended,
           sum(decode(lower(wc.state),'canceled',1,0)) as canceled,
           sum(decode(lower(wc.state),'finished',1,0)) as finished,
           max(tt.creation_date) as latest,
           min(tt.creation_date) as oldest
      from ttracker_tickets tt, 
           ttracker_categories tc, 
           wf_cases wc,
           wf_task_assignments wta
     where tc.package_id = :package_id
       and tt.category_id = tc.category_id
       and tt.ticket_id = wc.object_id
       and wta.task_id = (select max(wt.task_id) 
                                     from wf_tasks wt
                                    where wt.case_id = wc.case_id
                                      and wt.transition_key = 'resolve')
     [ad_dimensional_sql $dimensional] 
     group by wta.party_id
     [ad_order_by_from_sort_spec $orderby $table_def]     
"

set bind_ns_set [ad_tcl_vars_to_ns_set package_id]
set summary_table [ad_table -Torderby $orderby \
                            -Tmissing_text {<i>No tickets available.</i>} \
                            -Ttable_extra_html "width=100%" \
                            -bind $bind_ns_set \
                            tasks_select $sql $table_def]


ad_return_template "summary"
