ad_library {
    procedures used by the WAP module.

    @author Andrew Grumet (aegrumet@arsdigita.com) 
    @author Shan Shan Huang (shuang@arsdigita.com)
    @creation-date November 28, 2000
    @cvs-id $Id: wap-procs.tcl,v 1.5 2009/02/13 20:40:39 jeffd Exp $
}

ad_proc wap_user_agent_p {} "Returns 1 if User-Agent matches a list of known WAP user agents.  Returns 0 if no header is supplied." {
    # Get the user agent for the request.
    set agent [ns_set iget [ad_conn headers] User-Agent]

    if [empty_string_p $agent] {
	return 0
    }

    # Take the db hit in a memoize
    return [util_memoize "wap_db_user_agent_p {$agent}" [ad_parameter WapUADbRefresh wap 3600]]
}

ad_proc wap_db_user_agent_p {agent} "Returns 1 if agent matches a list of known WAP user agents allowed." {
    set compare_length [ad_parameter WapUAStringCompareLength wap 4]

    set wapUA_p [db_string "GetWAPAgentCount" "select count(*) as wapUA_p
    from wap_user_agents
    where lower(substr('[DoubleApos $agent]',1,$compare_length)) like
    lower(substr(name,1,$compare_length)) || '%'
    and active_p = 't'"]

    ns_log notice "wapUA_p: $wapUA_p"
    if { $wapUA_p > 0} {
	return 1
    } else {
	return 0
    }
}

ad_proc wap_add_user_agent { name creation_comment active_p } {
    tcl wrapper for plsql function to add a WAP user agent.
} {
    set user_id [ad_conn user_id]
    set ip [ad_conn peeraddr]
    
    db_dml "add_user_agent" "
    declare 
      new_id integer;
    begin
      new_id := wap_user_agent.new(null, '[DoubleApos $name]',
      '[DoubleApos $creation_comment]', '$active_p', $user_id, '[DoubleApos $ip]');
    end;"

    return
}

ad_proc wap_modify_user_agent { user_agent_id name creation_comment active_p } {
    tcl wrapper for plsql function that modifies a WAP user agent.
} {
    db_dml "modify_user_agent" "begin 
    wap_user_agent.modify($user_agent_id, '[DoubleApos $name]', '[DoubleApos $creation_comment]', '$active_p');
    end; "

    return
}

ad_proc wap_delete_user_agent { user_agent_id } {
    tcl wrapper for deleting WAP user agent
} {
    db_dml "delete_user_agent" "begin
    wap_user_agent.del($user_agent_id); 
    end;"

    return
}

ad_proc wap_header {{no_cache_p "f"}} "Returns headers for WAP devices.  Return no cache hearders if no_cache_p is t. " {
    if { $no_cache_p == "t" } {
	set headders [ad_conn outputheaders]
	ns_set put $headers "Cache-Control" "no-cache, must-revalidate"
	ns_set put $headers "Pragma" "no-cache"
    }
    ReturnHeaders text/vnd.wap.wml
    ns_write {<?xml version="1.0"?> 
    <!DOCTYPE wml PUBLIC "-//WAPFORUM//DTD WML 1.1//EN" "http://www.wapforum.org/DTD/wml_1.1.xml">
}
}

ad_proc wml_return {
    {
	-no_cache_p f
    } page_wml
} { Returns a page to the connection, optionally writing out as HTML for
debugging if the debug flag is set to 1.} {

    set wml_debug_p [ad_parameter WapDebugMode wap 0]

    if !$wml_debug_p {
	wap_header $no_cache_p
	ns_write "$page_wml"
    } else {
	ReturnHeaders
	ns_write "<html><pre>"
	ns_write [ns_quotehtml $page_wml]
	ns_write "</pre></html>"
    }
}

ad_proc wml_logout_link {} {
    Generate a logout link for a wml browser.
} {
    set return_wml "<do type=\"accept\" label=\"Logout\">
    <go href=\"/register/logout\"/>
    </do>"

    return $return_wml
}

ad_proc export_wml_form_vars args {
    Generate wml segments that export these form variables to the next 
    card/url.
} {
    set return_wml ""
    foreach var_spec $args {
	set var_spec_pieces [split $var_spec ":"]
	set var [lindex $var_spec_pieces 0]
	set type [lindex $var_spec_pieces 1]
	upvar 1 $var value
	if { [info exists value] } {
	    switch $type {
		multiple {
		    foreach item $value {
			append return_wml "<postfield name=\"$var\" value=\"[util_quote_double_quotes $item]\"/>"
		    }
		}
		default {
		    append return_wml "<postfield name=\"$var\" value=\"[util_quote_double_quotes $value]\"/>"
		}
	    }
	}
    }

    return $return_wml
}




